﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.Mvc;
using IntelSkill.Models;
using IntelSkill.Repository;
using IntelSkill.DAL;
using IntelSkill.Utilities; 

namespace IntelSkill.Controllers
{
    public class AccessController : Controller
    {
        private IClassRepository classRepository;
        private IUsersRepository usersRepository;

        private AccessModel accessAccount = null;

        private int _accessAttemptLimit = 3;

        //Constructor
        public AccessController()
        {
            this.classRepository = new ClassRepository(new SQLDBContext());
            this.usersRepository = new UsersRepository(new SQLDBContext());
        }

        // GET: /Access/

        public ActionResult Index()
        {
            if (Request.QueryString["accessRequest"] != null)
            {
                ViewBag.AccessRequestCode = Request.QueryString["accessRequest"].ToString();
            }
            else { return RedirectToAction("SignIn", "Account"); }

            return View();
        }

        #region  Access Login

        public ActionResult Role()
        {
            ActionResult newActionResult = CheckAccessAndRedirect();

            return newActionResult;
        }

        private ActionResult CheckAccessAndRedirect()
        {
            if (Request.QueryString["publisherId"] != null && Request.QueryString["roleId"] != null)
            {
                int publisherId = 0, currentUserId = 0, roleId = 0;

                Int32.TryParse(Request.QueryString["publisherId"].ToString(), out publisherId);
                Int32.TryParse(Request.QueryString["roleId"].ToString(), out roleId);

                if (Session["AccessModel"] != null)
                {
                    accessAccount = (AccessModel)Session["AccessModel"];
                    currentUserId = accessAccount.CurrentUserId;
                }

                AccessModel model = usersRepository.GetRoleAccessDetails(publisherId, currentUserId, roleId);

                if (model != null)
                {
                    if (model.PublisherUserId > 0 && model.RoleId > 0)
                    {
                        model.PublisherCompanyLogo = (!string.IsNullOrEmpty(model.PublisherCompanyLogo) ? model.PublisherCompanyLogo : string.Empty);

                        Session["AccessModel"] = model;

                        accessAccount = model;

                    }
                }



            }

            return RedirectToAction("Dashboard", "Publisher");
        }
        #endregion

        [HttpPost]
        public ActionResult Index(FormCollection collection)
        {
            string requestCode = string.Empty;

            if (Request.QueryString["accessRequest"] != null)
            {
                requestCode = Request.QueryString["accessRequest"].ToString();
                ViewBag.AccessRequestCode = Request.QueryString["accessRequest"].ToString();

                if (collection.GetValues("AccessCodePwd") != null)
                {
                    string accessPwd = collection.GetValues("AccessCodePwd")[0];

                    if (ValidatAccesseLinkAndSignIn(requestCode, accessPwd))
                    {
                        return RedirectToAction("BuildClass", "Publisher", new { id = accessAccount.ClassId });
                    }
                    else { ViewBag.PageMessage = "Invalid Access Code/ Password."; ViewBag.PageMessageType = "error"; }
                }
                else { ViewBag.PageMessage = "Please enter access code."; ViewBag.PageMessageType = "error"; }
            }
            else { return RedirectToAction("SignIn", "Account"); }


            return View();
        }

        private bool ValidatAccesseLinkAndSignIn(string requestCode, string accessPwd)
        {
            AccessModel model = classRepository.GetAccessDetails(requestCode, accessPwd, _accessAttemptLimit);

            if (model != null)
            {
                if (model.CurrentUserId > 0)
                {
                    model.PublisherCompanyLogo = (!string.IsNullOrEmpty(model.PublisherCompanyLogo) ? "/Uploads/CompanyLogoFolder/" + model.PublisherCompanyLogo : "");
                    Session["AccessModel"] = model;
                    accessAccount = model;
                    return true;
                }
            }

            return false;
        }

    }
}
