﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.Mvc;
using System.IO;
using System.Web.Security;
//IntelSkill Libraries
using IntelSkill.Models;
using IntelSkill.Repository;
using IntelSkill.DAL;
using IntelSkill.Utilities;
using System.Net.Mail;
using System.Net;



namespace IntelSkill.Controllers
{
    public class AccountController : Controller
    {
        private StringCryptography stringCryptography;
        private IUsersRepository usersRepository;

        private AccessModel accessAccount = null;
        static string mainFolder = GlobalSettings.UploadFolder;
        private string pagemessage = string.Empty;
        private string pagemessagetype = string.Empty;

        public AccountController()
        {
            this.usersRepository = new UsersRepository(new SQLDBContext());
        }

        #region Account Views

        // GET:
        public ActionResult Registration()
        {
            ViewBag.Title = "IntelSkill - Registration";

            var model = new UserModel();

            DefaultRegistrationDataLoad(model);

            return View(model);
        }

        public ActionResult SignIn()
        {
            ViewBag.Title = "IntelSkill - Login";

            var model = new SignInModel();

            GetSignInCookies(ref model);

            return View(model);
        }

        public ActionResult MyProfile()
        {
            ActionResult loginActionResult = UserAuthentication();

            if (accessAccount.CurrentUserId == 0)
                return loginActionResult;

            var model = new UserModel();


            UserDataLoad(accessAccount.CurrentUserId, ref model);


            DefaultUserDataLoad(model);

            model.PageRightsList = GetPageRights();

            SetMenuProperties("", "Account Summary", "View Profile");


            ViewBag.IsEditProfile = false;

            return View(model);
        }

        public ActionResult ChangePassword()
        {
            ActionResult loginActionResult = UserAuthentication();

            if (accessAccount.CurrentUserId == 0)
                return loginActionResult;

            var model = new ChangePasswordModel();

            SetMenuProperties("", "Account Summary", "Change Password");

            return View(model);
        }

        public ActionResult ForgotPassword()
        {
            var model = new ForgotPasswordModel();
            return View(model);
        }

        public ActionResult ResetPassword(string token)
        {

            var model = new ForgotPasswordModel();
            model.UserForm = new Users();
            model.IsForgotSessionExpired = false;
            stringCryptography = new StringCryptography();
            string token1 = stringCryptography.Decryptdata(Convert.ToString(token));
            model.UserForm.UserId = (Convert.ToInt32(token1));
            Users usrs = usersRepository.GetUserByID(model.UserForm.UserId);
            DateTime TokenRequestDate = Convert.ToDateTime(usrs.ForgotPassTokenReq);
            if (TokenRequestDate.AddMinutes(15) < DateTime.Now)
            {
                model.IsForgotSessionExpired = true;

            }
            return View(model);
        }

        // POST:
        [HttpPost]
        public ActionResult Registration(UserModel model)
        {
            ViewBag.Title = "IntelSkill - Registration";

            ActionResult actionResult = SaveRegistration(model);

            if (actionResult != null)
                return actionResult;

            model.ValidationMessage = pagemessage;
            model.ValidationMessageType = pagemessagetype;

            return View("Registration", model);
        }

        [HttpPost]
        public ActionResult SignIn(SignInModel model)
        {
            ViewBag.Title = "IntelSkill - Login";

            ActionResult actionResult = SignInNow(model);

            if (actionResult != null)
                return actionResult;

            model.ValidationMessage = pagemessage;
            model.ValidationMessageType = pagemessagetype;

            return View("signin", model);
        }

        [HttpPost]
        public ActionResult MyProfile(UserModel model, FormCollection collection, string btnsave = "")
        {
            ActionResult actionResult = UserAuthentication();

            if (accessAccount.CurrentUserId == 0)
                return actionResult;


            if (btnsave.ToLower() == "save")
            {

                ViewBag.IsSaveProfile = true;
                actionResult = SaveProfile(model);
            }
            else if (btnsave.ToLower() == "edit")
            {
                EditProfileDataLoad(ref model);
            }

            DefaultUserDataLoad(model);
            SetMenuProperties("", "Account Summary", "Edit Profile");


            model.ValidationMessage = pagemessage;
            model.ValidationMessageType = pagemessagetype;

            if (actionResult == null)
                return View(model);
            else
                return actionResult;
        }

        [HttpPost]
        public ActionResult ChangePassword(ChangePasswordModel model)
        {
            ActionResult actionResult = UserAuthentication();

            if (accessAccount.CurrentUserId == 0)
                return actionResult;

            SaveChangePassword(ref model);

            SetMenuProperties("", "Account Summary", "Change Password");

            model.ValidationMessage = pagemessage;
            model.ValidationMessageType = pagemessagetype;

            return View(model);
        }

        [HttpPost]
        public ActionResult ForgotPassword(ForgotPasswordModel model)
        {
            model.UserForm = usersRepository.GetUserbyEmail(model.Email);
            if (model.UserForm != null)
            {

                model.UserForm.ForgotPassTokenReq = DateTime.Now;
                usersRepository.UpdateUser(model.UserForm);
                usersRepository.Save();
                stringCryptography = new StringCryptography();
                string token = stringCryptography.Encryptdata(Convert.ToString(model.UserForm.UserId));
                string port = Request.Url.Host.Contains("localhost") ? ":" + Request.Url.Port.ToString() + "/" : string.Empty;
               // string hostUrl = Request.Url.Host + port;
                string hostUrl = (Request.Url.Host + port).EndsWith("/") ? Request.Url.Host + port : Request.Url.Host + "/";
                //string resetUrl = hostUrl + VirtualPathUtility.ToAbsolute("~/Account/ResetPassword?token=" + token);
                MailMessage msg = new MailMessage("smtpintelskill@sarvajithtechpark.com", model.UserForm.Email);
                msg.Subject = "Password reset request";
                msg.Body = @"<br/> <br/>You have been requested for forgotpassword <br/> <br/> 
                
                                    Please <a href='" + hostUrl + "Account/ResetPassword?token=" + token + @"'>click here</a> to access forgotpassword details.<br/><br/> 
                
                                    If any problem to view the link please copy & paste the below url to continue.<br/><br/>
                                    " + hostUrl + "Account/ResetPassword?token=" + token + @"<br/><br/> ";
                msg.IsBodyHtml = true;
                var smtp = new SmtpClient();
                smtp.Send(msg);
                model.IsForgotSuccess = true;

            }
            else
            {
                model.ValidationMessage = "No Account Found with that Email";
                model.ValidationMessageType = "error"; ;
            }
            return View(model);
        }

        [HttpPost]
        public ActionResult ResetPassword(ForgotPasswordModel model, string btnsubmit)
        {
            int id = model.UserForm.UserId;

            if (model.UserForm.Password.Length > 5)
            {
                if (model.UserForm.Password == model.ConfirmPassword)
                {

                    Users users = usersRepository.GetUserByID(id);
                    stringCryptography = new StringCryptography();
                    users.Password = stringCryptography.GetMD5HashData(model.UserForm.Password);
                    usersRepository.UpdateUser(users);
                    usersRepository.Save();
                    model.IsForgotSuccess = true;

                }
                else
                {
                    model.ValidationMessage = "Password not matched";
                    model.ValidationMessageType = "error";

                }
            }
            else
            {
                model.ValidationMessage = "Password At Least 6 Characters";
                model.ValidationMessageType = "error";
            }
            return View(model);

        }

        // FUNCTIONS: 

        private void SaveChangePassword(ref ChangePasswordModel model)
        {
            if (ValidateChangePassword(model))
            {
                stringCryptography = new StringCryptography();

                model.CurrentUserId = accessAccount.CurrentUserId;
                model.PublisherUserId = accessAccount.PublisherUserId;

                model.OldPassword = stringCryptography.GetMD5HashData(model.OldPassword);
                model.NewPassword = stringCryptography.GetMD5HashData(model.NewPassword);

                if (usersRepository.ChangePassword(model))
                {
                    pagemessagetype = "success";
                    pagemessage = "Password has been changed";
                    model = new ChangePasswordModel();

                }
                else
                {
                    pagemessagetype = "error";
                    pagemessage = "Invalid old password.";
                }

            }
            else { pagemessagetype = "error"; }

            SetMenuProperties("", "Account Summary", "Change Password");

            model.ValidationMessage = pagemessage;
            model.ValidationMessageType = pagemessagetype;


        }

        private void EditProfileDataLoad(ref UserModel model)
        {
            ViewBag.IsEditProfile = true;

            UserDataLoad(accessAccount.CurrentUserId, ref model);


        }

        private string SaveNewLogo()
        {
            if (Request.Files.Count > 0)
            {
                if (Request.Files["ModifyLogo"].ContentLength > 0)
                {
                    string localfolder = Server.MapPath(@"\Uploads\CompanyLogoFolder");
                    string CompanyLogoFolder = localfolder;// mainFolder + @"\CompanyLogoFolder";

                    if (CompanyLogoFolder.EndsWith(@"\") == false)
                    {
                        CompanyLogoFolder += @"\";
                    }

                    //if (Directory.Exists(CompanyLogoFolder) == false)
                    //{
                    //    Directory.CreateDirectory(CompanyLogoFolder);
                    //}

                    //string subDir = CompanyLogoFolder + DateTime.Now.ToString("yyyy") + @"\" + DateTime.Now.ToString("MM") + @"\" + DateTime.Now.ToString("dd") + @"\";

                    //if (Directory.Exists(subDir) == false)
                    //{
                    //    Directory.CreateDirectory(subDir);
                    //}

                    string file_extension = Path.GetExtension(Request.Files["ModifyLogo"].FileName);

                    string randomFileName = Guid.NewGuid().ToString();

                    string absolute_path = localfolder+@"\" + randomFileName + file_extension;

                    Request.Files["ModifyLogo"].SaveAs(absolute_path);

                    return randomFileName + file_extension;
                }
            }

            return string.Empty;
        }

        private ActionResult SaveProfile(UserModel model)
        {
            if (ValidateProfile(model))
            {
                Users userInfo = usersRepository.GetMyProfileInfo(accessAccount.CurrentUserId);
                userInfo.FirstName = model.User.FirstName;
                userInfo.LastName = model.User.LastName;
                userInfo.ModifiedDate = System.DateTime.Now;
                userInfo.ModifiedUserId = accessAccount.CurrentUserId;
                userInfo.CompanyName = model.User.CompanyName;

                string guid_filename = SaveNewLogo();

                if (guid_filename.Length > 0)
                    userInfo.CompanyLogo = guid_filename;



                if (usersRepository.MyProfileUpdate(userInfo))
                {
                    pagemessage = "Profile has been saved.";
                    pagemessagetype = "success";

                    UserListModel accessAccountDetail = usersRepository.GetUserDetail(userInfo.Email, userInfo.Password);

                    if (accessAccountDetail != null)
                    {
                        AccessModel accessModel = new AccessModel();

                        accessModel.DisplayName = accessAccountDetail.Email;
                        accessModel.CurrentUserId = accessAccountDetail.UserId;
                        accessModel.PublisherCompanyLogo = accessAccountDetail.CompanyLogo;
                        accessModel.PublisherUserId = accessAccountDetail.UserId;

                        Session["AccessModel"] = accessModel;
                    }

                    return RedirectToAction("MyProfile", "Account");
                }
                else
                {
                    pagemessage = "Profile saving failed. try again later.";
                    pagemessagetype = "error";
                }

            }
            else { pagemessagetype = "error"; }

            return null;
        }

        private ActionResult SaveRegistration(UserModel model)
        {
            if (ValidateRegistrationForm(model))
            {
                stringCryptography = new StringCryptography();

                model.User.UserTypeId = Convert.ToInt32(UserTypes.StandardUser);
                model.User.CreatedDate = System.DateTime.Now;
                model.User.UserStatus = true;
                model.User.Password = stringCryptography.GetMD5HashData(model.User.Password);

                if (usersRepository.RegistrationUserInsert(model))
                {

                    AutoSignIn(model);
                    //Write code to redirect page to signed page.
                    return RedirectToAction("ClassSubscription", "Reader", new { PublisherUserid = 0, id = model.User.UserId });

                }
                else { pagemessagetype = "saveerror"; pagemessage = model.User.Email + " already exists."; }
            }
            else { pagemessagetype = "saveerror"; }

            return null;
        }

        private void AutoSignIn(UserModel model)
        {

            SignInModel signInModel = new SignInModel();
            signInModel.Email = model.User.Email;
            signInModel.Password = model.User.Password;


            UserListModel accessAccountDetail = usersRepository.GetUserDetail(signInModel.Email, signInModel.Password);

                if (accessAccountDetail != null)
                {

                    AccessModel accessModel = new AccessModel();

                    accessModel.DisplayName = accessAccountDetail.Email;
                    accessModel.CurrentUser_FirstName = accessAccountDetail.FirstName;
                    accessModel.CurrentUser_LastName = accessAccountDetail.LastName;
                    accessModel.CurrentUserId = accessAccountDetail.UserId;
                    accessModel.PublisherCompanyLogo = accessAccountDetail.CompanyLogo;
                    accessModel.PublisherUserId = accessAccountDetail.UserId;

                    Session["AccessModel"] = accessModel;
                }
        }

        private ActionResult SignInNow(SignInModel model)
        {


            if (ValidateSignInForm(model))
            {
                stringCryptography = new StringCryptography();
                model.RememberMePassword = model.Password;
                model.Password = stringCryptography.GetMD5HashData(model.Password);
                model.RememberMe = Request.Form["RememberMe"] == "on" ? true : false;

                UserListModel accessAccountDetail = usersRepository.GetUserDetail(model.Email, model.Password);

                if (accessAccountDetail != null)
                {

                    AccessModel accessModel = new AccessModel();

                    accessModel.DisplayName = accessAccountDetail.Email;
                    accessModel.CurrentUser_FirstName = accessAccountDetail.FirstName;
                    accessModel.CurrentUser_LastName = accessAccountDetail.LastName;
                    accessModel.CurrentUserId = accessAccountDetail.UserId;
                    accessModel.PublisherCompanyLogo = accessAccountDetail.CompanyLogo;
                    accessModel.PublisherUserId = accessAccountDetail.UserId;

                    Session["AccessModel"] = accessModel;

                    SetRememberMe(model);

                    return RedirectToAction("Dashboard", "Reader");
                }
                else { pagemessage = "User name and Password invalid"; pagemessagetype = "saveerror"; }
            }
            else { pagemessagetype = "saveerror"; }

            return null;
        }

        private void UserDataLoad(int Id, ref UserModel model)
        {
            UserListModel userModel = usersRepository.GetUserbyId(Id);

            model.User = new Users() { UserId = userModel.UserId, Email = userModel.Email, Password = userModel.Password, FirstName = userModel.FirstName, LastName = userModel.LastName, CompanyLogo = userModel.CompanyLogo, CompanyName = userModel.CompanyName };
            model.RefUser = new RefUsers() { ChildUserId = userModel.ChildUserId, RoleId = userModel.RoleId, ParentUserId = userModel.ParentUserId, RefUserId = userModel.RefUserId };

            if (model.User.Password != null) { model.EditPassword = model.User.Password; model.User.Password = string.Empty; model.ConfirmPassword = string.Empty; }


        }

        private void DefaultUserDataLoad(UserModel model)
        {
            if (model.UserList == null)
                model.UserList = new List<UserListModel>();

            if (model.RoleList == null)
                model.RoleList = new List<Models.Roles>();

            if (model.User == null)
                model.User = new Users();

            if (model.RefUser == null)
                model.RefUser = new RefUsers();



        }

        private void DefaultRegistrationDataLoad(UserModel model)
        {
            model.User = new Users();

            if (Session["SignUpEmail"] != null)
            {
                model.User.Email = Session["SignUpEmail"].ToString();
                Session["SignUpEmail"] = null;
            }

        }

        private void GetSignInCookies(ref SignInModel model)
        {
            HttpCookie cookie = Request.Cookies["Login"];

            if (cookie != null)
            {
                stringCryptography = new StringCryptography();
                model.RememberMe = true;
                model.Email = cookie.Values[0];
                model.Password = cookie.Values[1];
            }

        }

        private void SetRememberMe(SignInModel model)
        {
            if (model.RememberMe == true)
            {
                HttpCookie myCookie = new HttpCookie("Login");
                if (myCookie != null)
                {
                    myCookie.Expires = DateTime.Now.AddDays(-1d);
                    Response.Cookies.Add(myCookie);
                }

                Response.Cookies["Login"]["UserName"] = model.Email;
                Response.Cookies["Login"]["Password"] = model.RememberMePassword;

                Response.Cookies["Login"].Expires = DateTime.Now.AddMinutes(20);
            }
        }

        #endregion

        private PageRightsModel GetPageRights()
        {
            PageAction _dopage_Action = new PageAction();

            PageRightsModel rightsmodel = new PageRightsModel();

            rightsmodel = _dopage_Action._DoRightsAction(accessAccount.CurrentUserId, accessAccount.PublisherUserId,accessAccount.RoleId);

            //People : Admin, Student, User Permission
            ViewBag._edit_student = rightsmodel._edit_student;
            ViewBag._delete_student = rightsmodel._delete_student;
            ViewBag._add_new_student = rightsmodel._add_new_student;

            ViewBag._add_new_adminuser = rightsmodel._add_new_adminuser;
            ViewBag._edit_adminuser = rightsmodel._edit_adminuser;
            ViewBag._delete_adminuser = rightsmodel._delete_adminuser;

            ViewBag._view_user_permission = rightsmodel._view_user_permission;

            //Class Tools: Build Class, All Class

            ViewBag._add_new_private_class = rightsmodel._add_new_private_class;
            ViewBag._add_new_public_class = rightsmodel._add_new_public_class;

            ViewBag._view_all_private_class = rightsmodel._view_all_private_class;
            ViewBag._view_all_public_class = rightsmodel._view_all_public_class;

            //Quiz Tools : Build Question, All Questions, Build Quiz, All Quiz
            ViewBag._add_question = rightsmodel._add_question;
            ViewBag._edit_question = rightsmodel._edit_question;
            ViewBag._view_question = rightsmodel._view_question;
            ViewBag._delete_question = rightsmodel._delete_question;

            ViewBag._add_new_content = rightsmodel._add_new_content;
            ViewBag._build_quiz = rightsmodel._build_quiz;

            ViewBag._view_all_content_list = rightsmodel._view_all_content_list;
            ViewBag._view_all_quiz = rightsmodel._view_all_quiz;

            return rightsmodel;
        }

        private ActionResult UserAuthentication()
        {
            RedirectToRouteResult returnAction = RedirectToAction("SignIn", "Account");

            ViewBag.IsAssigned = false; ViewBag.ClassId = 0; ViewBag.RoleId = 0;

            if (Session["AccessModel"] != null)
                accessAccount = (AccessModel)Session["AccessModel"];

            if (accessAccount != null)
            {
                ViewBag.DisplayName = accessAccount.DisplayName;
                ViewBag.PublisherCompanyLogo = (!string.IsNullOrEmpty(accessAccount.PublisherCompanyLogo) ? "/Uploads/CompanyLogoFolder/" + accessAccount.PublisherCompanyLogo : "");
                ViewBag.CurrentUser_FirstName = accessAccount.CurrentUser_FirstName;
                ViewBag.CurrentUser_LastName = accessAccount.CurrentUser_LastName;
                ViewBag.PublisherUser_FirstName = accessAccount.PublisherUser_FirstName;
                ViewBag.PublisherUser_LastName = accessAccount.PublisherUser_LastName;

                if (!string.IsNullOrEmpty(accessAccount.AccessCode))
                {
                    ViewBag.IsAssigned = true;
                    ViewBag.ClassId = accessAccount.ClassId;
                }
                else
                {
                    ViewBag.RoleId = accessAccount.RoleId;
                }

            }
            else { accessAccount = new AccessModel(); return returnAction; }
            return null;
        }

        private void SetMenuProperties(string mainmenu_name, string sidemenu_name, string pagenavigation_name)
        {
            ViewBag.Title = "IntelSkill - " + pagenavigation_name;

            ViewBag.MenuNavigation = mainmenu_name;
            ViewBag.SideMenuNavigation = sidemenu_name;
            ViewBag.PageNavigation = pagenavigation_name;


        }

        //Validations

        private bool ValidateProfile(UserModel model)
        {
            bool retval = true;


            if (string.IsNullOrEmpty(model.User.Email))
            { pagemessage += "* Email is required.<br/>"; retval = false; }
            else
            {
                if (!IsValidEmail(model.User.Email))
                {
                    pagemessage += "* Email is invalid.<br/>";
                    retval = false;
                }
                else if (model.User.Email.Length > 75)
                { pagemessage += "* Email must be less than 75 chars.<br/>"; retval = false; }
            }

            if (string.IsNullOrEmpty(model.User.FirstName))
            { pagemessage += "* First name is required.<br/>"; retval = false; }

            else
            {
                if (model.User.FirstName.Length > 20)
                { pagemessage += "* First name can be less than 20 chars.<br/>"; retval = false; }
            }

            if (string.IsNullOrEmpty(model.User.LastName))
            { pagemessage += "* Last name is required.<br/>"; retval = false; }

            else
            {
                if (model.User.LastName.Length > 20)
                { pagemessage += "* Last name can be less than 20 chars.<br/>"; retval = false; }
            }

            if (Request.Files["ModifyLogo"].ContentLength > 0)
            {
                string[] acceptedTypes = new string[] { "image/png", "image/x-png" };//"image/bmp", "image/jpeg", "image/gif", 

                if (!acceptedTypes.Contains(Request.Files["ModifyLogo"].ContentType))
                {
                    pagemessage += "* Company logo must be in png format.<br/>"; retval = false;
                }
                else
                {
                    if (Request.Files["ModifyLogo"].ContentLength > 10 * 1024)
                    {
                        System.Drawing.Image image = System.Drawing.Image.FromStream(Request.Files["ModifyLogo"].InputStream, true, true);

                        if (image.Width > 120 && image.Height > 41)
                        {
                            pagemessage += "* Maximum dimension of company logo could be 120x41.<br/>"; retval = false;
                        }
                    }
                }
            }


            return retval;
        }

        private bool ValidateSignInForm(SignInModel model)
        {
            bool retval = true;

            if (string.IsNullOrEmpty(model.Email))
            {
                pagemessage += "* User name is required<br/>";
                retval = false;
            }

            if (string.IsNullOrEmpty(model.Password))
            {
                pagemessage += "* Password is required<br/>";
                retval = false;
            }

            if (!retval)
                pagemessagetype = "error";
            return retval;
        }

        private bool ValidateRegistrationForm(UserModel model)
        {
            bool retval = true;

            if (string.IsNullOrEmpty(model.User.Email))
            { pagemessage += "* Email is required.<br/>"; retval = false; }
            else
            {
                if (!IsValidEmail(model.User.Email))
                {
                    pagemessage += "* Email is invalid.<br/>";
                    retval = false;
                }
                else if (model.User.Email.Length > 75)
                { pagemessage += "* Email must be less than 75 chars.<br/>"; retval = false; }
            }

            if (string.IsNullOrEmpty(model.User.Password))
            { pagemessage += "* Password is required.<br/>"; retval = false; }
            else
            {

                if (model.User.Password.Length > 10 || model.User.Password.Length < 6)
                { pagemessage += "* Password must be between 6 - 10 chars.<br/>"; retval = false; }
                else
                {
                    string confirm = string.Empty;

                    if (string.IsNullOrEmpty(Request.Form["ConfirmPassword"]))
                    {
                        confirm = "";
                    }
                    else { confirm = Request.Form["ConfirmPassword"].ToString(); model.ConfirmPassword = confirm; }


                    if (!IsPasswordMatch(model.User.Password, confirm))
                    { pagemessage += "* Password doesn't match.<br/>"; retval = false; }
                }
            }


            if (string.IsNullOrEmpty(model.User.FirstName))
            { pagemessage += "* First name is required.<br/>"; retval = false; }

            else
            {
                if (model.User.FirstName.Length > 20)
                { pagemessage += "* First name can be less than 20 chars.<br/>"; retval = false; }
            }

            if (string.IsNullOrEmpty(model.User.LastName))
            { pagemessage += "* Last name is required.<br/>"; retval = false; }

            else
            {
                if (model.User.LastName.Length > 20)
                { pagemessage += "* Last name can be less than 20 chars.<br/>"; retval = false; }
            }

            return retval;
        }

        private bool ValidateChangePassword(ChangePasswordModel model)
        {
            bool retval = true;


            if (string.IsNullOrEmpty(model.OldPassword))
            { pagemessage += "* Old password is required.<br/>"; retval = false; }

            if (string.IsNullOrEmpty(model.NewPassword))
            { pagemessage += "* New password is required.<br/>"; retval = false; }

            if (string.IsNullOrEmpty(model.ReTypePassword))
            { pagemessage += "* Re-type password is required.<br/>"; retval = false; }

            if (!string.IsNullOrEmpty(model.NewPassword) && !string.IsNullOrEmpty(model.ReTypePassword))
            {
                if (model.NewPassword != model.ReTypePassword)
                { pagemessage += "* New password and Retype New password doesn't match.<br/>"; retval = false; }
            }

            if ((!string.IsNullOrEmpty(model.NewPassword)) && (!string.IsNullOrEmpty(model.OldPassword)))
                if (model.NewPassword == model.OldPassword)
                {
                    pagemessage += "* New password & Old password might not be same.<br/>"; retval = false;
                }


            return retval;
        }

        public static bool IsValidEmail(string email)
        {

            System.Text.RegularExpressions.Regex rx = new System.Text.RegularExpressions.Regex(
            @"^[-!#$%&'*+/0-9=?A-Z^_a-z{|}~](\.?[-!#$%&'*+/0-9=?A-Z^_a-z{|}~])*@[a-zA-Z](-?[a-zA-Z0-9])*(\.[a-zA-Z](-?[a-zA-Z0-9])*)+$");
            return rx.IsMatch(email);
        }

        public static bool IsNumeric(string number)
        {

            System.Text.RegularExpressions.Regex rx = new System.Text.RegularExpressions.Regex(
            @"[0-9]+");
            return rx.IsMatch(number);
        }

        private bool IsPasswordMatch(string firstPassword, string secondPassword)
        {
            return (firstPassword == secondPassword);
        }
    }
}
