﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.Mvc;
using System.Web.Security;
using System.Text.RegularExpressions;
using System.IO;
using System.Web.Helpers;
using System.Net.Mail;
using System.Net;

using IntelSkill.Models;
using IntelSkill.Repository;
using IntelSkill.DAL;
using IntelSkill.Utilities;


namespace IntelSkill.Controllers
{
    public class PublisherController : Controller
    {

        private StringCryptography stringCryptography;

        private IUsersRepository usersRepository;
        private IQuizRepository QuizRepository;
        private IQuestionRepository questionRepository;
        private IContentRepository ContentRepository;
        private IClassRepository classRepository;

        static string mainFolder = GlobalSettings.UploadFolder;

        bool? IsAdmin = null;

        private int roleId = 0;

        private AccessModel accessAccount = null;

        ActionResult loginActionResult = null;


        private string pagemessage = string.Empty;
        private string pagemessagetype = string.Empty;

        public const int PageSize = 6;

        public PublisherController()
        {
            this.usersRepository = new UsersRepository(new SQLDBContext());
            this.classRepository = new ClassRepository(new SQLDBContext());
            this.QuizRepository = new QuizRepository(new SQLDBContext());
            this.questionRepository = new QuestionRepository(new SQLDBContext());
            this.ContentRepository = new ContentRepository(new SQLDBContext());


        }

        // GET: Dashboard
        public ActionResult Index()
        {
            return View();
        }

        #region People Menu View's

        //GET:
        public ActionResult Admin(int Id = 0, int rid = 0)
        {

            ActionResult loginActionResult = UserAuthentication();

            if (accessAccount.CurrentUserId == 0)
                return loginActionResult;
            else if (!string.IsNullOrEmpty(accessAccount.AccessCode))
            {
                return RedirectToAction("BuildClass", "Publisher");
            }

            var model = new UserModel();

            if (Id > 0 && rid > 0)
            {
                UserEditDataLoad(Id, accessAccount.PublisherUserId, rid, ref model);
                model.AddNewUser = true;
                model.Display = true;


            }

            DefaultUserDataLoad(model, "Administrator");

            model.PageRightsList = GetPageRights();

            SetMenuProperties("People", "", "Admin");

            return View(model);
        }

        public ActionResult Student(int Id = 0, int rid = 0, int? page = 1)
        {

            ActionResult loginActionResult = UserAuthentication();

            if (accessAccount.CurrentUserId == 0)
                return loginActionResult;
            else if (!string.IsNullOrEmpty(accessAccount.AccessCode))
            {
                return RedirectToAction("BuildClass", "Publisher");
            }
            var model = new UserModel();
            model.PageRightsList = GetPageRights();

            if (Id > 0)
            {
                UserEditDataLoad(Id, accessAccount.PublisherUserId, rid, ref model);
                model.AddNewUser = true;
                model.Display = true;
            }
            DefaultUserDataLoad(model, "Reader");
            SetMenuProperties("People", "", "Student");

            return View(model);
        }

        public ActionResult UserPermissions()
        {
            ActionResult loginActionResult = UserAuthentication();

            if (accessAccount.CurrentUserId == 0)
                return loginActionResult;
            else if (!string.IsNullOrEmpty(accessAccount.AccessCode))
            {
                return RedirectToAction("BuildClass", "Publisher");
            }
            var model = new UserPermissionsModel();
            model.PageRightsList = GetPageRights();
            DefaultUserPermissionDataLoad(model);

            return View(model);
        }

        public ActionResult StudentOverview(int? page = 1, int ContentId = 0, int ClassId = 0, int StudentId = 0, int QuizReportId = 0)
        {

            ActionResult loginActionResult = UserAuthentication();

            if (accessAccount.CurrentUserId == 0)
                return loginActionResult;

            var model = new UserModel();
            model.PageRightsList = GetPageRights();
            if (QuizReportId != 0)
            {
                LoadStudentQuizReport(model, QuizReportId, ClassId, ContentId);
            }
            else if (StudentId != 0 && ContentId == 0)
            {
                LoadContentbyStudent(model, StudentId);
            }
            else if (ContentId != 0)
            {
                LoadStudentbyContent(model, ContentId, ClassId);
            }
            return View(model);
        }

        public ActionResult Subscribers(int StudentId = 0, int ClassId = 0, int QuizReportId = 0, int? page = 1, int RequestedId = 0)
        {
            ActionResult loginActionResult = UserAuthentication();

            if (accessAccount.CurrentUserId == 0)
                return loginActionResult;

            var model = new UserModel();
            SetMenuProperties("People", "", "Subscribers");
            model.PageRightsList = GetPageRights();


            if (QuizReportId != 0)
            {
                LoadSubcriberQuizResult(QuizReportId, StudentId, ClassId, page.Value, model);

            }
            else if (StudentId != 0 && ClassId == 0)
            {
                LoadSubsriberClasses(StudentId, page.Value, model);
            }
            else if (ClassId != 0 && StudentId != 0)
            {
                LoadSubscribedContentResult(StudentId, ClassId, page.Value, model);
            }

            else
            {
                model.SubscribtionList = usersRepository.SubscribersList(accessAccount.PublisherUserId);
            }
            return View(model);
        }

        //POST: 
        [HttpPost]
        public ActionResult Admin(UserModel model, FormCollection collection, string btnSubmit = "")
        {

            ActionResult loginActionResult = UserAuthentication();

            if (accessAccount.CurrentUserId == 0)
                return loginActionResult;

            if (btnSubmit.ToLower() == "apply")
            {
                IsAdmin = true;
                UserBulkAction(collection);

            }
            else if (btnSubmit.ToLower() == "search")
            {
                UserSearchAction(model);

            }
            else if (btnSubmit.ToLower() == "searchusertoadd")
            {
                UserSearchActionToaddAdmin(model);

            }
            else if (btnSubmit.ToLower() == "save")
            {
                UserSaveAction(model);
                SendmailtoUser(model, "send_request");
            }
            else if (btnSubmit.ToLower() == "saveaddeduser")
            {
                AddExistingUser(model);
                SendmailtoUser(model, "send_request");

            }

            DefaultUserDataLoad(model, "Administrator");

            model.PageRightsList = GetPageRights();

            SetMenuProperties("People", "", "Admin");

            model.ValidationMessage = pagemessage;
            model.ValidationMessageType = pagemessagetype;

            return View("Admin", model);
        }

        [HttpPost]
        public ActionResult Student(UserModel model, FormCollection collection, string btnSubmit = "")
        {
            ActionResult loginActionResult = UserAuthentication();

            if (accessAccount.CurrentUserId == 0)
                return loginActionResult;

            if (btnSubmit.ToLower() == "apply")
            {
                UserBulkAction(collection);
            }
            else if (btnSubmit.ToLower() == "search")
            {
                UserSearchAction(model);
            }
            else if (btnSubmit.ToLower() == "searchusertoadd")
            {
                UserSearchActionToaddStudent(model);

            }
            else if (btnSubmit.ToLower() == "save")
            {
                UserSaveAction(model);
                SendmailtoUser(model, "send_request");
            }
            else if (btnSubmit.ToLower() == "saveaddeduser")
            {
                AddExistingUser(model);
                SendmailtoUser(model, "send_request");


            }


            DefaultUserDataLoad(model, "Reader");

            model.PageRightsList = GetPageRights();

            SetMenuProperties("People", "", "Student");

            model.ValidationMessage = pagemessage;
            model.ValidationMessageType = pagemessagetype;

            return View("Student", model);
        }

        [HttpPost]
        public ActionResult UserPermissions(UserPermissionsModel model, FormCollection collection, string btnSubmit = "")
        {
            ActionResult loginActionResult = UserAuthentication();

            if (accessAccount.CurrentUserId == 0)
                return loginActionResult;

            if (!string.IsNullOrEmpty(btnSubmit))
            {
                SaveUserPermission(model, collection, btnSubmit);
            }
            else if (!string.IsNullOrEmpty(Request.Form["RoleId"].ToString()))
            {
                LoadSelectedRoleUsers(model);
            }
            else if (Request.Form["radio"] != null)
            {
                LoadSelectedUserPermission(model);
            }


            DefaultUserPermissionDataLoad(model);
            model.PageRightsList = GetPageRights();
            model.ValidationMessage = pagemessage;
            model.ValidationMessageType = pagemessagetype;

            return View(model);
        }

        [HttpPost]
        public ActionResult Subscribers(int SubscriptionId = 0, int StudentId = 0, int? page = 1)
        {

            ActionResult loginActionResult = UserAuthentication();
            if (accessAccount.CurrentUserId == 0)
                return loginActionResult;

            var model = new UserModel();
            SetMenuProperties("People", "", "Subscribers");
            model.PageRightsList = GetPageRights();



            model.SubcribedClasses = new List<SubcribedClasses>();

            if (SubscriptionId != 0)
            {
                QuizRepository.DeleteClassSubId(SubscriptionId);
                QuizRepository.Save();
                model.ValidationMessageType = "success";
                model.ValidationMessage = "Class unsubscribed successfully";
            }
            if (StudentId != 0)
            {
                LoadSubsriberClasses(StudentId, page.Value, model);
            }
            return View(model);
        }

        //FUNCTION: 
        private void DefaultUserPermissionDataLoad(UserPermissionsModel model)
        {
            if (model.RoleList == null)
                model.RoleList = usersRepository.GetUserRoles("Administrator");

            if (model.UsersList == null)
                model.UsersList = usersRepository.GetUserList(accessAccount.PublisherUserId, "Administrator");

            if (model.PageRightsList == null)
                GetUserPermissionsPageRights(model);

            if (model.ModuleList == null)
                model.ModuleList = usersRepository.GetModules();

            if (model.RightList == null)
                model.RightList = usersRepository.GetRights().ToList().OrderBy(m => m.ModuleName);

            if (model.AssignedRightSelectList == null)
                LoadSelectedUserPermission(model);

            if (model.UserRightsSelectList == null)
                model.UserRightsSelectList = new List<RefUsers>();

            SetMenuProperties("People", "", "Users & Permissions");
        }

        private void DefaultUserDataLoad(UserModel model, string userGroup)
        {
            if (model.UserList == null)
                model.UserList = usersRepository.GetUserList(accessAccount.PublisherUserId, userGroup);

            if (model.RoleList == null)
                model.RoleList = usersRepository.GetUserRoles(userGroup);

            if (model.User == null)
                model.User = new Users();

            if (model.RefUser == null)
                model.RefUser = new RefUsers();



        }

        private void GetUserPermissionsPageRights(UserPermissionsModel model)
        {
            PageAction _dopage_Action = new PageAction();
            model.PageRightsList = _dopage_Action._DoRightsAction(accessAccount.CurrentUserId, accessAccount.PublisherUserId, accessAccount.RoleId);

            ViewBag._add_new_private_class = model.PageRightsList._add_new_private_class;
            ViewBag._add_new_public_class = model.PageRightsList._add_new_public_class;

            ViewBag._add_new_content = model.PageRightsList._add_new_content;
            ViewBag._view_all_content_list = model.PageRightsList._view_all_content_list;

            ViewBag._edit_student = model.PageRightsList._edit_student;
            ViewBag._delete_student = model.PageRightsList._delete_student;
            ViewBag._add_new_student = model.PageRightsList._add_new_student;

            ViewBag._add_new_adminuser = model.PageRightsList._add_new_adminuser;
            ViewBag._edit_adminuser = model.PageRightsList._edit_adminuser;
            ViewBag._delete_adminuser = model.PageRightsList._delete_adminuser;

            ViewBag._view_user_permission = model.PageRightsList._view_user_permission;
        }

        private void LoadSelectedRoleUsers(UserPermissionsModel model)
        {
            model.UsersList = usersRepository.GetUsersbyRole(accessAccount.PublisherUserId, model.RoleId).ToList().OrderBy(m => m.Email);
        }

        private void LoadSelectedUserPermission(UserPermissionsModel model)
        {
            int userId = 0;
            if (Request.Form["radio"] != null)
            {
                userId = Int32.Parse(Request.Form["radio"].ToString());

                if (userId > 0)
                {
                    model.AssignedRightSelectList = usersRepository.GetUserRights(userId);
                }
            }
            else { model.AssignedRightSelectList = new List<Rights>(); }
        }

        private void SaveUserPermission(UserPermissionsModel model, FormCollection collection, string btnSubmit)
        {
            if (ValidateApplyChangesData())
            {
                int selectedUserId = 0;

                pagemessagetype = string.Empty;
                pagemessage = string.Empty;

                List<int> rightid = new List<int>();

                Int32.TryParse(Request.Form["radio"].ToString(), out selectedUserId);

                if (Request.Form["chk"] != null)
                {
                    int j = Request.Form["chk"].Split(',').Count();

                    for (int i = 0; i < j; i++)
                    {
                        rightid.Add(Int32.Parse(collection.GetValues("chk")[i].ToString()));

                    }
                }


                if (usersRepository.SaveUserRights(selectedUserId, accessAccount.PublisherUserId, rightid))
                {
                    usersRepository.Save();

                    pagemessagetype = "success";
                    pagemessage = "You have successfully assigned rights.";
                }
                else
                {
                    pagemessagetype = "error";
                    pagemessage = "permission saving failed.";
                }

            }
            else { pagemessagetype = "error"; }
        }

        private bool ValidateApplyChangesData()
        {
            bool retval = true;

            if (Request.Form["radio"] == null)
            {
                pagemessage += "* Please select user<br/>";
                retval = false;
            }

            return retval;
        }

        private bool ValidateUserForm(UserModel model)
        {
            bool retval = true;


            if (string.IsNullOrEmpty(model.User.Email))
            { pagemessage += "* Email is required.<br/>"; retval = false; }
            else
            {
                if (!IsValidEmail(model.User.Email))
                {
                    pagemessage += "* Email is invalid.<br/>";
                    retval = false;
                }
                else if (model.User.Email.Length > 75)
                { pagemessage += "* Email must be less than 75 chars.<br/>"; retval = false; }
            }

            if (model.User.UserId == 0)
            {
                if (string.IsNullOrEmpty(model.User.Password))
                { pagemessage += "* Password is required.<br/>"; retval = false; }
                else
                {

                    if (model.User.Password.Length > 10 || model.User.Password.Length < 6)
                    { pagemessage += "* Password must be between 6 - 10 chars.<br/>"; retval = false; }
                    else
                    {
                        string confirm = string.Empty;

                        if (string.IsNullOrEmpty(Request.Form["ConfirmPassword"]))
                        {
                            confirm = "";
                        }
                        else { confirm = Request.Form["ConfirmPassword"].ToString(); model.ConfirmPassword = confirm; }


                        if (!IsPasswordMatch(model.User.Password, confirm))
                        { pagemessage += "* Password doesn't match.<br/>"; retval = false; }
                    }
                }

            }
            else
            {
                if (model.User.Password != null)
                {
                    if (model.User.Password.Length > 10 || model.User.Password.Length < 6)
                    { pagemessage += "* Password must be between 6 - 10 chars.<br/>"; retval = false; }
                    else
                    {
                        string confirm = string.Empty;

                        if (string.IsNullOrEmpty(Request.Form["ConfirmPassword"]))
                        {
                            confirm = "";
                        }
                        else { confirm = Request.Form["ConfirmPassword"].ToString(); model.ConfirmPassword = confirm; }


                        if (!IsPasswordMatch(model.User.Password, confirm))
                        { pagemessage += "* Password doesn't match.<br/>"; retval = false; }
                    }
                }
                else if (!string.IsNullOrEmpty(Request.Form["ConfirmPassword"]))
                {
                    if (Request.Form["ConfirmPassword"].Length > 10 || Request.Form["ConfirmPassword"].Length < 6)
                    { pagemessage += "* Password must be between 6 - 10 chars.<br/>"; retval = false; }
                    else
                    {
                        string confirm = string.Empty;

                        if (string.IsNullOrEmpty(Request.Form["ConfirmPassword"]))
                        {
                            confirm = "";
                        }
                        else { confirm = Request.Form["ConfirmPassword"].ToString(); model.ConfirmPassword = confirm; }


                        if (!IsPasswordMatch(model.User.Password, confirm))
                        { pagemessage += "* Password doesn't match.<br/>"; retval = false; }
                    }
                }
            }


            if (string.IsNullOrEmpty(model.User.FirstName))
            { pagemessage += "* First Name is required.<br/>"; retval = false; }

            else
            {
                if (model.User.FirstName.Length > 20)
                { pagemessage += "* First Name can be less than 20 chars.<br/>"; retval = false; }
            }

            if (string.IsNullOrEmpty(model.User.LastName))
            { pagemessage += "* Last Name is required.<br/>"; retval = false; }

            else
            {
                if (model.User.LastName.Length > 20)
                { pagemessage += "* Last Name can be less than 20 chars.<br/>"; retval = false; }
            }


            if (model.RefUser.RoleId == 0)
            { pagemessage += "* Please select Role.<br/>"; retval = false; }


            return retval;
        }

        private void UserEditDataLoad(int Id, int cid, int rid, ref UserModel model)
        {
            UserListModel userModel = usersRepository.GetUsersbyIdandcurrentuserid(Id, cid, rid);

            model.User = new Users() { UserId = userModel.UserId, Email = userModel.Email, Password = userModel.Password, FirstName = userModel.FirstName, LastName = userModel.LastName, CompanyLogo = userModel.CompanyLogo, CompanyName = userModel.CompanyName };
            model.RefUser = new RefUsers() { ChildUserId = userModel.ChildUserId, RoleId = userModel.RoleId, ParentUserId = userModel.ParentUserId, RefUserId = userModel.RefUserId };

            if (model.User.Password != null) { model.EditPassword = model.User.Password; model.User.Password = string.Empty; model.ConfirmPassword = string.Empty; }


        }

        private void UserSaveAction(UserModel model)
        {


            bool retval = false;
            RefUsers refuser = new RefUsers();
            Users user = new Users();
            if (ValidateUserForm(model))
            {
                stringCryptography = new StringCryptography();

                model.User.UserTypeId = Convert.ToInt32(UserTypes.StandardUser);
                model.User.UserStatus = true;

                if (model.User.UserId == 0)
                {
                    model.RefUser.ParentUserId = accessAccount.PublisherUserId;
                    model.User.Password = stringCryptography.GetMD5HashData(model.User.Password);
                    model.User.CreatedDate = System.DateTime.Now;
                    model.User.CreatedUserId = accessAccount.CurrentUserId;

                    retval = usersRepository.InsertUser(model);

                    if (retval)
                    {

                        pagemessage = model.User.Email + " created successfully.";
                        ModelState.Clear();
                        pagemessagetype = "savesuccess";
                        //model.RefUser = new RefUsers();
                        // model.User = new Users();
                    }
                    else
                    {
                        pagemessagetype = "saveerror";
                        pagemessage = model.User.Email + " already exists.";
                    }
                }
                else
                {
                    if (!string.IsNullOrEmpty(model.User.Password))
                        model.User.Password = stringCryptography.GetMD5HashData(model.User.Password);
                    else
                        model.User.Password = model.EditPassword;

                    model.User.ModifiedDate = System.DateTime.Now;
                    model.User.ModifiedUserId = accessAccount.CurrentUserId;
                    //RefUsers refusr = usersRepository.GetUserRow(accessAccount.UserId, model.User.UserId);
                    //if (refusr != null)
                    //{
                    //    refusr.RoleId=model.RefUser.RoleId;

                    //}

                    retval = usersRepository.UpdateUser(model);

                    if (retval)
                    {

                        ModelState.Clear();
                        pagemessage = model.User.Email + " updated successfully.";
                        model.RefUser = new RefUsers();
                        model.User = new Users();
                        pagemessagetype = "savesuccess";

                    }
                    else
                    {
                        pagemessagetype = "saveerror";
                        pagemessage = model.User.Email + " already exists.";
                    }

                }
            }
            else
            {
                pagemessagetype = "saveerror";
                model.AddNewUser = true;
            }

        }

        private void UserSearchAction(UserModel model)
        {
            if (!string.IsNullOrEmpty(Request.Form["searchuser"]))
            {
                string search_query = Request.Form["searchuser"].ToString();

                if (search_query.Length > 0)
                {
                    model.UserList = usersRepository.GetUsersSearchResults(accessAccount.PublisherUserId, search_query, roleId);

                    pagemessagetype = "info";
                    pagemessage = "Search Result(s) Found (" + model.UserList.Count() + ")";
                }
                else
                {
                    pagemessagetype = "info";
                    pagemessage = "No Search text entered.";
                }
            }
        }
        private void UserSearchActionToaddAdmin(UserModel model)
        {
            bool retval = true;

            if (string.IsNullOrEmpty(model.User.Email))
            {
                pagemessagetype = "saveerror";
                pagemessage += "* Email is required.<br/>"; retval = false;
            }
            else
            {

                if (model.User.Email.Length > 75)
                {
                    pagemessagetype = "saveerror";
                    pagemessage += "* Email must be less than 75 chars.<br/>"; retval = false;
                }
                else
                {

                    model.UserListonemail = usersRepository.GetUserListonEmailAdmin(accessAccount.PublisherUserId, model);

                    if (model.UserListonemail != null)
                    {
                        pagemessagetype = "saveerror";
                        pagemessage = "Already added this user as  " + model.UserListonemail.RoleName + "";

                    }
                    else
                    {

                        if (accessAccount.DisplayName == model.User.Email)
                        {
                            pagemessagetype = "saveerror";
                            pagemessage = "Sorry you cannot add yourself in user role";
                        }
                        else
                        {
                            retval = usersRepository.SearchEmail(model);
                            if (retval)
                            {
                                pagemessagetype = "";
                                pagemessage = "";
                                model.AddUser = true;

                            }
                            else
                            {
                                pagemessagetype = "";
                                pagemessage = " ";
                                model.AddNewUser = true;

                            }
                            if (!IsValidEmail(model.User.Email))
                            {
                                pagemessage = null;
                                pagemessagetype = "saveerror";
                                pagemessage += "* Email is invalid.<br/>";
                                retval = false;
                                model.AddNewUser = false;
                            }
                        }
                    }

                }

            }



        }
        private void UserSearchActionToaddStudent(UserModel model)
        {
            bool retval = true;

            if (string.IsNullOrEmpty(model.User.Email))
            {
                pagemessagetype = "saveerror";
                pagemessage += "* Email is required.<br/>"; retval = false;
            }
            else
            {

                if (model.User.Email.Length > 75)
                {
                    pagemessagetype = "saveerror";
                    pagemessage += "* Email must be less than 75 chars.<br/>"; retval = false;
                }
                else
                {

                    model.UserListonemail = usersRepository.GetUserListonEmailStudent(accessAccount.PublisherUserId, model);
                    if (model.UserListonemail != null)
                    {
                        pagemessagetype = "saveerror";
                        pagemessage = "Already added this user as  " + model.UserListonemail.RoleName + "";

                    }
                    else
                    {
                        if (accessAccount.DisplayName == model.User.Email)
                        {
                            pagemessagetype = "saveerror";
                            pagemessage = "Sorry you cannot add yourself in user role";
                        }
                        else
                        {
                            retval = usersRepository.SearchEmail(model);
                            if (retval)
                            {
                                pagemessagetype = "";
                                pagemessage = "";
                                model.AddUser = true;

                            }
                            else
                            {
                                pagemessagetype = "";
                                pagemessage = " ";
                                model.AddNewUser = true;

                            }
                            if (!IsValidEmail(model.User.Email))
                            {
                                pagemessage = null;
                                pagemessagetype = "saveerror";
                                pagemessage += "* Email is invalid.<br/>";
                                retval = false;
                                model.AddNewUser = false;
                            }
                        }
                    }

                }

            }



        }
        private void AddExistingUser(UserModel model)
        {
            bool retval = false;

            if (model.RefUser.RoleId == 0)
            {
                pagemessagetype = "saveerror";
                pagemessage += "* Please select role.<br/>";
                retval = false;
                model.AddUser = true;
            }
            else
            {
                model.RefUser.ParentUserId = accessAccount.PublisherUserId;
                model.User.CreatedDate = System.DateTime.Now;
                retval = usersRepository.InsertRefUser(model);


                ModelState.Clear();
                pagemessage = model.User.Email + " created successfully.";
                // model.RefUser = new RefUsers();
                // model.User = new Users();
                pagemessagetype = "savesuccess";
            }

        }



        private void SendmailtoUser(UserModel model, string storeType)
        {

            bool retval = true;
            if (model.User.Email != null)
                model.RequestMail = model.User.Email;
            model.IsRequestMailEnabled = !string.IsNullOrEmpty(model.RequestMail);
            retval = usersRepository.SearchEmail(model);

            #region Send Request Settings
            if (model.User.Password != null)
            {
                if (storeType == "send_request" && !string.IsNullOrEmpty(model.RequestMail))
                {
                    model.userpassword = model.ConfirmPassword;
                }
            }
            #endregion
            if (retval)
            {
                SendMailtoregUser(model, storeType);
            }


        }

        private void SendMailtoregUser(UserModel model, string storeType)
        {
            //Send mail to requested user.

            string BodyContent = null;
            string password = null;
            if (model.IsRequestMailEnabled && storeType == "send_request")
            {

                string port = Request.Url.Host.Contains("localhost") ? ":" + Request.Url.Port.ToString() + "/" : string.Empty;
                string imageHost = Request.Url.Host.Contains("localhost") ? "http://intelskill.com/" : Request.Url.Host + "/";
                string hostpath = (Request.Url.Host + port).EndsWith("/") ? Request.Url.Host + port : Request.Url.Host + "/";

                if (model.RefUser.RoleId == 3)
                {
                    model.UserListonemail = usersRepository.GetUserListonEmailStudent(accessAccount.PublisherUserId, model);

                    if (model.User.Password == null)
                    {
                        BodyContent = @"<br/> <br/>You have been added as  " + model.UserListonemail.RoleName + "&nbsp;" + "by" + "&nbsp;" + accessAccount.DisplayName + @"<br/><br/>
                       Please <a href='" + hostpath + "Account/SignIn" + @"'>click here</a>  to Sign in;
                         
                                    If any problem to view the link please copy & paste the below url to continue.<br/> 
                                    " + hostpath + "Account/SignIn" + @"<br/><br/> ";
                    }
                    else
                    {
                        BodyContent = @"<br/> <br/>You have been added as  " + model.UserListonemail.RoleName + "&nbsp;" + "by" + "&nbsp;" + accessAccount.DisplayName + @"<br/><br/>Your Username : " + model.RequestMail + " " + @"<br/><br/> Your Password : " + model.ConfirmPassword + " " + @"<br/><br/>
                       Please <a href='" + hostpath + "Account/SignIn" + @"'>click here</a> to Sign in;
                         
                                    If any problem to view the link please copy & paste the below url to continue.<br/> 
                                    " + hostpath + "Account/SignIn" + @"<br/><br/> ";
                    }

                }
                else
                {
                    model.UserListonemail = usersRepository.GetUserListonEmailAdmin(accessAccount.PublisherUserId, model);
                    if (model.User.Password == null)
                    {
                        BodyContent = @"<br/> <br/>You have been added as  " + model.UserListonemail.RoleName + "&nbsp;" + "by" + "&nbsp;" + accessAccount.DisplayName + @"<br/><br/>
                       Please <a href='" + hostpath + "Account/SignIn" + @"'>click here</a> to Sign in;
                         
                                    If any problem to view the link please copy & paste the below url to continue.<br/> 
                                    " + hostpath + "Account/SignIn" + @"<br/><br/> ";

                        //BodyContent = @"<br/> <br/>You have been added as  " + model.UserListonemail.RoleName + "&nbsp;" +"by"+"&nbsp;" + accessAccount.DisplayName + ""+@"<br/><br/>Please <a href='" + hostpath + "Access/?accessRequest=" + model.ClassModel.AccessCode + @"'>click here</a> to access the requested class details@"<br/>";
                    }

                    else
                    {

                        BodyContent = @"<br/> <br/>You have been added as  " + model.UserListonemail.RoleName + "&nbsp;" + "by" + "&nbsp;" + accessAccount.DisplayName + @"<br/><br/>Your Username : " + model.RequestMail + " " + @"<br/><br/> Your Password : " + model.ConfirmPassword + " " + @"<br/><br/>
                       Please <a href='" + hostpath + "Account/SignIn" + @"'>click here</a> to Sign in;
                         
                                    If any problem to view the link please copy & paste the below url to continue.<br/> 
                                    " + hostpath + "Account/SignIn" + @"<br/><br/> ";
                    }
                }


                string appPath = Server.MapPath("/Content/newsletter-template/Useraddress.htm");
                string fileData = System.IO.File.ReadAllText(appPath);

                /* Replace those word.
                 @CssHostLink          : newsletters.css
                 @TodayDate            : March 28, 2013
                 @Receiver             : Customer
                
                 @ClassImageLocation
              */
                string output = Regex.Replace(fileData, "@TodayDate", System.DateTime.Now.ToString("MMMM dd, yyyy"));
                output = Regex.Replace(output, "@Receiver", model.UserListonemail.FirstName);
                //output = Regex.Replace(output, "@Username",model.RequestMail );
                output = Regex.Replace(output, "@BodyContent", BodyContent);
                output = Regex.Replace(output, "@ClassImageLocation", imageHost + "Content");

                var fromAddress = new MailAddress(accessAccount.DisplayName, "IntelSkill");
                var toAddress = new MailAddress(model.RequestMail);
                string subject = accessAccount.DisplayName + " has sent request from IntelSkill.Com";
                string body = output;

                var smtp = new SmtpClient();

                using (var message = new MailMessage(fromAddress, toAddress)
                {
                    Subject = subject,
                    Body = body,
                    IsBodyHtml = true
                })
                    try
                    {
                        smtp.Send(message);
                        pagemessage = "Request has been sent to " + model.RequestMail;
                    }
                    catch (Exception ex) { pagemessage = ex.ToString(); pagemessagetype = "error"; }

            }
        }



        private void UserBulkAction(FormCollection collection)
        {
            if (Request.Form["chk"] != null && Request.Form["select"] != null)
            {
                int selected_count = collection.GetValues("chk").Count();

                string action = collection.GetValues("select")[0].ToLower();

                switch (action)
                {
                    case "remove":
                        {
                            List<bool> retval = new List<bool>();

                            int parentUserId = accessAccount.PublisherUserId;

                            for (int i = 0; i < selected_count; i++)
                            {
                                int childuserId = Int32.Parse(collection.GetValues("chk")[i].ToString());

                                if (IsAdmin != true)
                                {
                                    roleId = 3;
                                }

                                bool ret = usersRepository.DeleteUser(childuserId, parentUserId, roleId);

                                retval.Add(ret);
                            }
                            if (retval.Contains(false))
                            {
                                pagemessagetype = "error";
                                pagemessage = "Some user(s) can't delete. It may have refer in Content(s) or Class(s)";
                            }
                            else
                            {
                                pagemessagetype = "success";
                                pagemessage = "User(s) removed successfully.";
                            }

                            break;
                        }

                    default:
                        {
                            pagemessagetype = "warning";
                            pagemessage = "No Action Selected";
                            break;
                        }


                }

            }
        }

        #region StudentOverview

        private void LoadStudentQuizReport(UserModel model, int QuizReportId, int ClassId, int ContentId)
        {
            int page = 1;

            Users usr = ContentRepository.StudentbyQuizRptId(QuizReportId);
            ViewBag.ContentStudent = usr.FirstName + " " + usr.LastName;
            ViewBag.StudentId = usr.UserId;
            ViewBag.QuizReportId = QuizReportId;
            model.StudentQuizResult = ContentRepository.StudentQuizResult(QuizReportId);
            if (ContentId != 0)
            {
                SetMenuProperties("Dashboard", "", "");
                Classes cls = ContentRepository.GetClassbyId(Convert.ToInt32(ClassId));
                string ContentName = ContentRepository.GetContentTitle(ContentId);
                ViewBag.ClassId = Convert.ToInt32(ClassId);
                ViewBag.ContentId = ContentId;
                SetOverviewProperties(cls.ClassName, ContentName, ViewBag.ContentStudent, "Question and Answers(" + model.StudentQuizResult.Count() + ")");
                ViewBag.Parameter = ("&QuizReportId=" + QuizReportId + "" + "&ContentId=" + ContentId + "" + "&ClassId=" + ClassId);
            }
            else
            {
                Contents Cont = ContentRepository.ContentbyQuizRptId(QuizReportId);
                if (Cont != null)
                {
                    SetOverviewProperties("", Cont.ContentTitle, ViewBag.ContentStudent, "Question and Answers(" + model.StudentQuizResult.Count() + ")");

                }
                SetMenuProperties("People", "", "Student");

                ViewBag.Parameter = ("&QuizReportId=" + QuizReportId);
            }

            ViewBag.Title = "IntelSkill - Question and Answers";

            if (Request.QueryString["page"] != null) { Int32.TryParse(Request.QueryString["page"].ToString(), out page); }
            PopulatePagingData(model.StudentQuizResult.Count(), "Publisher", "StudentOverview", page);

            if (page <= 1)
            {
                model.StudentQuizResult = model.StudentQuizResult.Take(PageSize).ToList();
                model.QuestionOrder = page;
            }
            else
            {
                model.QuestionOrder = (PageSize * (page - 1)) + 1;
                model.StudentQuizResult = model.StudentQuizResult.Skip(PageSize * (page - 1)).Take(PageSize).ToList();

            }

        }

        private void LoadContentbyStudent(UserModel model, int StudentId)
        {
            int page = 1;
            model.StudentCont = ContentRepository.GetContentsbyStudentId(StudentId, accessAccount.PublisherUserId);



            UserListModel usr = usersRepository.GetUserbyId(StudentId);
            ViewBag.StudentId = StudentId;
            ViewBag.StudentName = usr.FirstName + " " + usr.LastName;
            SetMenuProperties("People", "", "Student");
            ViewBag.Parameter = ("&StudentId=" + StudentId);

            SetOverviewProperties("", "", "Contents(" + model.StudentCont.GroupBy(p => p.ContentId).Count() + ")", "");

            if (Request.QueryString["page"] != null) { Int32.TryParse(Request.QueryString["page"].ToString(), out page); }
            PopulatePagingData(model.StudentCont.Count(), "Publisher", "StudentOverview", page);

            if (page <= 1)
            {
                model.StudentCont = model.StudentCont.Take(PageSize).ToList();
            }
            else
            {
                model.StudentCont = model.StudentCont.Skip(PageSize * (page - 1)).Take(PageSize).ToList();

            }
            ViewBag.Title = "IntelSkill - AllContent";
        }

        private void LoadStudentbyContent(UserModel model, int ContentId, int ClassId)
        {
            int page = 1;

            model.OverViewContStudent = ContentRepository.GetStudentsbyContentId(ContentId);
            model.StudentCont = null;
            ViewBag.ClassId = Convert.ToInt32(ClassId);
            ViewBag.ContentId = ContentId;
            Classes cls = ContentRepository.GetClassbyId(Convert.ToInt32(ClassId));
            string ContentName = ContentRepository.GetContentTitle(ContentId);
            SetOverviewProperties(cls.ClassName, ContentName, "Students(" + model.OverViewContStudent.GroupBy(p => p.StudentForm.UserId).Count() + ")", "");
            SetMenuProperties("Dashboard", "", "");
            ViewBag.Parameter = ("&ContentId=" + ContentId + "" + "&ClassId=" + ClassId);

            if (Request.QueryString["page"] != null) { Int32.TryParse(Request.QueryString["page"].ToString(), out page); }
            PopulatePagingData(model.OverViewContStudent.Count(), "Publisher", "StudentOverview", page);

            if (page <= 1)
            {
                model.OverViewContStudent = model.OverViewContStudent.Take(PageSize).ToList();
            }
            else
            {
                model.OverViewContStudent = model.OverViewContStudent.Skip(PageSize * (page - 1)).Take(PageSize).ToList();

            }
            ViewBag.Title = "IntelSkill - Student";
        }
        #endregion

        #region Subscribers

        public void LoadSubsriberClasses(int StudentId, int page, UserModel model)
        {
            model.SubcribedClasses = usersRepository.SubcribedClasses(StudentId, accessAccount.PublisherUserId);

            UserListModel Usr = usersRepository.GetUserbyId(StudentId);
            ViewBag.StudentName = Usr.FirstName + " " + Usr.LastName;
            ViewBag.StudentId = Usr.UserId;

            PopulatePagingData(model.SubcribedClasses.Count(), "Publisher", "Subscribers", page);

            if (page <= 1)
            {
                model.SubcribedClasses = model.SubcribedClasses.Take(PageSize).ToList();
            }
            else
            {
                model.SubcribedClasses = model.SubcribedClasses.Skip(PageSize * (page - 1)).Take(PageSize).ToList();

            }
            ViewBag.Parameter = ("&StudentId=" + StudentId);
            ViewBag.Page = page;
        }
        public void LoadSubscribedContentResult(int StudentId, int ClassId, int page, UserModel model)
        {
            model.SubcribedContentResult = usersRepository.SubcribedContentResult(ClassId);

            UserListModel Usr = usersRepository.GetUserbyId(StudentId);
            ViewBag.StudentName = Usr.FirstName + " " + Usr.LastName;
            ViewBag.StudentId = Usr.UserId;

            Classes Cls = ContentRepository.GetClassbyId(ClassId);
            ViewBag.ClassId = Cls.ClassId;
            ViewBag.ClassName = Cls.ClassName;

            PopulatePagingData(model.SubcribedContentResult.Count(), "Publisher", "Subscribers", page);

            if (page <= 1)
            {
                model.SubcribedContentResult = model.SubcribedContentResult.Take(PageSize).ToList();
            }
            else
            {
                model.SubcribedContentResult = model.SubcribedContentResult.Skip(PageSize * (page - 1)).Take(PageSize).ToList();

            }
            ViewBag.Parameter = ("&StudentId=" + StudentId + "" + "&ClassId=" + ClassId);
        }
        public void LoadSubcriberQuizResult(int QuizReportId, int StudentId, int ClassId, int page, UserModel model)
        {
            model.StudentQuizResult = ContentRepository.StudentQuizResult(QuizReportId);

            UserListModel Usr = usersRepository.GetUserbyId(StudentId);
            ViewBag.StudentName = Usr.FirstName + " " + Usr.LastName;
            ViewBag.StudentId = Usr.UserId;

            Classes Cls = ContentRepository.GetClassbyId(ClassId);
            ViewBag.ClassId = Cls.ClassId;
            ViewBag.ClassName = Cls.ClassName;

            ViewBag.QuizReportId = QuizReportId;

            PopulatePagingData(model.StudentQuizResult.Count(), "Publisher", "Subscribers", page);
            ViewBag.Parameter = ("&QuizReportId=" + QuizReportId + "" + "&StudentId=" + StudentId + "" + "&ClassId=" + ClassId);
            if (page <= 1)
            {
                model.StudentQuizResult = model.StudentQuizResult.Take(PageSize).ToList();
                model.QuestionOrder = page;
            }
            else
            {
                model.QuestionOrder = (PageSize * (page - 1)) + 1;
                model.StudentQuizResult = model.StudentQuizResult.Skip(PageSize * (page - 1)).Take(PageSize).ToList();

            }

        }
        #endregion

        #endregion

        #region Class Tools Menu View's

        //GET:

        public ActionResult BuildClass(int id = 0)
        {
            loginActionResult = UserAuthentication();

            if (accessAccount.CurrentUserId == 0)
                return loginActionResult;

            var model = new BuildClassModel();

            BuildClassDataLoad(ref model, id);

            SetMenuProperties("Build", "Class Tools", "Build Class");

            return View(model);

        }

        public ActionResult ViewClass(int id = 0)
        {
            loginActionResult = UserAuthentication();

            if (accessAccount.CurrentUserId == 0)
                return loginActionResult;

            var model = new BuildClassModel();

            BuildClassDataLoad(ref  model, id);

            if (model.PageRights == null)
            {
                model.PageRights = GetPageRights();
            }

            SetMenuProperties("Build", "Class Tools", model.ClassModel.ClassName);

            return View(model);
        }



        public ActionResult PreviewClass()
        {
            loginActionResult = UserAuthentication();

            if (accessAccount.CurrentUserId == 0)
                return loginActionResult;

            var model = new BuildClassModel();

            if (Session["BuildClassPreview"] != null)
            {
                model = (BuildClassModel)Session["BuildClassPreview"];

                if (model.Assigned_ContentList != null)
                    model.ContentsViewModel = classRepository.GetContentList(model.PreviewContentIdList);
            }
            else
            {
                return RedirectToAction("PageExpired", "Class");
            }

            if (model.PageRights == null)
            {
                model.PageRights = GetPageRights();
            }

            SetMenuProperties("Build", "Class Tools", "Preview Class");

            return View(model);
        }

        public ActionResult AllClass()
        {
            loginActionResult = UserAuthentication();

            if (accessAccount.CurrentUserId == 0)
                return loginActionResult;
            else if (!string.IsNullOrEmpty(accessAccount.AccessCode))
            {
                return RedirectToAction("BuildClass", "Publisher");
            }
            var model = new AllClassModel();
            model.PageRights = GetPageRights();

            if (Request.QueryString["ToDate"] != null)
            {
                string FromDate = null;
                string ToDate = null;
                if (Request.QueryString["FromDate"] != null)
                {
                    FromDate = Request.QueryString["FromDate"].ToString();
                    ToDate = Request.QueryString["ToDate"].ToString();

                }
                else if (Request.QueryString["ToDate"] != "AllClasses")
                {
                    ToDate = Request.QueryString["ToDate"].ToString();
                }
                if (Request.QueryString["search"] != null)
                {
                    model.SearchBoxValue = Request.QueryString["search"].ToString();
                }
                LoadOverViewClass(model, FromDate, ToDate);

            }
            else
            {
                if (Request.QueryString["search"] != null)
                {
                    model.SearchBoxValue = Request.QueryString["search"].ToString();

                    ViewBag.Parameter = "&search=" + model.SearchBoxValue;
                }

                BindAllClassList(ref model);
                SetMenuProperties("Build", "Class Tools", "All Class");
            }



            return View(model);


        }

        //POST:

        [HttpPost]
        public ActionResult BuildClass(BuildClassModel model, FormCollection collection, string btnSubmit = "")
        {
            loginActionResult = UserAuthentication();

            if (accessAccount.CurrentUserId == 0)
                return loginActionResult;


            GetPropertyCollection(ref model, collection);
            int classId = 0;
            if (btnSubmit == "preview")
            {
                BuildClassPreview(model);
            }

            else if (btnSubmit == "save_draft")
            {
                classId = BuildClassSave(model, "save_draft");
            }
            else if (btnSubmit == "publish")
            {
                classId = BuildClassSave(model, "publish");
            }
            else if (btnSubmit == "send_request")
            {
                BuildClassSave(model, "send_request");
            }

            BuildClassDataLoad(ref model, classId);



            SetMenuProperties("Build", "Class Tools", "Build Class");

            model.ValidationMessage = pagemessage;
            model.ValidationMessageType = pagemessagetype;

            return View(model);
        }

        [HttpPost]
        public ActionResult AllClass(FormCollection formCollection, AllClassModel model, string btnSubmit = "")
        {
            loginActionResult = UserAuthentication();

            if (accessAccount.CurrentUserId == 0)
                return loginActionResult;

            model.PageRights = GetPageRights();

            if (btnSubmit == "apply")
            {
                AllClassBulkActionApply(ref model, formCollection);
            }
            if (!string.IsNullOrEmpty(model.ToDate))
            {

                if (model.ToDate == "AllClasses")
                {
                    model.ToDate = null;
                }
                LoadOverViewClass(model, model.FromDate, model.ToDate);
            }
            else
            {
                BindAllClassList(ref model);
                SetMenuProperties("Build", "Class Tools", "All Class");
            }

            model.ValidationMessage = pagemessage;
            model.ValidationMessageType = pagemessagetype;




            return View(model);
        }

        //FUNCTION:
        private SelectList BindAllClassBulkActionList(AllClassModel model)
        {
            List<SelectListItem> items = new List<SelectListItem>();

            items.Add(new SelectListItem() { Text = "Bulk Action", Value = "Bulk Action" });

            if (model.PageRights._delete_private_class == true || model.PageRights._delete_public_class == true)
            {
                items.Add(new SelectListItem() { Text = "Delete", Value = "Delete" });
            }
            var typeList = new SelectList(items.ToList(), "Text", "Value");

            return typeList;

        }

        private void BindAllClassList(ref AllClassModel model)
        {
            string searchquery = string.Empty;
            int page = 1;

            if (Request.QueryString["page"] != null) { Int32.TryParse(Request.QueryString["page"].ToString(), out page); }

            if (!string.IsNullOrEmpty(model.SearchBoxValue))
            {
                searchquery = model.SearchBoxValue;
                ViewBag.Parameter = "&search=" + model.SearchBoxValue;
            }

            model.BulkActionList = BindAllClassBulkActionList(model);

            model.CurrentPageList = classRepository.GetAllClassList(accessAccount.PublisherUserId, searchquery, model.PageRights);

            model.AllClassCount = model.CurrentPageList.Count;

            model.AllDraftCount = model.CurrentPageList.Where(x => x.ClassStatus == Convert.ToInt32(ClassStatus.Draft)).ToList().Count;

            model.AllPublishedCount = model.CurrentPageList.Where(x => x.ClassStatus == Convert.ToInt32(ClassStatus.Published)).ToList().Count;

            model.AllYetToPublishCount = model.CurrentPageList.Where(x => x.ClassStatus == Convert.ToInt32(ClassStatus.YetToPublish)).ToList().Count;

            //if (page <= 1)
            //    model.CurrentPageList = model.CurrentPageList.Take(PageSize).ToList();
            //else
            //    model.CurrentPageList = model.CurrentPageList.Skip(PageSize * (page - 1)).Take(PageSize).ToList();

            //PopulatePagingData(model.AllClassCount, "Publisher", "AllClass", page);
        }

        private void AllClassBulkActionApply(ref AllClassModel model, FormCollection formCollection)
        {
            string bulkAction = formCollection.GetValues("Bulk Action") != null ? formCollection.GetValues("Bulk Action")[0].ToString().ToLower() : string.Empty;

            int classCount = formCollection.GetValues("class_checkbox") != null ? formCollection.GetValues("class_checkbox").Count() : 0;



            switch (bulkAction)
            {

                case "delete":
                    {
                        for (int i = 0; i < classCount; i++)
                        {
                            int classId = 0;

                            Int32.TryParse(formCollection.GetValues("class_checkbox")[i].ToString(), out classId);

                            classRepository.DeleteClass(classId);

                            pagemessage = "Changes has been made";
                            pagemessagetype = "success";
                        }

                        break;
                    }
                default:
                    {
                        pagemessage = "Action not selected.";
                        pagemessagetype = "warning";
                        break;
                    }

            }

            if (classCount == 0)
            {
                pagemessage = "No Class selected";
                pagemessagetype = "warning";

            }


        }

        private void PopulatePagingData(int totalRows, string controllerName, string actionName, int? page = 1)
        {
            int pageCount = 0;
            if (ViewBag.ContentPreview == null)
            {
                double dblPageCount = (double)((decimal)totalRows / PageSize);
                pageCount = (int)Math.Ceiling(dblPageCount);

            }
            else
            {
                pageCount = totalRows;
            }

            ViewBag.CurrentPage = page;
            ViewBag.PageCount = pageCount;
            ViewBag.PagingNavigation = "/" + controllerName + "/" + actionName;
            ViewBag.EnablePaging = pageCount > 1;
        }

        private int BuildClassSave(BuildClassModel model, string storeType)
        {
            if (ValidateSaveClass(model, storeType))
            {

                model.ClassModel.PublisherUserId = accessAccount.PublisherUserId;

                #region On Publish Settings

                if (storeType == "publish")
                {
                    model.ClassModel.PublishedUserId = accessAccount.CurrentUserId;
                    model.ClassModel.PublishedDate = ConvertDatePickerValueToDate(Request.Form["ClassModel.PublishedDate"]);

                    if ((model.ClassModel.PublishedDate ?? DateTime.Now).Date <= System.DateTime.Now.Date)
                        model.ClassModel.ClassStatus = Convert.ToInt32(ClassStatus.Published);
                    else
                        model.ClassModel.ClassStatus = Convert.ToInt32(ClassStatus.YetToPublish);
                }
                else if (storeType == "save_draft")
                {
                    model.ClassModel.ClassStatus = Convert.ToInt32(ClassStatus.Draft);
                    model.ClassModel.PublishedUserId = null;
                    model.ClassModel.PublishedDate = null;
                }

                #endregion

                #region Create & Update settings.
                if (model.ClassModel.ClassId == 0)
                {
                    model.ClassModel.CreatedDate = System.DateTime.Now;
                    model.ClassModel.CreatedUserId = accessAccount.CurrentUserId;
                }
                else
                {
                    model.ClassModel.ModifiedDate = System.DateTime.Now;
                    model.ClassModel.ModifiedUserId = accessAccount.CurrentUserId;
                }
                #endregion

                #region Send Request Settings

                if (storeType == "send_request" && !string.IsNullOrEmpty(model.RequestMail))
                {
                    stringCryptography = new StringCryptography();

                    model.ClassModel.AccessCode = stringCryptography.GeneratePassword(25);
                    stringCryptography = new StringCryptography();
                    model.ClassModel.AccessCodePwd = stringCryptography.GeneratePassword(8);

                    model.ClassModel.AssignedUserId = classRepository.GetUserIdByMail(model.RequestMail);
                    model.ClassModel.AssignedUserAttempts = 0;
                    model.ClassModel.AssignedUserRequestDate = System.DateTime.Now;
                    model.ClassModel.ClassStatus = Convert.ToInt32(ClassStatus.Draft);
                }
                #endregion

                int returnClassId = classRepository.SaveClass(model);

                if (returnClassId > 0)
                {
                    //Sending mail to requested user.
                    pagemessage = model.ClassModel.ClassName + " saved successfully.";
                    pagemessagetype = "success";

                    SendMailIfRequested(model, storeType);
                    ModelState.Clear();


                }

                return returnClassId;
            }
            else
            {
                pagemessagetype = "error";
            }

            return 0;
        }

        private void SendMailIfRequested(BuildClassModel model, string storeType)
        {
            //Send mail to requested user.

            if (model.IsRequestMailEnabled && storeType == "send_request")
            {
                string port = Request.Url.Host.Contains("localhost") ? ":" + Request.Url.Port.ToString() + "/" : string.Empty;
                string imageHost = Request.Url.Host.Contains("localhost") ? "http://intelskill.com/" : Request.Url.Host + "/";
                string hostpath = (Request.Url.Host + port).EndsWith("/") ? Request.Url.Host + port : Request.Url.Host + "/";

                string BodyContent = @"<br/> <br/>You have been requested to compose contents for " + accessAccount.DisplayName + @"<br/> <br/> 
                
                                    Please <a href='" + hostpath + "Access/?accessRequest=" + model.ClassModel.AccessCode + @"'>click here</a> to access the requested class details.<br/>Access Password : " + model.ClassModel.AccessCodePwd + @"<br/> 
                
                                    If any problem to view the link please copy & paste the below url to continue.<br/> 
                                    " + hostpath + "Access/?accessRequest=" + model.ClassModel.AccessCode + @"<br/><br/> ";



                string appPath = Server.MapPath("/Content/newsletter-template/classsendrequest.htm");
                string fileData = System.IO.File.ReadAllText(appPath);

                /* Replace those word.
                 @CssHostLink          : newsletters.css
                 @TodayDate            : March 28, 2013
                 @Receiver             : Customer
                 @BuildClassHostLink   : 
                 @BuildContentHostLink : 
                 @BuildQuizHostLink    :
                 @ContactUsHostLink    :  
                 @ClassImageLocation
              */
                string output = Regex.Replace(fileData, "@TodayDate", System.DateTime.Now.ToString("MMMM dd, yyyy"));
                output = Regex.Replace(output, "@Receiver", model.RequestMail);
                output = Regex.Replace(output, "@BuildClassHostLink", hostpath + "BuilClass");
                output = Regex.Replace(output, "@BuildContentHostLink", hostpath + "BuildContent");
                output = Regex.Replace(output, "@BuildQuizHostLink", hostpath + "BuildQuiz");
                output = Regex.Replace(output, "@ContactUsHostLink", hostpath + "ContactUs");
                output = Regex.Replace(output, "@BodyContent", BodyContent);
                output = Regex.Replace(output, "@ClassImageLocation", imageHost + "Content");

                var fromAddress = new MailAddress(accessAccount.DisplayName, "IntelSkill");
                var toAddress = new MailAddress(model.RequestMail);
                string subject = accessAccount.DisplayName + " has sent request from IntelSkill.Com";
                string body = output;

                //SmtpClient smtp = new SmtpClient
                //{
                //    Host = "smtp.gmail.com",
                //    Port = 587,
                //    EnableSsl = true,
                //    DeliveryMethod = SmtpDeliveryMethod.Network,
                //    UseDefaultCredentials = false,
                //    Credentials = new NetworkCredential("clearlead.devlopers.demo@gmail.com", "Passw@rd"),
                //    Timeout = 3000
                //};


                SmtpClient smtp = new SmtpClient();

                using (var message = new MailMessage(fromAddress, toAddress)
                {
                    Subject = subject,
                    Body = body,
                    IsBodyHtml = true,



                })
                    try
                    {

                        smtp.Send(message);
                        pagemessage = "Request has been sent to " + model.RequestMail;
                    }
                    catch (Exception ex) { pagemessage = ex.ToString(); pagemessagetype = "error"; }

            }
        }

        private void BuildClassPreview(BuildClassModel model)
        {
            if (ValidateSaveClass(model, ""))
            {
                ViewBag.IsPreview = true;
                Session["BuildClassPreview"] = null;

                Session["BuildClassPreview"] = model;
            }
            else
            {
                pagemessagetype = "error";
            }
        }

        private void BuildClassDataLoad(ref BuildClassModel model, int classId)
        {

            if (model.ClassModel == null)
            {
                model.ClassModel = classRepository.GetMyClass(classId);

                var usermodel = usersRepository.GetUserbyId(model.ClassModel.AssignedUserId ?? 0);
                if (usermodel != null)
                    model.RequestMail = usermodel.Email;
            }

            if (ViewBag.IsPreview ?? false)
            {
                model.Assigned_ContentList = classRepository.GetContentList(model.PreviewContentIdList);
            }
            else
            {
                model.Assigned_ContentList = classRepository.GetAssignedContents(classId);
            }
            model.Assigned_ContentList_Count = model.Assigned_ContentList.Count;

            model.Assigned_StudentList = classRepository.GetAssignedStudents(classId);

            model.Assigned_StudentList_Count = model.Assigned_StudentList.Count;


            if (model.Assigned_ContentList != null)
            {
                model.Available_ContentList = classRepository.GetMyContents(accessAccount.PublisherUserId).Except(model.Assigned_ContentList).ToList();
            }

            if (model.Assigned_StudentList != null)
            {
                model.Available_StudentList = classRepository.GetMyStudents(accessAccount.PublisherUserId).Except(model.Assigned_StudentList).ToList();
            }

            if (model.Removed_StudentList == null)
            {
                model.Removed_StudentList = new List<Users>();
            }

            if (model.PageRights == null)
            {
                model.PageRights = GetPageRights();
            }

            if (model.ClassModel.ClassType == null)
            {
                if (model.ClassModel.ClassId == 0)
                {
                    if (model.PageRights._add_new_private_class)
                        model.ClassModel.ClassType = false;
                    else
                        model.ClassModel.ClassType = true;
                }
            }
            if (model.ClassModel.IsProfitable == null)
            {
                model.ClassModel.IsProfitable = false;
            }

            model.IsRequestMailEnabled = !string.IsNullOrEmpty(model.RequestMail);
        }

        private void GetPropertyCollection(ref BuildClassModel model, FormCollection collection)
        {
            model.Available_ContentList = new List<Contents>();
            model.Assigned_ContentList = new List<Contents>();
            model.Available_StudentList = new List<Users>();
            model.Assigned_StudentList = new List<Users>();

            model.PreviewContentIdList = new List<int>();

            int Available_ContentList_Count = (collection.GetValues("Available_ContentList.ContentId") != null) ? collection.GetValues("Available_ContentList.ContentId").Count() : 0;
            int Assigned_ContentList_Count = (collection.GetValues("Assigned_ContentList.ContentId") != null) ? collection.GetValues("Assigned_ContentList.ContentId").Count() : 0;
            int Available_StudentList_Count = (collection.GetValues("Available_StudentList.UserId") != null) ? collection.GetValues("Available_StudentList.UserId").Count() : 0;
            int Assigned_StudentList_Count = (collection.GetValues("Assigned_StudentList.UserId") != null) ? collection.GetValues("Assigned_StudentList.UserId").Count() : 0;

            for (int i = 0; i < Available_ContentList_Count; i++)
            {
                model.Available_ContentList.Add(new Contents() { ContentId = Int32.Parse(collection.GetValues("Available_ContentList.ContentId")[i].ToString()) });
            }

            for (int j = 0; j < Assigned_ContentList_Count; j++)
            {
                model.Assigned_ContentList.Add(new Contents() { ContentId = Int32.Parse(collection.GetValues("Assigned_ContentList.ContentId")[j].ToString()) });
                model.PreviewContentIdList.Add(Int32.Parse(collection.GetValues("Assigned_ContentList.ContentId")[j].ToString()));
            }

            for (int k = 0; k < Available_StudentList_Count; k++)
            {
                model.Available_StudentList.Add(new Users() { UserId = Int32.Parse(collection.GetValues("Available_StudentList.UserId")[k].ToString()) });
            }

            for (int l = 0; l < Assigned_StudentList_Count; l++)
            {
                model.Assigned_StudentList.Add(new Users() { UserId = Int32.Parse(collection.GetValues("Assigned_StudentList.UserId")[l].ToString()) });
            }

        }

        private DateTime? ConvertDatePickerValueToDate(string published_date)
        {
            DateTime publishedon;
            published_date = published_date.Replace(',', ' ').Trim();

            if (published_date != null)
            {
                if (published_date.ToLower() == "today")
                    publishedon = System.DateTime.Now.Date;
                else
                    DateTime.TryParse(published_date, out publishedon);

                return publishedon;
            }
            return null;
        }

        private bool ValidateSaveClass(BuildClassModel model, string storeType)
        {
            bool retval = true;

            if (string.IsNullOrEmpty(model.ClassModel.ClassName))
            { pagemessage += "* Class name is required.<br/>"; retval = false; }

            if (!string.IsNullOrEmpty(model.ClassModel.ClassDescription))
            {

                if (model.ClassModel.ClassDescription.Length > 500)
                { pagemessage += "* Class description characters can be 500<br/>"; retval = false; }

            }

            if ((model.ClassModel.IsProfitable ?? false) == true && (model.ClassModel.ProfitAmount ?? 0) == 0)
            { pagemessage += "* Please enter profit amount.<br/>"; retval = false; }

            if (storeType == "send_request")
            {
                if (string.IsNullOrEmpty(model.RequestMail))
                { pagemessage += "* Please enter requesting email.<br/>"; retval = false; }
                else if (!IsValidEmail(model.RequestMail))
                {
                    pagemessage += "* Please enter valid email.<br/>"; retval = false;
                }
                else
                {
                    bool tempval = classRepository.CheckValidUser(model.RequestMail, accessAccount.CurrentUserId, accessAccount.PublisherUserId);

                    if (!tempval)
                    { pagemessage += "* Send request mail address (" + model.RequestMail + ") is not valid one.<br/>"; retval = false; }

                }
            }

            return retval;
        }

        private void LoadOverViewClass(AllClassModel model, string FromDate, string ToDate)
        {
            string searchquery = string.Empty;
            int page = 1;

            if (!string.IsNullOrEmpty(model.SearchBoxValue))
            {
                searchquery = model.SearchBoxValue.Trim();

            }
            else
            {
                pagemessage = "No search text given";
                pagemessagetype = "warning";
            }


            model.OverViewClass = ContentRepository.GetOverviewClasses(accessAccount.PublisherUserId, FromDate, ToDate);

            if (model.PageRights._view_all_public_class == true && model.PageRights._view_all_private_class == false)
            {
                model.OverViewClass = (from PublicClass in model.OverViewClass where PublicClass.ClassType == true select PublicClass).ToList();
            }
            else if (model.PageRights._view_all_public_class == false && model.PageRights._view_all_private_class == true)
            {
                model.OverViewClass = (from PublicClass in model.OverViewClass where PublicClass.ClassType == false select PublicClass).ToList();
            }

            if (searchquery.Length > 0)
            {
                model.OverViewClass = (from c in model.OverViewClass where c.ClassName.ToLower().Contains(searchquery.ToLower()) select c).ToList();
            }
            model.AllClassCount = model.OverViewClass.Count;

            model.AllDraftCount = model.OverViewClass.Where(x => x.ClassStatus == Convert.ToInt32(ClassStatus.Draft)).ToList().Count;

            model.AllPublishedCount = model.OverViewClass.Where(x => x.ClassStatus == Convert.ToInt32(ClassStatus.Published)).ToList().Count;

            model.AllYetToPublishCount = model.OverViewClass.Where(x => x.ClassStatus == Convert.ToInt32(ClassStatus.YetToPublish)).ToList().Count;

            if (Request.QueryString["page"] != null) { Int32.TryParse(Request.QueryString["page"].ToString(), out page); }

            //if (page <= 1)
            //    model.OverViewClass = model.OverViewClass.Take(PageSize).ToList();
            //else
            //    model.OverViewClass = model.OverViewClass.Skip(PageSize * (page - 1)).Take(PageSize).ToList();

            //PopulatePagingData(model.AllClassCount, "Publisher", "AllClass", page);

            if (!string.IsNullOrEmpty(FromDate) || !string.IsNullOrEmpty(ToDate))
            {
                ViewBag.Parameter = ((!string.IsNullOrEmpty(FromDate) ? "&FromDate=" + FromDate : "") + "" + (!string.IsNullOrEmpty(ToDate) ? "&ToDate=" + ToDate : "")
                                         + "" + (!string.IsNullOrEmpty(model.SearchBoxValue) ? "&search=" + model.SearchBoxValue : ""));
                model.FromDate = FromDate;
                model.ToDate = ToDate;
            }
            else
            {
                ViewBag.Parameter = "&ToDate=AllClasses" + "" + (!string.IsNullOrEmpty(model.SearchBoxValue) ? "&search=" + model.SearchBoxValue : "");
                model.ToDate = "AllClasses";
            }
            SetMenuProperties("Dashboard", "", "");
            ViewBag.Title = "IntelSkill - AllClass";



        }

        #endregion

        #region Questions View's

        public ActionResult AddQuestion(int Id = 0)
        {
            loginActionResult = UserAuthentication();

            if (accessAccount.CurrentUserId == 0)
                return loginActionResult;

            var model = new ViewQuestionModel();
            model.PageRights = GetPageRights();
            model.SelectedQuizId = Id;

            ViewQuestionDataLoad(ref model);

            model.ValidationMessage = pagemessage;
            model.ValidationMessageType = pagemessagetype;

            SetMenuProperties("Build", "Quiz Tools", "Add Question");

            return View("ViewQuestions", model);

        }

        public ActionResult ViewQuestions(int Id = 0)
        {
            loginActionResult = UserAuthentication();

            if (accessAccount.CurrentUserId == 0)
                return loginActionResult;

            var model = new ViewQuestionModel();

            model.PageRights = GetPageRights();

            model.SelectedQuizId = Id;

            ViewQuestionDataLoad(ref model);

            model.ValidationMessage = pagemessage;
            model.ValidationMessageType = pagemessagetype;



            SetMenuProperties("Build", "Quiz Tools", "All Questions");

            return View(model);

        }

        [HttpPost]
        public ActionResult ViewQuestions(ViewQuestionModel model, FormCollection formCollection, string btnSubmit = "")
        {
            loginActionResult = UserAuthentication();

            if (accessAccount.CurrentUserId == 0)
                return loginActionResult;
            model.PageRights = GetPageRights();
            if (btnSubmit == "apply")
            {
                QuestionListBulkActionApply(ref model, formCollection);
            }
            else if (btnSubmit.ToLower() == "savequestion")
            {
                SaveQuestion(ref model, formCollection);
            }

            ViewQuestionDataLoad(ref model);

            model.ValidationMessage = pagemessage;
            model.ValidationMessageType = pagemessagetype;



            SetMenuProperties("Build", "Quiz Tools", "All Questions");

            return View(model);

        }


        public ActionResult Question(int id = 0)
        {
            loginActionResult = UserAuthentication();

            if (accessAccount.CurrentUserId == 0)
                return loginActionResult;

            var model = new ViewQuestionModel();
            model.PageRights = GetPageRights();
            ViewQuestionDataLoad(ref model, id);

            model.ValidationMessage = pagemessage;
            model.ValidationMessageType = pagemessagetype;



            SetMenuProperties("Build", "Quiz Tools", "Edit Questions");

            return View("ViewQuestions", model);

        }

        [HttpPost]
        public ActionResult Question(ViewQuestionModel model, FormCollection formCollection, string btnSubmit = "")
        {
            loginActionResult = UserAuthentication();

            if (accessAccount.CurrentUserId == 0)
                return loginActionResult;
            model.PageRights = GetPageRights();
            if (btnSubmit == "apply")
            {
                QuestionListBulkActionApply(ref model, formCollection);
            }
            else if (btnSubmit.ToLower() == "savequestion")
            {
                if (SaveQuestion(ref model, formCollection))
                    return RedirectToAction("ViewQuestions", "Publisher");
            }

            ViewQuestionDataLoad(ref model);

            model.ValidationMessage = pagemessage;
            model.ValidationMessageType = pagemessagetype;



            if (model.ValidationMessageType == "success")
            {
                SetMenuProperties("Build", "Quiz Tools", "All Questions");
            }
            else { SetMenuProperties("Build", "Quiz Tools", "Edit Questions"); }

            return View("ViewQuestions", model);

        }

        private bool SaveQuestion(ref ViewQuestionModel model, FormCollection formCollection)
        {
            model.QuestionModel.Question.QuizId = model.SelectedQuizId;

            InitializeAnswers(ref model, formCollection);

            if (ValidateQuestion(model))
            {
                if (model.QuestionModel.Question.QuestTypeId == 4)
                {
                    model.QuestionModel.AnswerList = new List<QuestionAnswers>();

                    model.QuestionModel.AnswerList.AddRange(model.QuestionModel.MatchAList);

                    model.QuestionModel.AnswerList.AddRange(model.QuestionModel.MatchBList);
                }

                if (questionRepository.SaveQuestion(model, accessAccount.CurrentUserId))
                {
                    ModelState.Clear();
                    model.QuestionModel.Question = new Questions();
                    model.QuestionModel.AnswerList = new List<QuestionAnswers>();
                    model.QuestionModel.ImageList = new List<QuestionImages>();
                    model.QuestionModel.MatchAList = new List<QuestionAnswers>();
                    model.QuestionModel.MatchBList = new List<QuestionAnswers>();

                    pagemessage = "Question has been saved.";
                    pagemessagetype = "success";
                    return true;
                }
                else
                {
                    pagemessage = "Question failed to save.";
                    pagemessagetype = "error";
                }
            }
            else
            { pagemessagetype = "error"; }
            return false;
        }

        private void InitializeAnswers(ref ViewQuestionModel model, FormCollection formCollection)
        {
            model.QuestionModel.AnswerList = new List<QuestionAnswers>();

            model.QuestionModel.ImageList = new List<QuestionImages>();

            if (model.QuestionModel.Question.QuestTypeId == 1)
            {

                if (formCollection.GetValues("multiple_Item.AnswerValue") != null)
                {
                    int totalAnswers = formCollection.GetValues("multiple_Item.AnswerValue").Count();

                    string _correctAnswer = string.Empty;

                    if (formCollection.GetValues("multiple_Item.CorrectAnswer") != null)
                    {
                        if (formCollection.GetValues("multiple_Item.CorrectAnswer").Count() > 0)
                        {
                            _correctAnswer = formCollection.GetValues("multiple_Item.CorrectAnswer")[0].ToString();
                        }
                    }

                    for (int i = 0; i < totalAnswers; i++)
                    {
                        model.QuestionModel.AnswerList.Add(new QuestionAnswers() { AnswerId = Convert.ToInt32(formCollection.GetValues("multiple_Item.AnswerId")[i].ToString()), AnswerValue = formCollection.GetValues("multiple_Item.AnswerValue")[i].ToString(), CorrectAnswer = (_correctAnswer == formCollection.GetValues("multiple_Item.AnswerValue")[i].ToString()) });
                    }
                }

            }
            else if (model.QuestionModel.Question.QuestTypeId == 2)
            {
                if (formCollection.GetValues("fillintheblank") != null)
                {
                    int totalAnswers = formCollection.GetValues("fillintheblank").Count();

                    for (int i = 0; i < totalAnswers; i++)
                    {
                        model.QuestionModel.AnswerList.Add(new QuestionAnswers() { AnswerValue = formCollection.GetValues("fillintheblank")[i].ToString(), CorrectAnswer = true });
                    }
                }

            }

            else if (model.QuestionModel.Question.QuestTypeId == 3)
            {
                if (formCollection.GetValues("trueoffalse_radio") != null)
                {
                    int totalAnswers = formCollection.GetValues("trueoffalse_radio").Count();

                    for (int i = 0; i < totalAnswers; i++)
                    {
                        model.QuestionModel.AnswerList.Add(new QuestionAnswers() { AnswerValue = formCollection.GetValues("trueoffalse_radio")[i].ToString(), CorrectAnswer = true });
                    }
                }

            }

            else if (model.QuestionModel.Question.QuestTypeId == 4)
            {

                model.QuestionModel.MatchAList = new List<QuestionAnswers>();

                model.QuestionModel.MatchBList = new List<QuestionAnswers>();

                model.QuestionModel.AnswerList = new List<QuestionAnswers>();

                List<string> sorted_answer = new List<string>();


                if (formCollection.GetValues("MatchedAnswer_Value") != null)
                {
                    model.QuestionModel.AnswerList = new List<QuestionAnswers>();

                    int matchedanswer = formCollection.GetValues("MatchedAnswer_Value").Count();

                    for (int i = 0; i < matchedanswer; i++)
                    {
                        sorted_answer.Add(formCollection.GetValues("MatchedAnswer_Value")[i].ToString());

                        model.QuestionModel.AnswerList.Add(new QuestionAnswers() { AnswerId = Convert.ToInt32(formCollection.GetValues("MatchedAnswer_valueId")[i].ToString()), AnswerValue = formCollection.GetValues("MatchedAnswer_Value")[i].ToString(), OrderBy = (i + 1).ToString(), AnswerGroup = "Answer" });
                    }
                }

                if (formCollection.GetValues("matchthefollowing_A_value") != null)
                {
                    int totalAnswersGroupA = formCollection.GetValues("matchthefollowing_A_value").Count();

                    for (int i = 0; i < totalAnswersGroupA; i++)
                    {
                        model.QuestionModel.MatchAList.Add(new QuestionAnswers() { AnswerId = Convert.ToInt32(formCollection.GetValues("matchthefollowing_A_valueId")[i].ToString()), AnswerValue = formCollection.GetValues("matchthefollowing_A_value")[i].ToString(), OrderBy = (i + 1).ToString(), AnswerGroup = "GroupA" });
                    }
                }
                if (formCollection.GetValues("matchthefollowing_B_value") != null)
                {
                    int totalAnswersGroupB = formCollection.GetValues("matchthefollowing_B_value").Count();

                    for (int i = 0; i < totalAnswersGroupB; i++)
                    {
                        findName = formCollection.GetValues("matchthefollowing_B_value")[i].ToString();
                        model.QuestionModel.MatchBList.Add(new QuestionAnswers() { AnswerId = Convert.ToInt32(formCollection.GetValues("matchthefollowing_B_valueId")[i].ToString()), AnswerValue = formCollection.GetValues("matchthefollowing_B_value")[i].ToString(), OrderBy = (sorted_answer.FindIndex(isName) + 1).ToString(), AnswerGroup = "GroupB" });
                    }
                }

            }

            if (formCollection.GetValues("ImageValue") != null)
            {
                int imageitem = formCollection.GetValues("ImageValue").Count();

                for (int i = 0; i < imageitem; i++)
                {
                    model.QuestionModel.ImageList.Add(new QuestionImages() { ImageId = Convert.ToInt32(formCollection.GetValues("ImageValue")[i].ToString()), QuestionId = model.QuestionModel.Question.QuestionId });
                }
            }

            if (ValidateImage())
            {
                if (Request.Files.Count > 0)
                {
                    if (Request.Files[0].ContentLength > 0)
                    {

                        string localfolder = Server.MapPath(@"\Uploads\QuestionImageFolder");  

                        string file_extension = Path.GetExtension(Request.Files["questionimages"].FileName);
                        string filename = Request.Files["questionimages"].FileName;
                        string randomFileName = Guid.NewGuid().ToString() + file_extension;
                        string absolute_path = localfolder +@"\"+ randomFileName;

                        Request.Files["questionimages"].SaveAs(absolute_path);

                        model.QuestionModel.ImageList.Add(new QuestionImages() { ImageUrl = randomFileName, ImageName = filename });
                    }
                }
            }
        }

        private string findName = "";

        private bool isName(string name)
        {
            return (name == findName);
        }

        private bool ValidateQuestion(ViewQuestionModel model)
        {
            bool retval = true;
            if (model.QuestionModel.Question.QuizId <= 0)
            { pagemessage += "* Quiz name is not selected.<br/>"; retval = false; }

            if (string.IsNullOrEmpty(model.QuestionModel.Question.QuestionDescription))
            { pagemessage += "* Question is required<br/>"; retval = false; }

            if ((model.QuestionModel.Question.QuestTypeId ?? 0) <= 0)
            { pagemessage += "* Question type is not selected.<br/>"; retval = false; }

            if ((model.QuestionModel.Question.QuestTypeId ?? 0) > 0)
            {
                if (model.QuestionModel.Question.QuestTypeId == 1)
                {
                    if (model.QuestionModel.AnswerList.Count == 0)
                    {
                        pagemessage += "* Please add multiple choice answers.<br/>"; retval = false;
                    }
                    else
                    {
                        var item = (from a in model.QuestionModel.AnswerList where a.CorrectAnswer ?? false == true select a).ToList();

                        if (item.Count == 0)
                        { pagemessage += "* Please select correct answer for multiple choice answers.<br/>"; retval = false; }

                    }
                }

                if (model.QuestionModel.Question.QuestTypeId == 2)
                {
                    if (model.QuestionModel.AnswerList.Count == 0)
                    {
                        pagemessage += "* Please add fill in the blank answer.<br/>"; retval = false;
                    }
                }

                if (model.QuestionModel.Question.QuestTypeId == 3)
                {
                    if (model.QuestionModel.AnswerList.Count == 0)
                    {
                        pagemessage += "* Please select true or false answer.<br/>"; retval = false;
                    }
                }

                if (model.QuestionModel.Question.QuestTypeId == 4)
                {
                    if (model.QuestionModel.MatchAList.Count < model.QuestionModel.MatchBList.Count)
                    { pagemessage += "* Match A column must be equal to Match B.<br/>"; retval = false; }

                    if (model.QuestionModel.MatchAList.Count > model.QuestionModel.MatchBList.Count)
                    { pagemessage += "* Match B column must be equal to Match A.<br/>"; retval = false; }

                }
            }


            if (Request.Files.Count > 0)
            {
                string[] acceptedTypes = new string[] { "image/png", "image/bmp", "image/jpeg", "image/gif" };

                if (Request.Files["questionimages"].ContentLength > 0)
                {
                    if (!acceptedTypes.Contains(Request.Files["questionimages"].ContentType))
                    {
                        pagemessage += "* image must be in png, bmp, jpeg, gif format.<br/>"; retval = false;
                    }
                    else
                    {
                        if (Request.Files["questionimages"].ContentLength > 50 * 1024)
                        {
                            System.Drawing.Image image = System.Drawing.Image.FromStream(Request.Files["questionimages"].InputStream, true, true);

                            if (image.Width > 448 && image.Height > 261)
                            {
                                pagemessage += "* Maximum dimension of image cannot be more than 448x261.<br/>"; retval = false;
                            }

                        }

                    }
                }
            }

            return retval;
        }

        private bool ValidateImage()
        {
            bool retval = true;

            if (Request.Files.Count > 0)
            {
                string[] acceptedTypes = new string[] { "image/png", "image/bmp", "image/jpeg", "image/gif" };

                if (Request.Files["questionimages"].ContentLength > 0)
                {
                    if (!acceptedTypes.Contains(Request.Files["questionimages"].ContentType))
                    {
                        retval = false;
                    }
                    else
                    {
                        if (Request.Files["questionimages"].ContentLength > 50 * 1024)
                        {
                            System.Drawing.Image image = System.Drawing.Image.FromStream(Request.Files["questionimages"].InputStream, true, true);

                            if (image.Width > 448 && image.Height > 261)
                            {
                                retval = false;
                            }

                        }

                    }
                }
            }

            return retval;

        }

        private void QuestionListBulkActionApply(ref ViewQuestionModel model, FormCollection formCollection)
        {
            string bulkAction = formCollection.GetValues("Bulk Action") != null ? formCollection.GetValues("Bulk Action")[0].ToString().ToLower() : string.Empty;

            int questionCount = formCollection.GetValues("question_checkbox") != null ? formCollection.GetValues("question_checkbox").Count() : 0;


            switch (bulkAction)
            {

                case "delete":
                    {
                        for (int i = 0; i < questionCount; i++)
                        {
                            int questionId = 0;

                            Int32.TryParse(formCollection.GetValues("question_checkbox")[i].ToString(), out questionId);

                            questionRepository.DeleteQuestion(questionId);

                            pagemessage = "Changes has been made";
                            pagemessagetype = "success";

                        }

                        break;
                    }
                case "active":
                    {
                        for (int i = 0; i < questionCount; i++)
                        {
                            int questionId = 0;

                            Int32.TryParse(formCollection.GetValues("question_checkbox")[i].ToString(), out questionId);

                            questionRepository.UpdateStatus(questionId, 1);

                            pagemessage = "Changes has been made";
                            pagemessagetype = "success";

                        }

                        break;
                    }
                case "in-active":
                    {
                        for (int i = 0; i < questionCount; i++)
                        {
                            int questionId = 0;

                            Int32.TryParse(formCollection.GetValues("question_checkbox")[i].ToString(), out questionId);

                            questionRepository.UpdateStatus(questionId, 0);

                            pagemessage = "Changes has been made";
                            pagemessagetype = "success";

                        }

                        break;
                    }
                default:
                    {
                        pagemessage = "Action not selected.";
                        pagemessagetype = "warning";
                        break;
                    }

            }

            if (questionCount == 0)
            {
                pagemessage = "No Questions selected";
                pagemessagetype = "warning";

            }
        }

        private void ViewQuestionDataLoad(ref ViewQuestionModel model, int questionId = 0)
        {
            model.BulkActionList = BindAllQuestionBulkActionList(model);

            model.PendingQuestionMessage = string.Empty;

            if (model.QuizList == null) { model.QuizList = questionRepository.GetMyQuizList(accessAccount.PublisherUserId); }

            if (model.QuestionModel == null) { model.QuestionModel = new QuestionModel(); }

            if (model.QuestionModel.Question == null)
            {
                model.QuestionModel.Question = new Questions();
                model.QuestionModel.Question.QuestionId = questionId;
                model.QuestionModel = questionRepository.GetQuestionDetail(model.QuestionModel.Question.QuestionId);
                model.SelectedQuizId = model.QuestionModel.Question.QuizId ?? model.SelectedQuizId;
            }

            if (model.QuestionModel.AnswerList == null) { model.QuestionModel.AnswerList = new List<QuestionAnswers>(); }
            if (model.QuestionModel.MatchAList == null) { model.QuestionModel.MatchAList = new List<QuestionAnswers>(); }
            if (model.QuestionModel.MatchBList == null) { model.QuestionModel.MatchBList = new List<QuestionAnswers>(); }
            if (model.QuestionModel.ImageList == null) { model.QuestionModel.ImageList = new List<QuestionImages>(); }


            model.CurrentPageList = (model.SelectedQuizId > 0) ? questionRepository.GetMyQuestions(accessAccount.PublisherUserId, model.SelectedQuizId) : questionRepository.GetMyQuestions(accessAccount.PublisherUserId);


            model.QuestionModel.TypeList = questionRepository.GetQuestionTypes();

            int selectedquizId = model.SelectedQuizId;

            int question_count = model.CurrentPageList.Count;

            var selected_quiz = questionRepository.GetQuizDetail(model.SelectedQuizId);

            if (selected_quiz != null)
            {
                if (question_count < selected_quiz.QuestionAmount)
                {
                    model.PendingQuestionMessage = "This Quiz has " + (selected_quiz.QuestionAmount - question_count) + " Question(s) pending. ";
                }
                else { model.PendingMessageStyle = string.Empty; }
            }
            else { model.PendingMessageStyle = string.Empty; }


        }


        private SelectList BindAllQuestionBulkActionList(ViewQuestionModel model)
        {
            List<SelectListItem> items = new List<SelectListItem>();

            items.Add(new SelectListItem() { Text = "Bulk Action", Value = "Bulk Action" });

            if (model.PageRights._delete_question == true)
            {
                items.Add(new SelectListItem() { Text = "Delete", Value = "Delete" });
            }

            items.Add(new SelectListItem() { Text = "Active", Value = "Active" });
            items.Add(new SelectListItem() { Text = "In-Active", Value = "In-Active" });
            var typeList = new SelectList(items.ToList(), "Text", "Value");

            return typeList;

        }
        #endregion

        #region DashboardModule
        // GET: /Dashboard/
        public ActionResult Dashboard()
        {
            ActionResult loginActionResult = UserAuthentication();
            if (accessAccount.CurrentUserId == 0)
                return loginActionResult;


            SetMenuProperties("Dashboard", "", "");
            var model = new PublisherDashBoardModel();

            LoadData(model, null, null);
            ViewBag.Dates = "?ToDate=AllClasses";
            return View(model);
        }

        // Post: /Dashboard/
        [HttpPost]
        public ActionResult Dashboard(PublisherDashBoardModel model, FormCollection collection, string btnSubmit, int id = 0)
        {

            ActionResult loginActionResult = UserAuthentication();
            if (accessAccount.CurrentUserId == 0)
                return loginActionResult;

            SetMenuProperties("Dashboard", "", "");


            if (btnSubmit == "Accept")
            {
                if (id > 0)
                {
                    ClassSubscription Clssub = QuizRepository.GetClsSubbyId(id);
                    if (Clssub != null)
                    {
                        Clssub.SubscriptionStatus = 1;
                        Clssub.ConfirmedDate = DateTime.Now;
                        QuizRepository.UpdateClsSub(Clssub);
                        QuizRepository.Save();
                        model.MessageType = "success";
                        model.Message = "Pending private class approval accepted";
                    }
                }

            }
            if (btnSubmit == "Decline")
            {
                if (id > 0)
                {
                    ClassSubscription Clssub = QuizRepository.GetClsSubbyId(id);
                    if (Clssub != null)
                    {
                        Clssub.SubscriptionStatus = 3;
                        Clssub.ConfirmedDate = DateTime.Now;
                        QuizRepository.UpdateClsSub(Clssub);
                        QuizRepository.Save();
                        model.MessageType = "success";
                        model.Message = "Pending private class approval declined";
                    }
                }

            }
            if (!string.IsNullOrEmpty(model.Dates) && model.Dates != "Select Dates")
            {
                string FromDate = null;
                string ToDate = null;
                if (model.Dates.Contains('-'))
                {
                    List<string> Dates = model.Dates.Split('-').Select(p => p.Trim()).ToList();
                    FromDate = Dates[0];
                    ToDate = Dates[1];

                    DateTime FromDt = Convert.ToDateTime(Dates[0]);
                    DateTime ToDt = Convert.ToDateTime(Dates[1]);
                    if (FromDt.Date > ToDt.Date)
                    {
                        model.MessageType = "error";
                        model.Message = "Start Date should be before End Date";
                        LoadData(model, null, null);
                    }
                    else
                    {
                        LoadData(model, FromDate, ToDate);
                    }
                    ViewBag.Dates = "?FromDate=" + FromDate + "" + "&ToDate=" + ToDate;
                }

                else
                {
                    ToDate = model.Dates;
                    LoadData(model, FromDate, ToDate);
                    ViewBag.Dates = "?ToDate=" + model.Dates;
                }

            }
            else
            {
                LoadData(model, null, null);

            }
            return View(model);
        }

        // Functions: /Dashboard/
        public void LoadData(PublisherDashBoardModel model, string FromDate, string ToDate)
        {
            model.PageRights = GetPageRights();

            model.DashboardOverview = QuizRepository.DashboardOverview(accessAccount.PublisherUserId, FromDate, ToDate).FirstOrDefault();
            if (model.PageRights._view_all_public_class == true && model.PageRights._view_all_private_class == false)
            {
                model.DashboardOverview.TotalClasses = (from PublicClass in model.DashboardOverview.TotalClasses where PublicClass.ClassType == true select PublicClass).ToList();
            }
            else if (model.PageRights._view_all_public_class == false && model.PageRights._view_all_private_class == true)
            {
                model.DashboardOverview.TotalClasses = (from PublicClass in model.DashboardOverview.TotalClasses where PublicClass.ClassType == false select PublicClass).ToList();
            }

            if (accessAccount.RoleId == 0)
            {
                model.DashboardApproval = QuizRepository.DashboardApprval(accessAccount.PublisherUserId);
                model.DashboardMyRole = usersRepository.DashboardRoles(accessAccount.PublisherUserId);
                model.DashboardPostContent = classRepository.DashboardGetAssignedClasses(accessAccount.PublisherUserId);
            }
            else
            { model.IsRoleId = true; }

            model.ListQuiz = QuizRepository.GetQuizList(accessAccount.PublisherUserId).ToList();
            var Quiz = model.ListQuiz.FirstOrDefault();

            if (model.QuizId == 0)
            {
                if (Quiz != null)
                {
                    model.QuizId = Quiz.QuizId;
                }
            }
            CheckDatebyValue(ref FromDate, ref ToDate);
            model.ListNewStudent = usersRepository.GetNewStudent(accessAccount.PublisherUserId, FromDate, ToDate).ToList();
            model.ListStudentExamOverview = usersRepository.GetStudentExamOverView(accessAccount.PublisherUserId, FromDate, ToDate).ToList();
            model.ListStudentExamOverviewByQuiz = usersRepository.GetStudentExamOverviewByQuiz(accessAccount.PublisherUserId, model.QuizId, FromDate, ToDate).ToList();





        }
        private void CheckDatebyValue(ref string FromDate, ref string ToDate)
        {
            if (string.IsNullOrEmpty(FromDate) && !string.IsNullOrEmpty(ToDate))
            {
                FromDate = ToDate;
            }
            else if (!string.IsNullOrEmpty(FromDate) && string.IsNullOrEmpty(ToDate))
            {
                ToDate = FromDate;
            }
        }

        #endregion

        #region QuizModule

        // GET: /Quiz/
        public ActionResult BuildQuiz()
        {
            ActionResult loginActionResult = UserAuthentication();
            if (accessAccount.CurrentUserId == 0)
                return loginActionResult;

            SetMenuProperties("Build", "Quiz Tools", "Build Quiz");
            var model = new QuizModel();
            model.QuizForm = new Quizzes();
            model.IsQuizTitle = false;
            model.QuizForm.ExamTimed = false;
            model.QuizForm.DisplayAnswers = false;
            model.QuizForm.QuestionOrder = true;

            model.StatusName = "New";
            model.PublishOn = "Today";
            model.PageRights = GetPageRights();
            return View(model);
        }
        public ActionResult EditQuiz(int QuizId)
        {
            ActionResult loginActionResult = UserAuthentication();
            if (accessAccount.CurrentUserId == 0)
                return loginActionResult;

            SetMenuProperties("Build", "Quiz Tools", "Edit Quiz");
            var model = new QuizModel();
            model.QuizForm = new Quizzes();
            model.IsQuizTitle = false;
            Quizzes quiz = QuizRepository.GetQuizbyId(QuizId);
            if (quiz != null)
            {
                model.QuizForm.QuizId = quiz.QuizId;
                model.QuizForm.QuizTitle = quiz.QuizTitle;
                model.QuizForm.PassingScore = quiz.PassingScore;
                model.QuizForm.QuestionAmount = quiz.QuestionAmount;
                model.QuizForm.AttemptLimit = quiz.AttemptLimit;
                model.QuizForm.ExamTimed = quiz.ExamTimed;
                model.QuizForm.TimeAllowed = quiz.TimeAllowed;
                model.QuizForm.DisplayAnswers = quiz.DisplayAnswers;
                model.QuizForm.QuestionOrder = quiz.QuestionOrder;
                model.QuizForm.Instructions = quiz.Instructions;
                model.QuizForm.FeedbackPass = quiz.FeedbackPass;
                model.QuizForm.FeedbackFail = quiz.FeedbackFail;

                #region StatusName
                if (quiz.QuizStatus == 0)
                {
                    model.StatusName = "Draft";
                }
                else if (quiz.QuizStatus == 1)
                {
                    model.StatusName = "Published";
                }
                else if (quiz.QuizStatus == 2)
                {
                    model.StatusName = "Yet to Publish";
                }
                #endregion

                #region PublishDate
                if (quiz.QuizStatus == 2)
                {
                    if (quiz.PublishedDate != null)
                    {
                        DateTime PublishDate = Convert.ToDateTime(quiz.PublishedDate);
                        if (PublishDate.Date > DateTime.Now.Date)
                        {
                            string date = String.Format("{0:MM/dd/yyyy}", PublishDate);
                            model.PublishOn = date;
                        }

                    }
                }
                if (quiz.QuizStatus == 1)
                {
                    if (quiz.PublishedDate != null)
                    {
                        DateTime PublishDate = Convert.ToDateTime(quiz.PublishedDate);
                        if (PublishDate.Date <= DateTime.Now.Date)
                        {
                            string date = String.Format("{0:MM/dd/yyyy}", PublishDate);
                            model.PublishOn = date;
                            model.IsPublished = true;
                        }

                    }
                }
                if (quiz.QuizStatus == 0)
                {

                    model.PublishOn = "Today";
                }
                #endregion
            }
            model.QuestionsList = QuizRepository.GetQuestions(QuizId);

            model.PageRights = GetPageRights();
            if (TempData["EditQuizModel"] != null)
            {
                var quizmodel = TempData["EditQuizModel"] as QuizModel;
                model.ValidationMessage = quizmodel.ValidationMessage;
                model.ValidationMessageType = quizmodel.ValidationMessageType;
            }
            return View(model);
        }
        public ActionResult PreviewQuiz()
        {
            ActionResult loginActionResult = UserAuthentication();
            if (accessAccount.CurrentUserId == 0)
                return loginActionResult;

            SetMenuProperties("Build", "Quiz Tools", "Preview Quiz");

            var model = new QuizModel();

            if (Session["QuizModel"] != null)
            {
                model = (QuizModel)Session["QuizModel"];


            }
            else
            {

                return RedirectToAction("PageExpired", "Class");
            }


            return View(model);
        }
        public ActionResult AllQuizzes(int? page = 1)
        {
            ActionResult loginActionResult = UserAuthentication();
            if (accessAccount.CurrentUserId == 0)
                return loginActionResult;

            SetMenuProperties("Build", "Quiz Tools", "All Quizzes");
            var model = new QuizListModel();
            LoadAllQuiz(ref model);

            return View(model);
        }


        // Post: /Quiz/
        [HttpPost]
        public ActionResult BuildQuiz(QuizModel model, string btnsubmit, string output_text, FormCollection collection)
        {
            ActionResult loginActionResult = UserAuthentication();
            if (accessAccount.CurrentUserId == 0)
                return loginActionResult;

            SetMenuProperties("Build", "Quiz Tools", "Build Quiz");
            if (ValidateQuizInputs(out output_text, model, collection))
            {
                #region SaveDraft
                if (btnsubmit == "SaveDraft")
                {
                    model.QuizForm.QuizStatus = 0;
                    model.QuizForm.CreatedDate = DateTime.Now;
                    model.QuizForm.PublishedDate = null;

                    if (Request.Form["RadioExamTimed"].ToString() == "Yes")
                        model.QuizForm.ExamTimed = true;
                    else
                        model.QuizForm.ExamTimed = false;

                    if (Request.Form["DisplayAnswers"].ToString() == "Yes")
                        model.QuizForm.DisplayAnswers = true;
                    else
                        model.QuizForm.DisplayAnswers = false;

                    if (Request.Form["QuestionOrder"].ToString() == "Random")
                        model.QuizForm.QuestionOrder = true;
                    else
                        model.QuizForm.QuestionOrder = false;

                    CreateQuiz(model, accessAccount.PublisherUserId);
                    if (model.IsQuizTitle == true)
                    {
                        model.ValidationMessageType = "warning";
                        model.ValidationMessage = "Quiz title already exists";
                        model.IsQuizTitle = false;
                        model.PageRights = GetPageRights();
                        LoadDefaultData(model, collection);
                        model.StatusName = "New";
                        return View("BuildQuiz", model);
                    }
                    else
                    {
                        model.ValidationMessageType = "success";
                        model.ValidationMessage = "Quiz saved in draft";
                        model.PageRights = GetPageRights();
                        if (model.PageRights._edit_quiz == true)
                        {
                            TempData["EditQuizModel"] = model;
                            return RedirectToAction("EditQuiz", new { QuizId = model.QuizForm.QuizId });
                        }
                        else if (model.PageRights._edit_quiz == false)
                        {
                            model.QuizForm = new Quizzes();
                            ModelState.Clear();
                            LoadDefaultData(model, collection);
                            model.StatusName = "New";
                            return View("BuildQuiz", model);
                        }
                    }
                }
                #endregion

                #region Publish
                if (btnsubmit == "Publish")
                {
                    model.QuizForm.QuizStatus = 1;
                    model.QuizForm.PublishedUserId = accessAccount.CurrentUserId;
                    model.QuizForm.CreatedDate = DateTime.Now;
                    model.QuizForm.PublishedDate = DateTime.Now;

                    if (Request.Form["RadioExamTimed"].ToString() == "Yes")
                        model.QuizForm.ExamTimed = true;
                    else
                        model.QuizForm.ExamTimed = false;

                    if (Request.Form["DisplayAnswers"].ToString() == "Yes")
                        model.QuizForm.DisplayAnswers = true;
                    else
                        model.QuizForm.DisplayAnswers = false;

                    if (Request.Form["QuestionOrder"].ToString() == "Random")
                        model.QuizForm.QuestionOrder = true;
                    else
                        model.QuizForm.QuestionOrder = false;

                    CreateQuiz(model, accessAccount.PublisherUserId);
                    if (model.IsQuizTitle == true)
                    {
                        model.ValidationMessageType = "warning";
                        model.ValidationMessage = "Quiz title already exists";
                        model.IsQuizTitle = false;
                        model.PageRights = GetPageRights();
                        LoadDefaultData(model, collection);
                        model.StatusName = "New";
                        return View("BuildQuiz", model);
                    }
                    else
                    {

                        model.ValidationMessageType = "success";
                        model.ValidationMessage = "Quiz publish successfully";
                        model.PageRights = GetPageRights();
                        if (model.PageRights._edit_quiz == true)
                        {
                            TempData["EditQuizModel"] = model;
                            return RedirectToAction("EditQuiz", new { QuizId = model.QuizForm.QuizId });
                        }
                        else if (model.PageRights._edit_quiz == false)
                        {
                            model.QuizForm = new Quizzes();
                            ModelState.Clear();
                            LoadDefaultData(model, collection);
                            model.StatusName = "New";
                            return View("BuildQuiz", model);
                        }
                    }
                }
                #endregion

                #region Preview
                if (btnsubmit == "Preview")
                {
                    Session["QuizModel"] = null;
                    Session["QuizModel"] = model;
                    ViewBag.IsQuizPreview = "yes";
                }
                #endregion
            }
            else
            {
                model.ValidationMessage = output_text;
                model.ValidationMessageType = "error";
            }
            LoadDefaultData(model, collection);
            model.StatusName = "New";
            model.PageRights = GetPageRights();
            return View("BuildQuiz", model);
        }
        [HttpPost]
        public ActionResult EditQuiz(int QuizId, QuizModel model, string btnsubmit, string output_text, FormCollection collection)
        {
            ActionResult loginActionResult = UserAuthentication();
            if (accessAccount.CurrentUserId == 0)
                return loginActionResult;

            SetMenuProperties("Build", "Quiz Tools", "Edit Quiz");
            if (ValidateQuizInputs(out output_text, model, collection))
            {
                #region SaveDraft
                if (btnsubmit == "SaveDraft")
                {
                    model.QuizForm.QuizStatus = 0;
                    model.QuizForm.ModifiedDate = DateTime.Now;
                    model.QuizForm.ModifiedUserId = accessAccount.CurrentUserId;
                    model.QuizForm.PublishedDate = null;

                    if (Request.Form["RadioExamTimed"].ToString() == "Yes")
                        model.QuizForm.ExamTimed = true;
                    else
                        model.QuizForm.ExamTimed = false;

                    if (Request.Form["DisplayAnswers"].ToString() == "Yes")
                        model.QuizForm.DisplayAnswers = true;
                    else
                        model.QuizForm.DisplayAnswers = false;

                    if (Request.Form["QuestionOrder"].ToString() == "Random")
                        model.QuizForm.QuestionOrder = true;
                    else
                        model.QuizForm.QuestionOrder = false;

                    UpdateQuiz(model, accessAccount.PublisherUserId, QuizId);
                    if (model.IsQuizTitle == true)
                    {
                        model.ValidationMessageType = "warning";
                        model.ValidationMessage = "Quiz title already exists";
                        model.IsQuizTitle = false;
                        model.PageRights = GetPageRights();
                        LoadDefaultData(model, collection);
                        model.QuestionsList = QuizRepository.GetQuestions(QuizId);
                        return View("EditQuiz", model);
                    }
                    else
                    {
                        model.ValidationMessageType = "success";
                        model.ValidationMessage = "Quiz saved in draft";
                        model.PageRights = GetPageRights();
                        if (model.QuizForm.QuizStatus == 0)
                        {
                            model.StatusName = "Draft";
                            model.PublishOn = "Today";
                        }
                        model.QuestionsList = QuizRepository.GetQuestions(QuizId);
                        return View("EditQuiz", model);
                    }
                }
                #endregion

                #region Publish
                if (btnsubmit == "Publish")
                {
                    model.QuizForm.QuizStatus = 1;
                    model.QuizForm.ModifiedDate = DateTime.Now;
                    model.QuizForm.PublishedDate = DateTime.Now;
                    model.QuizForm.ModifiedUserId = accessAccount.CurrentUserId;
                    model.QuizForm.PublishedUserId = accessAccount.CurrentUserId;

                    if (Request.Form["RadioExamTimed"].ToString() == "Yes")
                        model.QuizForm.ExamTimed = true;
                    else
                        model.QuizForm.ExamTimed = false;

                    if (Request.Form["DisplayAnswers"].ToString() == "Yes")
                        model.QuizForm.DisplayAnswers = true;
                    else
                        model.QuizForm.DisplayAnswers = false;

                    if (Request.Form["QuestionOrder"].ToString() == "Random")
                        model.QuizForm.QuestionOrder = true;
                    else
                        model.QuizForm.QuestionOrder = false;

                    UpdateQuiz(model, accessAccount.PublisherUserId, QuizId);
                    if (model.IsQuizTitle == true)
                    {
                        model.ValidationMessageType = "warning";
                        model.ValidationMessage = "Quiz title already exists";
                        model.IsQuizTitle = false;
                        model.PageRights = GetPageRights();
                        LoadDefaultData(model, collection);
                        model.QuestionsList = QuizRepository.GetQuestions(QuizId);
                        return View("EditQuiz", model);
                    }
                    else
                    {
                        model.ValidationMessageType = "success";
                        model.ValidationMessage = "Quiz Publish successfully";
                        model.PageRights = GetPageRights();
                        if (model.QuizForm.QuizStatus == 1)
                        {
                            model.StatusName = "Published";
                            model.IsPublished = true;
                            if (model.PublishOn == "Today")
                            {
                                string date = String.Format("{0:MM/dd/yyyy}", DateTime.Now);
                                model.PublishOn = date;
                            }
                            else
                            {
                                string date = String.Format("{0:MM/dd/yyyy}", model.QuizForm.PublishedDate);
                                model.PublishOn = date;
                            }
                        }
                        else
                        {
                            model.StatusName = "Yet to Publish";
                        }
                        model.QuestionsList = QuizRepository.GetQuestions(QuizId);
                        return View("EditQuiz", model);
                    }
                }
                #endregion

                #region Preview
                if (btnsubmit == "Preview")
                {
                    Session["QuizModel"] = null;
                    Session["QuizModel"] = model;
                    ViewBag.IsQuizPreview = "yes";
                }
                #endregion
            }
            else
            {
                model.ValidationMessage = output_text;
                model.ValidationMessageType = "error";
            }
            Quizzes quiz = QuizRepository.GetQuizbyId(QuizId);
            #region StatusName
            if (quiz.QuizStatus == 0)
            {
                model.StatusName = "Draft";
            }
            else if (quiz.QuizStatus == 1)
            {
                model.StatusName = "Published";
            }
            else if (quiz.QuizStatus == 2)
            {
                model.StatusName = "Yet to Publish";
            }
            #endregion
            model.PageRights = GetPageRights();
            LoadDefaultData(model, collection);
            model.QuestionsList = QuizRepository.GetQuestions(QuizId);
            return View("EditQuiz", model);
        }
        [HttpPost]
        public ActionResult AllQuizzes(FormCollection collection, string btnSubmit)
        {
            ActionResult loginActionResult = UserAuthentication();
            if (accessAccount.CurrentUserId == 0)
                return loginActionResult;

            SetMenuProperties("Build", "Quiz Tools", "All Quizzes");
            var model = new QuizListModel();
            if (btnSubmit == "Apply")
            {
                if (Request.Form["Actions"] == "Delete")
                {
                    if (collection.GetValues("chk") != null)
                    {
                        int j = Request.Form["chk"].Split(',').Count();
                        List<bool> retval = new List<bool>();
                        for (int i = 0; i < j; i++)
                        {
                            var QuizId = Convert.ToInt32(collection.GetValues("chk")[i]);
                            List<Contents> ContentList = ContentRepository.ContentbyQuizId(QuizId);
                            if (ContentList.Count == 0)
                            {
                                bool ret = QuizRepository.DeleteQuiz(QuizId);
                                retval.Add(ret);
                                QuizRepository.Save();
                            }
                            else
                            {
                                model.ValidationMessageType = "warning";
                                model.ValidationMessage = "Cannot delete this Quiz because its added in content";
                                LoadAllQuiz(ref model);
                                return View("AllQuizzes", model);
                            }
                        }
                        if (retval.Contains(true))
                        {
                            model.ValidationMessageType = "success";
                            model.ValidationMessage = "Quiz(s) deleted successfully.";
                            LoadAllQuiz(ref model);
                            return View("AllQuizzes", model);
                        }
                    }
                    else
                    {
                        model.ValidationMessageType = "warning";
                        model.ValidationMessage = "Please select checkbox";
                        LoadAllQuiz(ref model);
                        return View("AllQuizzes", model);
                    }

                }
                else
                {
                    model.ValidationMessageType = "warning";
                    model.ValidationMessage = "No action selected";
                    LoadAllQuiz(ref model);
                    return View("AllQuizzes", model);
                }

            }
            return View("AllQuizzes", model);
        }


        // Functions: /Quiz/
        public bool ValidateQuizInputs(out string output_text, QuizModel model, FormCollection collection)
        {
            bool retval = true;
            output_text = "";
            if (string.IsNullOrEmpty(model.QuizForm.QuizTitle))
            { output_text += "* Quiz title is required.<br/>"; retval = false; }

            if (model.QuizForm.PassingScore == 0 || model.QuizForm.PassingScore == null)
            {
                output_text += "* Passing Score is required.<br/>"; retval = false;
            }


            if (model.QuizForm.QuestionAmount == 0 || model.QuizForm.QuestionAmount == null)
            { output_text += "* Question Amount is required.<br/>"; retval = false; }

            if (model.QuizForm.AttemptLimit == 0 || model.QuizForm.AttemptLimit == null)
            { output_text += "* Attempt Limit is required.<br/>"; retval = false; }

            var RadioExamTimed = collection.GetValues("RadioExamTimed").FirstOrDefault();
            if (RadioExamTimed == "Yes")
            {
                if (model.QuizForm.TimeAllowed == 0 || model.QuizForm.TimeAllowed == null)
                { output_text += "* Time Allowed is required.<br/>"; retval = false; }
            }

            if (string.IsNullOrEmpty(model.QuizForm.Instructions))
            { output_text += "* Instructions is required.<br/>"; retval = false; }

            if (string.IsNullOrEmpty(model.QuizForm.FeedbackPass))
            { output_text += "* Feedback/Pass is required.<br/>"; retval = false; }

            if (string.IsNullOrEmpty(model.QuizForm.FeedbackFail))
            { output_text += "* Feedback/Fail is required.<br/>"; retval = false; }
            return retval;
        }
        private void LoadDefaultData(QuizModel model, FormCollection collection)
        {
            var RadioExamTimed = collection.GetValues("RadioExamTimed").FirstOrDefault();
            if (RadioExamTimed == "Yes")
            {
                model.QuizForm.ExamTimed = true;
            }
            else if (RadioExamTimed == "No")
            {
                model.QuizForm.ExamTimed = false;
            }
            var DisplayAnswers = collection.GetValues("DisplayAnswers").FirstOrDefault();
            if (DisplayAnswers == "Yes")
            {
                model.QuizForm.DisplayAnswers = true;
            }
            else if (DisplayAnswers == "No")
            {
                model.QuizForm.DisplayAnswers = false;
            }
            var QuestionOrder = collection.GetValues("QuestionOrder").FirstOrDefault();
            if (QuestionOrder == "Random")
            {
                model.QuizForm.QuestionOrder = true;
            }
            else if (QuestionOrder == "Static")
            {
                model.QuizForm.QuestionOrder = false;
            }

        }
        public void CreateQuiz(QuizModel model, int UserId)
        {
            Quizzes Quiz = new Quizzes();

            Quizzes Quizdetails = QuizRepository.GetQuiz(model.QuizForm.QuizTitle, UserId);

            if (Quizdetails != null)
            {
                model.IsQuizTitle = true;

            }
            else
            {
                Quiz.QuizTitle = model.QuizForm.QuizTitle;
                Quiz.PassingScore = model.QuizForm.PassingScore;
                Quiz.QuestionAmount = model.QuizForm.QuestionAmount;
                Quiz.AttemptLimit = model.QuizForm.AttemptLimit;
                Quiz.ExamTimed = model.QuizForm.ExamTimed;
                Quiz.TimeAllowed = model.QuizForm.TimeAllowed;
                Quiz.DisplayAnswers = model.QuizForm.DisplayAnswers;
                Quiz.QuestionOrder = model.QuizForm.QuestionOrder;
                Quiz.Instructions = model.QuizForm.Instructions;
                Quiz.FeedbackPass = model.QuizForm.FeedbackPass;
                Quiz.FeedbackFail = model.QuizForm.FeedbackFail;
                Quiz.PublishedUserId = model.QuizForm.PublishedUserId;

                if (model.QuizForm.PublishedDate != null)
                {
                    if (!string.IsNullOrEmpty(model.PublishOn))
                    {
                        if (model.PublishOn == "Today")
                        {
                            DateTime PublishDate = DateTime.Now;
                            Quiz.PublishedDate = PublishDate.Date;
                        }
                        else
                        {
                            DateTime PublishDate = Convert.ToDateTime(model.PublishOn);
                            if (PublishDate.Date == DateTime.Now.Date)
                            {
                                model.QuizForm.QuizStatus = 1;
                                model.QuizForm.PublishedDate = DateTime.Now;
                                Quiz.PublishedDate = model.QuizForm.PublishedDate;
                            }
                            else if (PublishDate.Date > DateTime.Now.Date)
                            {

                                model.QuizForm.PublishedDate = PublishDate;
                                Quiz.PublishedDate = model.QuizForm.PublishedDate;
                                model.QuizForm.QuizStatus = 2; //Yet to Publish
                            }


                        }


                    }
                    else
                    {
                        Quiz.PublishedDate = null;
                    }
                }
                else
                {
                    Quiz.PublishedDate = model.QuizForm.PublishedDate;
                }
                Quiz.CreatedDate = model.QuizForm.CreatedDate;
                Quiz.QuizStatus = model.QuizForm.QuizStatus;
                Quiz.PublisherUserId = accessAccount.PublisherUserId;
                Quiz.CreatedUserId = accessAccount.CurrentUserId;

                QuizRepository.InsertQuiz(Quiz);
                QuizRepository.Save();
                model.QuizForm.QuizId = Quiz.QuizId;
            }
        }
        public void UpdateQuiz(QuizModel model, int UserId, int QuizId)
        {


            Quizzes Quizdetails = QuizRepository.GetQuiz(model.QuizForm.QuizTitle, UserId);
            Quizzes Quiz = QuizRepository.GetQuizbyId(QuizId);
            if ((Quizdetails != null) && (Quiz.QuizTitle != model.QuizForm.QuizTitle))
            {

                model.IsQuizTitle = true;

            }
            else
            {

                Quiz.QuizTitle = model.QuizForm.QuizTitle;
                Quiz.PassingScore = model.QuizForm.PassingScore;
                Quiz.QuestionAmount = model.QuizForm.QuestionAmount;
                Quiz.AttemptLimit = model.QuizForm.AttemptLimit;
                Quiz.ExamTimed = model.QuizForm.ExamTimed;
                Quiz.TimeAllowed = model.QuizForm.TimeAllowed;
                Quiz.DisplayAnswers = model.QuizForm.DisplayAnswers;
                Quiz.QuestionOrder = model.QuizForm.QuestionOrder;
                Quiz.Instructions = model.QuizForm.Instructions;
                Quiz.FeedbackPass = model.QuizForm.FeedbackPass;
                Quiz.FeedbackFail = model.QuizForm.FeedbackFail;
                Quiz.PublishedUserId = model.QuizForm.PublishedUserId;
                Quiz.ModifiedUserId = model.QuizForm.ModifiedUserId;

                if (model.QuizForm.PublishedDate != null)
                {
                    if (!string.IsNullOrEmpty(model.PublishOn))
                    {
                        if (model.PublishOn == "Today")
                        {
                            DateTime PublishDate = DateTime.Now;
                            Quiz.PublishedDate = PublishDate.Date;
                        }
                        else
                        {
                            DateTime PublishDate = Convert.ToDateTime(model.PublishOn);
                            if (PublishDate.Date == DateTime.Now.Date)
                            {
                                model.QuizForm.QuizStatus = 1;
                                model.QuizForm.PublishedDate = DateTime.Now;
                                Quiz.PublishedDate = model.QuizForm.PublishedDate;
                            }
                            else if (PublishDate.Date > DateTime.Now.Date)
                            {

                                model.QuizForm.PublishedDate = PublishDate;
                                Quiz.PublishedDate = model.QuizForm.PublishedDate;
                                model.QuizForm.QuizStatus = 2; //Yet to Publish
                            }
                            else if (model.QuizForm.QuizStatus == 1)
                            {
                                Quiz.PublishedDate = model.QuizForm.PublishedDate;
                            }


                        }


                    }
                    else
                    {
                        Quiz.PublishedDate = model.QuizForm.PublishedDate;
                    }
                }
                else
                {
                    Quiz.PublishedDate = model.QuizForm.PublishedDate;
                }
                Quiz.ModifiedDate = model.QuizForm.ModifiedDate;
                Quiz.QuizStatus = model.QuizForm.QuizStatus;
                Quiz.PublisherUserId = accessAccount.PublisherUserId;

                QuizRepository.UpdateQuiz(Quiz);
                QuizRepository.Save();

            }
        }
        private void LoadAllQuiz(ref QuizListModel model)
        {
            model.PageRights = GetPageRights();


            model.TotalQuizList = QuizRepository.GetAllQuiz(accessAccount.PublisherUserId).OrderByDescending(m => m.QuizId).ToList();



            List<SelectListItem> Actions = new List<SelectListItem>();
            if (model.PageRights._delete_quiz == true)
            {
                Actions.Add(new SelectListItem
                {
                    Text = "Delete",
                    Value = "Delete"
                });

            }
            ViewData["ddlActions"] = Actions;
        }

        #endregion

        #region Contents & Categories Modules

        // GET: /Content & Categories/
        public ActionResult BuildContent(ContentModel model, FormCollection collection)
        {
            ActionResult loginActionResult = UserAuthentication();
            if (accessAccount.CurrentUserId == 0)
                return loginActionResult;

            SetMenuProperties("Build", "Content Tools", "Build Content");
            model.IsContentTitle = false;
            model.IsCategory = false;
            model.IsCatgEdit = false;
            // model.IsPageEdit = false;


            if (Session["ContentModel"] != null || Session["UploadContentImages"] != null)
            {
                Session["ContentModel"] = null;
                Session["UploadContentImages"] = null;
            }
            if (TempData["TempModel"] != null)
            {
                model = TempData["TempModel"] as ContentModel;
            }

            if (model.IsCatgEdit == false)
            {
                model.Form = new Categories();
                SetupCategoriesModel(model);
            }


            model.StatusName = "New";
            model.PublishOn = "Today";
            if (ViewBag.IsPageDiscard == null)
            {
                ViewBag.IsPageDiscard = false;
            }
            LoadQuizzes(collection, model);
            LoadClasses(model, collection);
            model.PageRights = GetPageRights();
            return View(model);
        }
        public ActionResult EditContent(int ContentId, ContentModel model, FormCollection collection)
        {
            ActionResult loginActionResult = UserAuthentication();
            if (accessAccount.CurrentUserId == 0)
                return loginActionResult;
            SetMenuProperties("Build", "Content Tools", "Edit Content");

            model.IsContentTitle = false;
            model.IsCategory = false;
            model.IsCatgEdit = false;
            model.IsPageEdit = false;
            if (ContentId != 0)
            {

                string ContTitle = ContentRepository.GetContentTitle(ContentId);

                model.ContentTitle = ContTitle;


                IEnumerable<ContentPages> ContPages = ContentRepository.GetContentPages(ContentId);
                if (ContPages != null)
                {
                    foreach (var ContPage in ContPages)
                    {
                        model.PageDesc += ContPage.PageDescription;
                    }
                }


            }
            model.IsPageEdit = true;
            model.IsCatgDefault = true;

            Contents Contdetail = ContentRepository.GetContentListById(ContentId).FirstOrDefault();

            #region StatusName

            string PublishedDate = Convert.ToString(Contdetail.PublishedDate);
            if (Contdetail.ContentStatus == 2)
            {
                if (!string.IsNullOrEmpty(PublishedDate))
                {
                    DateTime PublishDate = Convert.ToDateTime(PublishedDate);
                    if (PublishDate.Date > DateTime.Now.Date)
                    {
                        string date = String.Format("{0:MM/dd/yyyy}", PublishDate);
                        model.PublishOn = date;
                    }

                }
            }
            if (Contdetail.ContentStatus == 1)
            {
                if (!string.IsNullOrEmpty(PublishedDate))
                {
                    DateTime PublishDate = Convert.ToDateTime(PublishedDate);
                    if (PublishDate.Date <= DateTime.Now.Date)
                    {
                        string date = String.Format("{0:MM/dd/yyyy}", PublishDate);
                        model.PublishOn = date;
                        model.IsPublished = true;
                    }

                }
            }
            if (Contdetail.ContentStatus == 0)
            {

                model.PublishOn = "Today";

            }
            #endregion

            EditContentLoadData(model, collection, ContentId);
            ViewBag.IsPageDiscard = false;
            if (TempData["ContentModel"] != null)
            {
                var contentmodel = TempData["ContentModel"] as ContentModel;
                model.Message = contentmodel.Message;
                model.MessageType = contentmodel.MessageType;

            }
            if (Contdetail.QuizId != null && Contdetail.QuizId != 0)
            {
                model.QuizId = Contdetail.QuizId.Value;

            }
            if (Session["UploadContentImages"] != null)
            {
                Session["UploadContentImages"] = null;
            }
            LoadQuizzes(collection, model);
            LoadClasses(model, collection);
            return View(model);
        }
        public ActionResult UploadContentImages(HttpPostedFileBase upload, string CKEditorFuncNum, string CKEditor, string langCode)
        {
            ActionResult loginActionResult = UserAuthentication();
            if (accessAccount.CurrentUserId == 0)
                return loginActionResult;
            ContentModel.UploadImages model = new ContentModel.UploadImages();
            string url = null; // url to return
            string message; // message to display (optional) 
            string fileName = upload.FileName;
            string output = null;
            string FileNameRandom = Guid.NewGuid().ToString();
            string path = "Uploads/ContentImageFolder/" + FileNameRandom;
            string basePath = Server.MapPath("/Uploads/ContentImageFolder");
            upload.SaveAs(basePath + "\\" + FileNameRandom);
            basePath = basePath + "\\" + FileNameRandom;
            FileInfo fileinfo = new FileInfo(basePath);
            if (fileinfo.Exists)
            {
                model.FileName = new List<string>();
                model.FolderName = new List<string>();

                if (Session["UploadContentImages"] != null)
                {
                    model = (ContentModel.UploadImages)Session["UploadContentImages"];
                    model.FileName.Add(fileName);
                    model.FolderName.Add(FileNameRandom);
                    Session["UploadContentImages"] = model;

                }
                else
                {
                    model.FileName.Add(fileName);
                    model.FolderName.Add(FileNameRandom);
                    Session["UploadContentImages"] = model;
                }


                message = "Image was saved correctly";

            }
            else
            {
                message = "Image not saved correctly";
            }
            url = Request.Url.GetLeftPart(UriPartial.Authority) + "/" + path;
            output = @"<html><body>   <script>window.parent.CKEDITOR.tools.callFunction(" + CKEditorFuncNum + ", \"" + url + "\", \"" + message + "\");</script></body></html>";

            return Content(output);



        }
        public ActionResult PreviewContent(int ContentId = 0, int? page = 1)
        {
            ActionResult loginActionResult = UserAuthentication();
            if (accessAccount.CurrentUserId == 0)
                return loginActionResult;


            ContentModel model = new ContentModel();
            if (ContentId != 0)
            {
                SetMenuProperties("Build", "Content Tools", "All Content");
                model.IsContentListPreview = true;

                Contents cnt = ContentRepository.GetContentListById(ContentId).FirstOrDefault();
                model.ContentId = cnt.ContentId;
                model.ContentTitle = cnt.ContentTitle;
                List<ContentPages> ContPage = ContentRepository.GetContentPages(cnt.ContentId).ToList();
                model.ContentPagingList = new List<ContentPages>();
                if (ContPage.Count > 0)
                {
                    for (int j = 1; j <= ContPage.Count(); j++)
                    {
                        ContentPages pageObj = new ContentPages();
                        pageObj.ContentPageId = j;
                        pageObj.PageDescription = ContPage[j - 1].PageDescription;
                        pageObj.ContentId = ContPage[j - 1].ContentId;
                        model.ContentPagingList.Add(pageObj);
                    }

                    if (Session["ContentPreview"] != null)
                    {
                        Session["ContentPreview"] = null;
                        Session["ContentPreview"] = model;

                    }
                    else
                    {
                        Session["ContentPreview"] = model;
                    }

                }

                ViewBag.Parameter = ("&ContentId=" + ContentId);
            }
            else
            {
                SetMenuProperties("Build", "Content Tools", "");
                if (Session["ContentPreview"] != null)
                {
                    model = (ContentModel)Session["ContentPreview"];

                }

            }
            if (model.ContentPagingList != null && model.ContentPagingList.Count > 0)
            {
                List<ContentPages> ContPages = new List<ContentPages>();
                ViewBag.ContentPreview = true;
                PopulatePagingData(model.ContentPagingList.Count, "Publisher", "PreviewContent", page);
                if (page >= 1)
                {
                    ContPages = model.ContentPagingList.FindAll(p => p.ContentPageId == page);
                }

                model.PageDesc = ContPages.FirstOrDefault().PageDescription;

            }
            model.PageRights = GetPageRights();
            ViewBag.ContentNameNavigation = model.ContentTitle;
            return View(model);
        }
        public ActionResult AllContent(int? page = 1, int ClassId = 0, string CategoryId = null, string Day = null)
        {
            ActionResult loginActionResult = UserAuthentication();
            if (accessAccount.CurrentUserId == 0)
                return loginActionResult;

            var model = new ContentListModel();

            if (Session["PagedModelData"] != null || Session["CategoryId"] != null || Session["Date"] != null)
            {
                Session["PagedModelData"] = null;
                Session["CategoryId"] = null;
                Session["Date"] = null;
            }
            if (ClassId == 0)
            {

                SetMenuProperties("Build", "Content Tools", "All Content");
                LoadAllContent(ref model, page ?? 0, CategoryId, Day);
                model.OverViewClsContent = null;
                ViewBag.Parameter = ((!string.IsNullOrEmpty(CategoryId) ? "&CategoryId=" + CategoryId : "") + "" + (!string.IsNullOrEmpty(Day) ? "&Day=" + Day : ""));

            }
            else
            {
                model.PageRights = GetPageRights();
                model.OverViewClsContent = ContentRepository.GetContentsbyClassId(ClassId);
                ViewBag.ClassId = ClassId;
                Classes cls = ContentRepository.GetClassbyId(ClassId);
                SetOverviewProperties(cls.ClassName, "Contents(" + model.OverViewClsContent.Count + ")", "", "");
                SetMenuProperties("Dashboard", "", "");

                ViewBag.Parameter = ("&ClassId=" + ClassId);
                PopulatePagingData(model.OverViewClsContent.Count(), "Publisher", "AllContent", page);
                if (page <= 1)
                {
                    model.OverViewClsContent = model.OverViewClsContent.Take(PageSize).ToList();
                }
                else
                {
                    model.OverViewClsContent = model.OverViewClsContent.Skip(PageSize * (page.Value - 1)).Take(PageSize).ToList();

                }
                ViewBag.Title = "IntelSkill - AllContent";
            }

            if (ViewBag.IsPageDiscard == null)
            {
                ViewBag.IsPageDiscard = false;
            }
            if (TempData["EditContentModel"] != null)
            {
                var contentmodel = TempData["EditContentModel"] as ContentModel;
                model.Message = contentmodel.Message;
                model.MessageType = contentmodel.MessageType;

            }
            return View(model);
        }



        // Post: /Content & Categories/
        [HttpPost]
        [ValidateInput(false)]
        public ActionResult BuildContent(string btnSubmit, ContentModel model, FormCollection collection, string id, string CatgNameId)
        {
            ActionResult loginActionResult = UserAuthentication();
            if (accessAccount.CurrentUserId == 0)
                return loginActionResult;

            SetMenuProperties("Build", "Content Tools", "Build Content");
            List<int> CategoryId = new List<int>();
            if (collection.GetValues("chkCatg") != null)
            {
                int CatCount = Convert.ToInt32(collection.GetValues("chkCatg").Count());

                if (CatCount != 0)
                {
                    for (int i = 0; i < CatCount; i++)
                    {

                        int CatId = Convert.ToInt32(collection.GetValues("chkCatg")[i]);
                        CategoryId.Add(CatId);
                    }
                }
            }
            if (btnSubmit != "Add" || btnSubmit != "Save")
            {
                if (collection.GetValues("AllQuizzes") != null)
                {
                    if (collection.GetValues("AllQuizzes")[0].Count() > 0)
                    {
                        model.QuizId = Convert.ToInt32(collection.GetValues("AllQuizzes")[0]);
                    }
                }
            }

            model.Assigned_ClassList = new List<Classes>();
            model.Available_ClassList = new List<Classes>();
            int Assigned_ClassList_Count = (collection.GetValues("Assigned_ClassList.ClassId") != null) ? collection.GetValues("Assigned_ClassList.ClassId").Count() : 0;

            for (int i = 0; i < Assigned_ClassList_Count; i++)
            {
                model.Assigned_ClassList.Add(new Classes() { ClassId = Int32.Parse(collection.GetValues("Assigned_ClassList.ClassId")[i].ToString()) });
            }

            #region SaveDraft
            if (btnSubmit == "SaveDraft")
            {

                model.Status = 0;
                model.CreatedDate = DateTime.Now;
                model.PublishedDate = null;

                // model.PublisherId = 1;
                if (model.ContentTitle != null && model.ContentTitle != " Enter Title Here")
                {
                    if (model.PageDesc != null)
                    {
                        if (collection.GetValues("chkCatg") != null)
                        {
                            //string PublishDate = collection.GetValues("Publishon").FirstOrDefault();
                            CreateContent(model, CategoryId, accessAccount.PublisherUserId);
                        }
                        else
                        {
                            model.MessageType = "error";
                            model.Message = "Please select atleast one category";
                            BuildContentLoadData(model, collection);
                            model.PageRights = GetPageRights();
                            LoadQuizzes(collection, model);
                            LoadClasses(model, collection);
                            return View("BuildContent", model);

                        }
                        if (model.IsContentTitle == true)
                        {
                            model.MessageType = "warning";
                            model.Message = "Content title already exists";
                            BuildContentLoadData(model, collection);
                            model.PageRights = GetPageRights();
                            model.IsContentTitle = false;
                            LoadQuizzes(collection, model);
                            LoadClasses(model, collection);
                            return View("BuildContent", model);
                        }
                        else
                        {
                            model.MessageType = "success";
                            model.Message = "Content saved in draft";
                            model.PageRights = GetPageRights();
                            if (model.PageRights._edit_content == true)
                            {

                                TempData["ContentModel"] = model;
                                return RedirectToAction("EditContent", "Publisher", new { ContentId = model.ContentId });
                            }
                            else if (model.PageRights._edit_content == false)
                            {


                                BuildContentLoadData(model, collection);

                                model.ContentTitle = null;
                                model.PageDesc = null;
                                model.IsCategorySelectId = null;
                                ModelState.Clear();
                                collection = null;
                                LoadQuizzes(collection, model);
                                LoadClasses(model, collection);
                                return View("BuildContent", model);
                            }
                        }
                    }
                    else
                    {

                        model.MessageType = "error";
                        model.Message = "Please enter page description";
                        BuildContentLoadData(model, collection);
                        model.PageRights = GetPageRights();
                        LoadQuizzes(collection, model);
                        LoadClasses(model, collection);
                        return View("BuildContent", model);
                    }
                }

                else
                {

                    model.MessageType = "error";
                    model.Message = "Please enter content title";
                    BuildContentLoadData(model, collection);
                    model.PageRights = GetPageRights();
                    LoadQuizzes(collection, model);
                    LoadClasses(model, collection);
                    return View("BuildContent", model);
                }


            }
            #endregion

            #region Publish
            if (btnSubmit == "Publish")
            {

                model.Status = 1;
                // model.PublisherId = 1;
                model.PublishedUserId = accessAccount.CurrentUserId;
                model.PublishedDate = DateTime.Now.Date;
                model.CreatedDate = DateTime.Now;
                if (model.ContentTitle != null && model.ContentTitle != " Enter Title Here")
                {
                    if (model.PageDesc != null)
                    {
                        if (collection.GetValues("chkCatg") != null)
                        {

                            CreateContent(model, CategoryId, accessAccount.PublisherUserId);
                        }
                        else
                        {
                            model.MessageType = "error";
                            model.Message = "Please select atleast one category";
                            BuildContentLoadData(model, collection);
                            model.PageRights = GetPageRights();
                            LoadQuizzes(collection, model);
                            LoadClasses(model, collection);
                            return View("BuildContent", model);
                        }
                        if (model.IsContentTitle == true)
                        {
                            model.MessageType = "warning";
                            model.Message = "Content title already exist";
                            BuildContentLoadData(model, collection);
                            model.PageRights = GetPageRights();
                            model.IsContentTitle = false;
                            LoadQuizzes(collection, model);
                            LoadClasses(model, collection);
                            return View("BuildContent", model);
                        }
                        else
                        {
                            model.MessageType = "success";
                            model.Message = "Content published sucessfully";
                            model.PageRights = GetPageRights();
                            if (model.PageRights._edit_content == true)
                            {
                                TempData["ContentModel"] = model;
                                return RedirectToAction("EditContent", "Publisher", new { ContentId = model.ContentId });
                            }
                            else if (model.PageRights._edit_content == false)
                            {

                                BuildContentLoadData(model, collection);
                                model.ContentTitle = null;
                                model.PageDesc = null;
                                model.IsCategorySelectId = null;
                                ModelState.Clear();
                                collection = null;
                                LoadQuizzes(collection, model);
                                LoadClasses(model, collection);
                                return View("BuildContent", model);
                            }
                        }
                    }
                    else
                    {
                        model.MessageType = "error";
                        model.Message = "Please enter page description";
                        BuildContentLoadData(model, collection);
                        model.PageRights = GetPageRights();
                        model.IsPageEdit = true;
                        LoadQuizzes(collection, model);
                        LoadClasses(model, collection);
                        return View("BuildContent", model);
                    }
                }
                else
                {
                    model.MessageType = "error";
                    model.Message = "Please enter content title";
                    BuildContentLoadData(model, collection);
                    model.PageRights = GetPageRights();
                    LoadQuizzes(collection, model);
                    LoadClasses(model, collection);
                    return View("BuildContent", model);
                }

            }
            #endregion

            #region Preview

            if (btnSubmit == "Preview")
            {
                Session["ContentPreview"] = null;
                if (model.ContentTitle != null && model.ContentTitle != " Enter Title Here")
                {
                    if (model.PageDesc != null)
                    {

                        List<string> ContPages = LoadContentPages(model);
                        model.ContentPagingList = new List<ContentPages>();
                        for (int j = 1; j <= ContPages.Count(); j++)
                        {
                            ContentPages pageObj = new ContentPages();
                            pageObj.ContentPageId = j;
                            pageObj.PageDescription = ContPages[j - 1].ToString();
                            pageObj.ContentId = -1;
                            model.ContentPagingList.Add(pageObj);
                        }


                        // _previewModel.PageDesc = str;

                        //string filterstring = GetFirstSecondParagraph(simplestr);
                        // _previewModel.PageDesc = filterstring;

                        if (ContPages.Count > 0)
                        {
                            model.PageDesc = ContPages[0].ToString();
                        }
                        model.ContentPageId = 1;
                        ViewBag.IsPreview = "yes";


                    }
                    else
                    {
                        model.MessageType = "error";
                        model.Message = "Please enter page description";
                        BuildContentLoadData(model, collection);
                        model.PageRights = GetPageRights();
                        LoadQuizzes(collection, model);
                        LoadClasses(model, collection);
                        return View("BuildContent", model);
                    }
                }
                else
                {
                    model.MessageType = "error";
                    model.Message = "Please enter content title";
                    BuildContentLoadData(model, collection);
                    model.PageRights = GetPageRights();
                    LoadQuizzes(collection, model);
                    LoadClasses(model, collection);
                    return View("BuildContent", model);
                }

                Session["ContentPreview"] = model;
                model.Form = new Categories();
                SetupCategoriesModel(model);
                if (collection.GetValues("Publishon") != null)
                {
                    model.PublishOn = collection.GetValues("Publishon").FirstOrDefault();
                }
                var ch = collection.GetValues("chkCatg");
                if (ch != null)
                {
                    List<int> catid = new List<int>();
                    foreach (var chkid in ch)
                    {

                        int selectChkid = Convert.ToInt32(chkid);
                        catid.Add(selectChkid);
                        model.IsCategorySelectId = catid;
                    }
                }

                LoadQuizzes(collection, model);
                LoadClasses(model, collection);
                model.PageRights = GetPageRights();
                return View("BuildContent", model);

            }
            #endregion

            #region Category Edit,Add,Delete
            if (btnSubmit == null)
            {
                if (id == "AddCategory")
                {

                    model.IsCategory = true;
                    BuildContentLoadData(model, collection);
                    model.PageRights = GetPageRights();
                    LoadQuizzes(collection, model);
                    LoadClasses(model, collection);
                    return View("BuildContent", model);

                }
                else if (id != null)
                {
                    int CatgId = Convert.ToInt32(id);
                    if (CatgId != 0)
                    {
                        List<int> CatId = ContentRepository.GetCategoryContentId(CatgId);
                        int Catgidcount = CatId.Count();
                        if (Catgidcount == 0)
                        {
                            ContentRepository.DeleteCategories(CatgId);
                            ContentRepository.Save();
                            model.MessageType = "success";
                            model.Message = "Category deleted successfully";
                            BuildContentLoadData(model, collection);
                            model.PageRights = GetPageRights();
                            LoadQuizzes(collection, model);
                            LoadClasses(model, collection);
                            return View("BuildContent", model);
                        }
                        else
                        {
                            model.MessageType = "warning";
                            model.Message = "Cannot delete this category because its added in content";
                            BuildContentLoadData(model, collection);
                            model.PageRights = GetPageRights();
                            LoadQuizzes(collection, model);
                            LoadClasses(model, collection);
                            return View("BuildContent", model);

                        }
                    }
                    BuildContentLoadData(model, collection);
                    LoadQuizzes(collection, model);
                    LoadClasses(model, collection);
                    model.PageRights = GetPageRights();
                    return View(model);
                }
            }
            if (btnSubmit == null)
            {
                if (CatgNameId != null)
                {
                    model.IsCatgEdit = true;
                    model.Form = new Categories();
                    SetupCategoriesModel(model);
                    int CatgId = Convert.ToInt32(CatgNameId);
                    Categories CatgName = ContentRepository.GetCategorybyId(CatgId);
                    model.Form.CategoryName = CatgName.CategoryName;
                    model.Form.CategoryId = CatgId;
                    int ContId = ContentRepository.GetContentId(model.ContentTitle);
                    model.PublishOn = collection.GetValues("Publishon").FirstOrDefault();
                    if (Session["ContentModel"] != null)
                    {
                        ContentModel TempModel = (ContentModel)Session["ContentModel"];
                        model.ContentPagingList = TempModel.ContentPagingList;
                    }
                    var ch = collection.GetValues("chkCatg");
                    if (ch != null)
                    {
                        List<int> catid = new List<int>();
                        foreach (var chkid in ch)
                        {

                            int selectChkid = Convert.ToInt32(chkid);
                            catid.Add(selectChkid);
                            model.IsCategorySelectId = catid;
                        }
                    }
                    // model.Form = new Categories();
                    SetupCategoriesModel(model);
                    ViewBag.ContentStatus = "New";
                    model.PageRights = GetPageRights();
                    LoadQuizzes(collection, model);
                    LoadClasses(model, collection);
                    return View("BuildContent", model);
                }
            }
            if (btnSubmit == "Add")
            {
                if (model.Form.CategoryName != null)
                {
                    int CatgId = ContentRepository.GetCategoryId(model.Form.CategoryName.Trim(), accessAccount.PublisherUserId);
                    if (CatgId == 0)
                    {
                        model.Form.PublisherUserId = accessAccount.PublisherUserId;
                        model.Form.CreatedUserId = accessAccount.CurrentUserId;
                        ContentRepository.InsertCategories(model.Form);
                        ContentRepository.Save();
                        model.MessageType = "success";
                        model.Message = "Category created successfully";

                        BuildContentLoadData(model, collection);
                        model.PageRights = GetPageRights();
                        LoadQuizzes(collection, model);
                        LoadClasses(model, collection);
                        return View("BuildContent", model);
                    }
                    else
                    {
                        model.MessageType = "warning";
                        model.Message = "Category already exists";
                        BuildContentLoadData(model, collection);
                        model.PageRights = GetPageRights();
                        LoadQuizzes(collection, model);
                        LoadClasses(model, collection);
                        return View("BuildContent", model);
                    }
                }
                BuildContentLoadData(model, collection);
                model.PageRights = GetPageRights();
                LoadQuizzes(collection, model);
                LoadClasses(model, collection);
                return View(model);
            }
            if (btnSubmit == "Save")
            {
                if (model.Form.CategoryName != null)
                {
                    int CatgId = ContentRepository.GetCategoryId(model.Form.CategoryName.Trim(), accessAccount.PublisherUserId);
                    if (CatgId == 0)
                    {
                        int Catgid = model.Form.CategoryId;
                        Categories Catg = ContentRepository.GetCategorybyId(Catgid);
                        Catg.PublisherUserId = accessAccount.PublisherUserId;
                        Catg.ModifiedUserId = accessAccount.CurrentUserId;
                        Catg.CategoryName = model.Form.CategoryName;
                        ContentRepository.UpdateCategories(Catg);
                        ContentRepository.Save();
                        model.MessageType = "success";
                        model.Message = "Category edited successfully";
                        BuildContentLoadData(model, collection);
                        model.PageRights = GetPageRights();
                        LoadQuizzes(collection, model);
                        LoadClasses(model, collection);
                        return View("BuildContent", model);
                    }
                    else if (model.Form.CategoryName != model.Form.CategoryName)
                    {
                        model.MessageType = "warning";
                        model.Message = "Category already exists";
                        BuildContentLoadData(model, collection);
                        model.PageRights = GetPageRights();
                        LoadQuizzes(collection, model);
                        LoadClasses(model, collection);
                        return View("BuildContent", model);
                    }
                }

            }
            #endregion

            BuildContentLoadData(model, collection);
            model.PageRights = GetPageRights();
            LoadQuizzes(collection, model);
            LoadClasses(model, collection);
            return View(model);
        }
        [HttpPost]
        [ValidateInput(false)]
        public ActionResult EditContent(string btnSubmit, ContentModel model, int ContentId, FormCollection collection, string CatgId, string CatgNameId)
        {
            ActionResult loginActionResult = UserAuthentication();
            if (accessAccount.CurrentUserId == 0)
                return loginActionResult;
            SetMenuProperties("Build", "Content Tools", "Edit Content");

            List<int> CategoryId = new List<int>();
            if (collection.GetValues("chkCatg") != null)
            {
                int CatCount = Convert.ToInt32(collection.GetValues("chkCatg").Count());
                if (CatCount != 0)
                {
                    for (int i = 0; i < CatCount; i++)
                    {

                        int CatId = Convert.ToInt32(collection.GetValues("chkCatg")[i]);
                        CategoryId.Add(CatId);
                    }
                }
            }

            if (btnSubmit != "Add" || btnSubmit != "Save")
            {
                if (collection.GetValues("AllQuizzes") != null)
                {
                    if (collection.GetValues("AllQuizzes")[0].Count() > 0)
                    {
                        model.QuizId = Convert.ToInt32(collection.GetValues("AllQuizzes")[0]);
                    }
                    else
                    {
                        model.QuizId = 0;
                    }
                }
            }

            model.Assigned_ClassList = new List<Classes>();
            model.Available_ClassList = new List<Classes>();
            int Assigned_ClassList_Count = (collection.GetValues("Assigned_ClassList.ClassId") != null) ? collection.GetValues("Assigned_ClassList.ClassId").Count() : 0;

            for (int i = 0; i < Assigned_ClassList_Count; i++)
            {
                model.Assigned_ClassList.Add(new Classes() { ClassId = Int32.Parse(collection.GetValues("Assigned_ClassList.ClassId")[i].ToString()) });
            }

            #region SaveDraft
            if (btnSubmit == "SaveDraft")
            {


                if (model.ContentTitle != null && model.ContentTitle != " Enter Title Here")
                {
                    if (model.PageDesc != null)
                    {

                        if (collection.GetValues("chkCatg") != null)
                        {
                            model.Status = 0;
                            model.PublishedDate = DateTime.MinValue;
                            string PublishDate = collection.GetValues("Publishon").FirstOrDefault();
                            UpdateContent(model, ContentId, CategoryId, PublishDate, accessAccount.PublisherUserId);
                        }
                        else
                        {
                            model.MessageType = "error";
                            model.Message = "Please select atleast one category";
                            EditContentLoadData(model, collection, ContentId);
                            return View("EditContent", model);
                        }
                        if (model.IsContentTitle == true)
                        {
                            model.MessageType = "warning";
                            model.Message = "Content title already exists";
                            EditContentLoadData(model, collection, ContentId);
                            model.IsContentTitle = false;
                            return View("EditContent", model);
                        }

                    }
                    else
                    {
                        model.MessageType = "error";
                        model.Message = "Page description is not there to preview";
                        EditContentLoadData(model, collection, ContentId);
                        return View("EditContent", model);
                    }
                }
                else
                {
                    model.MessageType = "error";
                    model.Message = "Please enter content title";
                    EditContentLoadData(model, collection, ContentId);
                    return View("EditContent", model);
                }
                EditContentLoadData(model, collection, ContentId);
                model.MessageType = "success";
                model.Message = "Content updated successfully";
                TempData["EditContentModel"] = model;
                return RedirectToAction("AllContent", "Publisher");
            }
            #endregion

            #region Publish
            if (btnSubmit == "Publish")
            {


                if (model.ContentTitle != null && model.ContentTitle != " Enter Title Here")
                {
                    if (model.PageDesc != null)
                    {

                        if (collection.GetValues("chkCatg") != null)
                        {

                            if (model.PublishOn == "Today")
                            {
                                model.Status = 1;
                                model.PublishedDate = DateTime.Now;
                            }
                            else
                            {
                                DateTime PublishDate = Convert.ToDateTime(model.PublishOn);
                                if (PublishDate.Date <= DateTime.Now.Date)
                                {
                                    model.Status = 1;
                                    model.PublishedDate = DateTime.Now;
                                }
                                else if (PublishDate.Date > DateTime.Now.Date)
                                {
                                    model.PublishedDate = PublishDate;
                                    model.Status = 2;
                                }

                            }
                            model.PublishedUserId = accessAccount.CurrentUserId;
                            UpdateContent(model, ContentId, CategoryId, null, accessAccount.PublisherUserId);
                            if (model.IsContentTitle == true)
                            {
                                model.MessageType = "warning";
                                model.Message = "Content title already exists";
                                EditContentLoadData(model, collection, ContentId);
                                model.IsContentTitle = false;
                                return View("EditContent", model);
                            }
                        }
                        else
                        {
                            model.MessageType = "error";
                            model.Message = "Please select atleast one category";
                            EditContentLoadData(model, collection, ContentId);
                            return View("EditContent", model);
                        }
                    }
                    else
                    {
                        model.MessageType = "error";
                        model.Message = "Page description is not there to preview";
                        EditContentLoadData(model, collection, ContentId);
                        return View("EditContent", model);
                    }
                }
                else
                {
                    model.MessageType = "error";
                    model.Message = "Please enter content title";
                    EditContentLoadData(model, collection, ContentId);
                    return View("EditContent", model);
                }
                EditContentLoadData(model, collection, ContentId);
                model.MessageType = "success";
                model.Message = "Content updated successfully";
                TempData["EditContentModel"] = model;
                return RedirectToAction("AllContent", "Publisher");
            }
            #endregion

            #region Preview
            if (btnSubmit == "Preview")
            {

                Session["ContentPreview"] = null;
                if (model.ContentTitle != null && model.ContentTitle != " Enter Title Here")
                {
                    if (model.PageDesc != null)
                    {

                        List<string> ContPages = LoadContentPages(model);
                        model.ContentPagingList = new List<ContentPages>();
                        for (int j = 1; j <= ContPages.Count(); j++)
                        {
                            ContentPages pageObj = new ContentPages();
                            pageObj.ContentPageId = j;
                            pageObj.PageDescription = ContPages[j - 1].ToString();
                            pageObj.ContentId = -1;
                            model.ContentPagingList.Add(pageObj);
                        }
                        if (ContPages.Count > 0)
                        {
                            model.PageDesc = ContPages[0].ToString();
                        }
                        model.ContentPageId = 1;
                        ViewBag.IsPreview = "yes";
                    }
                    else
                    {
                        model.MessageType = "error";
                        model.Message = "Page description is not there to preview";
                        EditContentLoadData(model, collection, ContentId);
                        return View("EditContent", model);
                    }
                }
                else
                {
                    model.MessageType = "error";
                    model.Message = "Please enter content title";
                    EditContentLoadData(model, collection, ContentId);
                    return View("EditContent", model);
                }
                Session["ContentPreview"] = model;
                EditContentLoadData(model, collection, ContentId);
                return View("EditContent", model);
            }


            #endregion

            #region Add,Edit,Delete Category
            if (btnSubmit == null)
            {
                if (CatgId == "AddCategory")
                {

                    model.IsCategory = true;
                    EditContentLoadData(model, collection, ContentId);
                    return View("EditContent", model);

                }
                else if (CatgId != null)
                {
                    int CatgyId = Convert.ToInt32(CatgId);
                    if (CatgyId != 0)
                    {
                        List<int> CatId = ContentRepository.GetCategoryContentId(CatgyId);
                        int Catgidcount = CatId.Count();
                        if (Catgidcount == 0)
                        {
                            ContentRepository.DeleteCategories(CatgyId);
                            ContentRepository.Save();
                            model.MessageType = "success";
                            model.Message = "Category deleted successfully";
                            EditContentLoadData(model, collection, ContentId);
                            return View("EditContent", model);
                        }
                        else
                        {
                            model.MessageType = "warning";
                            model.Message = "Cannot delete this category because its added in content";
                            EditContentLoadData(model, collection, ContentId);
                            return View("EditContent", model);
                        }
                    }

                    model.ContCategoriesList = ContentCategory(ContentId);
                    LoadQuizzes(collection, model);
                    LoadClasses(model, collection);
                    model.PageRights = GetPageRights();
                    return View("EditContent", model);
                }
            }
            if (btnSubmit == null)
            {
                if (CatgNameId != null)
                {
                    model.IsCatgEdit = true;
                    model.ContCategoriesList = ContentCategory(ContentId);

                    int CatgyId = Convert.ToInt32(CatgNameId);
                    Categories CatgName = ContentRepository.GetCategorybyId(CatgyId);
                    model.Form = new Categories();
                    model.Form.CategoryName = CatgName.CategoryName;
                    model.Form.CategoryId = CatgyId;
                    EditContentLoadData(model, collection, ContentId);
                    return View("EditContent", model);
                }
            }
            if (btnSubmit == "Add")
            {
                if (model.Form.CategoryName != null)
                {
                    int CatgyId = ContentRepository.GetCategoryId(model.Form.CategoryName.Trim(), accessAccount.PublisherUserId);
                    if (CatgyId == 0)
                    {
                        model.Form.PublisherUserId = accessAccount.PublisherUserId;
                        model.Form.CreatedUserId = accessAccount.CurrentUserId;
                        ContentRepository.InsertCategories(model.Form);
                        ContentRepository.Save();
                        model.MessageType = "success";
                        model.Message = "Category created successfully";
                        EditContentLoadData(model, collection, ContentId);
                        return View("EditContent", model);
                    }
                    else
                    {
                        model.MessageType = "warning";
                        model.Message = "Category already exists";
                        EditContentLoadData(model, collection, ContentId);
                        return View("EditContent", model);
                    }
                }
                model.ContCategoriesList = ContentCategory(ContentId);
                LoadQuizzes(collection, model);
                LoadClasses(model, collection);
                model.PageRights = GetPageRights();
                return View("EditContent", model);
            }
            if (btnSubmit == "Save")
            {
                if (model.Form.CategoryName != null)
                {
                    int CatgyId = ContentRepository.GetCategoryId(model.Form.CategoryName.Trim(), accessAccount.PublisherUserId);
                    if (CatgyId == 0)
                    {
                        int Catgid = model.Form.CategoryId;
                        Categories Catg = ContentRepository.GetCategorybyId(Catgid);
                        Catg.PublisherUserId = accessAccount.PublisherUserId;
                        Catg.ModifiedUserId = accessAccount.CurrentUserId;
                        Catg.CategoryName = model.Form.CategoryName;
                        ContentRepository.UpdateCategories(Catg);
                        ContentRepository.Save();
                        model.MessageType = "success";
                        model.Message = "Category edited successfully";
                        EditContentLoadData(model, collection, ContentId);
                        return View("EditContent", model);
                    }
                    else if (model.Form.CategoryName != model.Form.CategoryName)
                    {
                        model.MessageType = "warning";
                        model.Message = "Category already exists";
                        EditContentLoadData(model, collection, ContentId);
                        return View("EditContent", model);
                    }
                }

            }
            #endregion

            EditContentLoadData(model, collection, ContentId);
            return View("EditContent", model);
        }
        [HttpPost]
        public ActionResult AllContent(FormCollection collection, string btnSubmit, string ContentPreview, int? page = 1)
        {
            ActionResult loginActionResult = UserAuthentication();
            if (accessAccount.CurrentUserId == 0)
                return loginActionResult;
            SetMenuProperties("Build", "Content Tools", "All Content");

            var model = new ContentListModel();
            var ddlCategoryvalue = collection.GetValues("Titles")[0].ToString();
            var ddlDates = collection.GetValues("Dates")[0].ToString();
            if (btnSubmit == "Apply")
            {
                if (Request.Form["Actions"] == "Delete")
                {
                    if (collection.GetValues("chk") != null)
                    {
                        int j = Request.Form["chk"].Split(',').Count();
                        List<bool> retval = new List<bool>();
                        for (int i = 0; i < j; i++)
                        {
                            var ContentId = Convert.ToInt32(collection.GetValues("chk")[i]);

                            List<string> ContImageUrl = ContentRepository.GetContentImageUrl(ContentId);
                            if (ContImageUrl.Count > 0)
                            {
                                for (int ContImage = 0; ContImage < ContImageUrl.Count; ContImage++)
                                {
                                    string appPath = Server.MapPath("~/Uploads/ContentImageFolder" + "\\" + ContImageUrl[ContImage]);

                                    FileInfo fileInfo = new FileInfo(appPath);

                                    if (fileInfo.Exists)
                                    {
                                        fileInfo.Delete();
                                    }
                                }
                            }
                            bool ret = ContentRepository.DeleteContent(ContentId);
                            retval.Add(ret);
                            ContentRepository.Save();
                        }
                        if (retval.Contains(true))
                        {
                            model.MessageType = "success";
                            model.Message = "Content's deleted successfully.";
                            LoadAllContent(ref model, page.Value, ddlCategoryvalue, ddlDates);

                        }
                    }
                    else
                    {
                        model.MessageType = "warning";
                        model.Message = "Please select checkbox";
                        LoadAllContent(ref model, page.Value, ddlCategoryvalue, ddlDates);

                    }

                }
                else
                {
                    model.MessageType = "warning";
                    model.Message = "No action selected";
                    LoadAllContent(ref model, page.Value, ddlCategoryvalue, ddlDates);

                }

            }
            if (btnSubmit == "Filter")
            {


                LoadAllContent(ref model, page.Value, ddlCategoryvalue, ddlDates);

            }
            ViewBag.Parameter = ((!string.IsNullOrEmpty(ddlCategoryvalue) ? "&CategoryId=" + ddlCategoryvalue : "") + "" + (!string.IsNullOrEmpty(ddlDates) ? "&Day=" + ddlDates : ""));
            return View("AllContent", model);
        }


        // Function: /Content & Categories/
        private void SetupCategoriesModel(ContentModel model)
        {
            model.CategoriesSelectList = ContentRepository.GetCategories(accessAccount.PublisherUserId).ToList().OrderBy(m => m.CategoryName);

        }
        public void LoadQuizzes(FormCollection collection, ContentModel model)
        {


            List<SelectListItem> QuizName = new List<SelectListItem>();

            List<Quizzes> listquiz = QuizRepository.GetPublishedQuizzes(accessAccount.PublisherUserId).ToList();
            if (listquiz.Count > 0)
            {
                foreach (var quizName in listquiz)
                {
                    SelectListItem sl = new SelectListItem();
                    sl.Text = Convert.ToString(quizName.QuizTitle);
                    sl.Value = Convert.ToString(quizName.QuizId);
                    if (collection != null)
                    {
                        if (collection.GetValues("AllQuizzes") != null)
                        {
                            var ddlQuizvalue = collection.GetValues("AllQuizzes")[0].ToString();
                            if (!string.IsNullOrEmpty(ddlQuizvalue))
                            {

                                if (ddlQuizvalue == Convert.ToString(quizName.QuizId))
                                {
                                    sl.Selected = true;
                                }
                            }

                        }
                    }
                    if (model.QuizId != 0)
                    {
                        if (model.QuizId == quizName.QuizId)
                        {
                            sl.Selected = true;
                        }
                    }
                    QuizName.Add(sl);
                    ViewData["ddlAllQuizzes"] = QuizName;
                }
            }
            else
            {
                ViewData["ddlAllQuizzes"] = QuizName;
            }

        }
        public void LoadClasses(ContentModel model, FormCollection collection)
        {
            model.Assigned_ClassList = new List<Classes>();
            model.Available_ClassList = new List<Classes>();

            if (collection.GetValues("Assigned_ClassList.ClassId") != null || collection.GetValues("Available_ClassList.ClassId") != null)
            {
                int Assigned_ClassList_Count = (collection.GetValues("Assigned_ClassList.ClassId") != null) ? collection.GetValues("Assigned_ClassList.ClassId").Count() : 0;
                int Available_ClassList_Count = (collection.GetValues("Available_ClassList.ClassId") != null) ? collection.GetValues("Available_ClassList.ClassId").Count() : 0;

                for (int i = 0; i < Assigned_ClassList_Count; i++)
                {
                    int ClassId = Int32.Parse(collection.GetValues("Assigned_ClassList.ClassId")[i].ToString());
                    model.Assigned_ClassList.Add(ContentRepository.GetClassbyId(ClassId));
                }

                for (int i = 0; i < Available_ClassList_Count; i++)
                {
                    int ClassId = Int32.Parse(collection.GetValues("Available_ClassList.ClassId")[i].ToString());
                    model.Available_ClassList.Add(ContentRepository.GetClassbyId(ClassId));
                }
            }
            else
            {
                if (model.ContentId == 0)
                {
                    model.Available_ClassList = ContentRepository.GetAvailableClasses(accessAccount.PublisherUserId);
                    model.Assigned_ClassList = new List<Classes>();
                }
                else
                {
                    model.Assigned_ClassList = ContentRepository.GetAssignedClasses(model.ContentId);
                    model.Available_ClassList = ContentRepository.GetAvailableClasses(accessAccount.PublisherUserId).Except(model.Assigned_ClassList).ToList();
                }
            }


        }
        public void BuildContentLoadData(ContentModel model, FormCollection collection)
        {
            model.StatusName = "New";
            model.Form = new Categories();
            SetupCategoriesModel(model);
            if (collection.GetValues("Publishon") != null)
            {
                model.PublishOn = collection.GetValues("Publishon").FirstOrDefault();
            }
            if (Session["ContentModel"] != null)
            {
                ContentModel TempModel = (ContentModel)Session["ContentModel"];
                if (TempModel.ContentPagingList != null)
                {
                    model.ContentPagingList = TempModel.ContentPagingList;
                    int TempCount = TempModel.ContentPagingList.Count();
                    ContentPages result = (from cpl in TempModel.ContentPagingList where (cpl.ContentPageId == TempCount && cpl.ContentId == -1) select cpl).FirstOrDefault();
                    if (result != null)
                    {
                        TempModel.ContentPagingList.Remove(result);

                    }
                    model.ContentPagingList = TempModel.ContentPagingList;
                }
                if (TempModel.ContentPageSelectList != null)
                {
                    model.ContentPageSelectList = TempModel.ContentPageSelectList;
                    int TempCount = TempModel.ContentPageSelectList.Count();
                    TempModel.ContentPagingList = TempModel.ContentPageSelectList as List<ContentPages>;
                    ContentPages result = (from cpl in TempModel.ContentPagingList where (cpl.ContentPageId == TempCount && cpl.ContentId == -1) select cpl).FirstOrDefault();
                    if (result != null)
                    {
                        TempModel.ContentPagingList.Remove(result);

                    }
                    model.ContentPageSelectList = TempModel.ContentPagingList;
                }
            }


            var ch = collection.GetValues("chkCatg");
            if (ch != null)
            {
                List<int> catid = new List<int>();
                foreach (var chkid in ch)
                {

                    int selectChkid = Convert.ToInt32(chkid);
                    catid.Add(selectChkid);
                    model.IsCategorySelectId = catid;
                }
            }

        }
        public void EditContentLoadData(ContentModel model, FormCollection collection, int ContId)
        {
            var ch = collection.GetValues("chkCatg");
            if (ch != null)
            {
                List<int> catid = new List<int>();
                foreach (var chkid in ch)
                {

                    int selectChkid = Convert.ToInt32(chkid);
                    catid.Add(selectChkid);
                    model.IsCategorySelectId = catid;
                }
            }
            if (collection.GetValues("Publishon") != null)
            {
                model.PublishOn = collection.GetValues("Publishon").FirstOrDefault();
            }
            Contents Contdetail = ContentRepository.GetContentListById(ContId).FirstOrDefault();
            if (Contdetail != null)
            {
                model.Status = Contdetail.ContentStatus.Value;
                if (model.Status == 0)
                {
                    model.StatusName = "Draft";
                }
                else if (model.Status == 1)
                {

                    model.StatusName = "Published";
                }
                else if (model.Status == 2)
                {
                    model.StatusName = "Yet to Publish";

                }
            }
            if (model.StatusName != null)
            {
                ViewBag.ContentStatus = model.StatusName;
            }
            if (Session["EditContentModel"] != null)
            {
                ContentModel TempModel = (ContentModel)Session["EditContentModel"];
                if (TempModel.ContentPagingList != null)
                {
                    model.ContentPagingList = TempModel.ContentPagingList;

                }
            }


            model.ContCategoriesList = ContentCategory(ContId).ToList().OrderBy(m => m.CategoryName);
            model.PageRights = GetPageRights();
            LoadQuizzes(collection, model);
            LoadClasses(model, collection);
        }
        public List<ContentModel.CategoryList> ContentCategory(int Id)
        {
            var modelList = new List<ContentModel.CategoryList>();

            var Category = ContentRepository.GetCategories(accessAccount.PublisherUserId);
            var ContentCatID = ContentRepository.GetContentCategory(Id);
            foreach (var CategorytList in Category)
            {
                var model = new ContentModel.CategoryList();
                model.CategoryName = CategorytList.CategoryName;
                model.CategoryId = CategorytList.CategoryId;
                foreach (var ContentCatgID in ContentCatID)
                {
                    if (ContentCatgID.CategoryId == CategorytList.CategoryId)
                    {
                        model.ContCat = true;
                    }
                }
                modelList.Add(model);
            }


            return modelList;
        }
        public void CreateContent(ContentModel Model, List<int> CategoryId, int UserId)
        {
            Contents Cont = new Contents();
            ContentPages ContPages = new ContentPages();
            ContentCategories ContCatg = new ContentCategories();
            // ContentAttachments ContAtt = new ContentAttachments();

            Contents Contdetails = ContentRepository.GetContentList(Model.ContentTitle, UserId).FirstOrDefault();

            if (Contdetails != null)
            {
                Model.IsContentTitle = true;

            }
            else
            {
                if (Model.ContentTitle != null)
                {
                    Cont.ContentTitle = Model.ContentTitle;
                    Cont.CreatedDate = DateTime.Now;
                    Cont.ModifiedDate = null;
                    if (Model.PublishedDate != null)
                    {
                        if (!string.IsNullOrEmpty(Model.PublishOn))
                        {
                            if (Model.PublishOn == "Today")
                            {
                                DateTime PublishDate = DateTime.Now;
                                Cont.PublishedDate = PublishDate.Date;
                            }
                            else
                            {
                                DateTime PublishDate = Convert.ToDateTime(Model.PublishOn);
                                if (PublishDate.Date == DateTime.Now.Date)
                                {
                                    Model.Status = 1;
                                    Model.PublishedDate = DateTime.Now;
                                    Cont.PublishedDate = Model.PublishedDate;
                                }
                                else if (PublishDate.Date > DateTime.Now.Date)
                                {

                                    Model.PublishedDate = PublishDate;
                                    Cont.PublishedDate = Model.PublishedDate;
                                    Model.Status = 2; //Yet to Publish
                                }


                            }


                        }
                        else
                        {
                            Cont.PublishedDate = null;
                        }
                    }
                    else
                    {
                        Cont.PublishedDate = Model.PublishedDate;
                    }
                    Cont.ContentStatus = Model.Status;
                    if (Model.QuizId != 0)
                    {
                        Cont.QuizId = Model.QuizId;
                    }
                    Cont.PublisherUserId = accessAccount.PublisherUserId;
                    Cont.CreatedUserId = accessAccount.CurrentUserId;

                    Cont.PublishedUserId = Model.PublishedUserId;

                    ContentRepository.InsertContent(Cont);
                    ContentRepository.Save();

                    List<string> ContPagesList = LoadContentPages(Model);
                    if (ContPagesList.Count > 0)
                    {
                        for (int j = 1; j <= ContPagesList.Count(); j++)
                        {

                            ContPages.PageDescription = ContPagesList[j - 1].ToString();
                            ContPages.ContentId = Cont.ContentId;
                            ContentRepository.InsertContentPage(ContPages);
                            ContentRepository.Save();
                            ViewBag.IsPageDiscard = false;
                        }
                    }
                    else
                    {
                        if (Model.PageDesc != null)
                        {
                            ContPages.PageDescription = Model.PageDesc;
                            ContPages.ContentId = Cont.ContentId;
                            ContentRepository.InsertContentPage(ContPages);
                            ContentRepository.Save();
                            ViewBag.IsPageDiscard = false;
                        }
                    }
                    if (CategoryId != null)
                    {
                        foreach (int CatgId in CategoryId)
                        {
                            ContCatg.CategoryId = CatgId;
                            ContCatg.ContentId = Cont.ContentId;
                            ContentRepository.InsertContCategories(ContCatg);
                            ContentRepository.Save();
                        }
                    }
                    Model.ContentId = Cont.ContentId;
                    if (Session["UploadContentImages"] != null)
                    {
                        ContentModel.UploadImages UploadModel = new ContentModel.UploadImages();
                        UploadModel = (ContentModel.UploadImages)Session["UploadContentImages"];
                        for (int i = 0; i < UploadModel.FileName.Count; i++)
                        {
                            ContentAttachments ContAtt = new ContentAttachments();
                            ContAtt.ContentId = Cont.ContentId;
                            ContAtt.ContentFilename = UploadModel.FileName[i];
                            ContAtt.FolderFileName = UploadModel.FolderName[i];
                            ContentRepository.InsertContentPageAttach(ContAtt);
                            ContentRepository.Save();
                        }
                    }

                    if (Model.Assigned_ClassList != null && Model.Assigned_ClassList.Count > 0)
                    {
                        foreach (var Clss in Model.Assigned_ClassList)
                        {
                            ClassContents ClsCont = new ClassContents();
                            ClsCont.ClassId = Clss.ClassId;
                            ClsCont.ContentId = Cont.ContentId;
                            ContentRepository.InsertClassCont(ClsCont);
                            ContentRepository.Save();
                        }
                    }

                    //if (File != null)
                    //{
                    //    //string path = AppDomain.CurrentDomain.BaseDirectory + "Content/" + "Files/";
                    //    string Filename = File.FileName;
                    //    //var fileName = DateTime.Today.ToString("yy.MM.dd") + Path.GetFileName(file.FileName);
                    //    //var path = Path.Combine(Server.MapPath("~/Attachments"), fileName);
                    //    File.SaveAs(Server.MapPath("Content/Files/" + Filename));

                    //   int ContPageId= ContentRepository.GetContPageId(Cont.ContentId);
                    //   ContAtt.FolderPath = Server.MapPath("Content/Files/" + Filename);
                    //   ContAtt.ContentPageId = ContPageId;
                    //   ContAtt.Filename = Filename;
                    //   ContentRepository.InsertContentPageAttach(ContAtt);
                    //   ContentRepository.Save();
                    //}
                }
            }


        }
        public void UpdateContent(ContentModel Model, int ContId, List<int> CategoryId, string Publishon, int UserId)
        {
            Contents Cont = new Contents();
            ContentPages ContPages = new ContentPages();
            ContentCategories ContCatg = new ContentCategories();

            Contents Contdetails = ContentRepository.GetContentListById(ContId).FirstOrDefault();
            Contents Conts = ContentRepository.GetContentList(Model.ContentTitle, UserId).FirstOrDefault();

            if ((Conts != null) && (Contdetails.ContentTitle != Model.ContentTitle))
            {
                Model.IsContentTitle = true;
            }

            else
            {
                Contdetails.ContentTitle = Model.ContentTitle;
                Contdetails.ContentStatus = Model.Status;
                Contdetails.PublishedUserId = Model.PublishedUserId;

                Contdetails.ModifiedDate = DateTime.Now;
                if (Model.PublishedDate == DateTime.MinValue)
                {


                    Contdetails.PublishedDate = null;

                }
                else
                {
                    Contdetails.PublishedDate = Model.PublishedDate;
                }
                if (Model.QuizId != 0)
                {
                    Contdetails.QuizId = Model.QuizId;
                }
                else
                {
                    Contdetails.QuizId = null;
                }
                Contdetails.ModifiedUserId = accessAccount.CurrentUserId;
                ContentRepository.UpdateContent(Contdetails);
                ContentRepository.Save();

                IEnumerable<ContentPages> ContPagesdb = ContentRepository.GetContentPages(ContId);
                if (ContPagesdb != null)
                {
                    foreach (var ContPageId in ContPagesdb)
                    {
                        ContentRepository.DeleteContPagebyId(ContPageId.ContentPageId);
                        ContentRepository.Save();
                    }
                    List<string> ContPagesList = LoadContentPages(Model);
                    if (ContPagesList.Count > 0)
                    {
                        for (int j = 1; j <= ContPagesList.Count(); j++)
                        {

                            ContPages.PageDescription = ContPagesList[j - 1].ToString();
                            ContPages.ContentId = ContId;
                            ContentRepository.InsertContentPage(ContPages);
                            ContentRepository.Save();
                            ViewBag.IsPageDiscard = false;
                        }
                    }
                    else
                    {
                        if (Model.PageDesc != null)
                        {
                            ContPages.PageDescription = Model.PageDesc;
                            ContPages.ContentId = ContId;
                            ContentRepository.InsertContentPage(ContPages);
                            ContentRepository.Save();
                            ViewBag.IsPageDiscard = false;
                        }
                    }
                }


                ContentRepository.DeleteContCategories(ContId);
                ContentRepository.Save();
                // var ContentCatID = ContentRepository.GetContentCategory(ContId);
                foreach (int CatgId in CategoryId)
                {
                    ContCatg.CategoryId = CatgId;
                    ContCatg.ContentId = ContId;
                    ContentRepository.InsertContCategories(ContCatg);
                    ContentRepository.Save();
                }
                if (Session["UploadContentImages"] != null)
                {
                    ContentModel.UploadImages UploadModel = new ContentModel.UploadImages();
                    UploadModel = (ContentModel.UploadImages)Session["UploadContentImages"];
                    for (int i = 0; i < UploadModel.FileName.Count; i++)
                    {
                        ContentAttachments ContAtt = new ContentAttachments();
                        ContAtt.ContentId = ContId;
                        ContAtt.ContentFilename = UploadModel.FileName[i];
                        ContAtt.FolderFileName = UploadModel.FolderName[i];
                        ContentRepository.InsertContentPageAttach(ContAtt);
                        ContentRepository.Save();
                    }
                }

                if (Model.Assigned_ClassList != null && Model.Assigned_ClassList.Count > 0)
                {
                    ContentRepository.DeleteClassCont(ContId);
                    ContentRepository.Save();
                    foreach (var Clss in Model.Assigned_ClassList)
                    {
                        ClassContents ClsCont = new ClassContents();
                        ClsCont.ClassId = Clss.ClassId;
                        ClsCont.ContentId = ContId;
                        ContentRepository.InsertClassCont(ClsCont);
                        ContentRepository.Save();
                    }
                }

            }
        }
        public List<string> LoadContentPages(ContentModel model)
        {
            List<string> ContPagesList = new List<string>();
            if (model.PageDesc != null)
            {
                string[] _contentPage = null;
                if (model.PageDesc.Contains("<div style=\"page-break-after: always;\">\r\n\t<span style=\"display: none;\">&nbsp;</span></div>"))
                {
                    _contentPage = model.PageDesc.Split(new string[] { "<div style=\"page-break-after: always;\">\r\n\t<span style=\"display: none;\">&nbsp;</span></div>" }, StringSplitOptions.RemoveEmptyEntries);
                    for (int i = 0; i < _contentPage.Count(); i++)
                    {
                        if (i != _contentPage.Count() - 1)
                        {
                            _contentPage[i] += "<div style=\"page-break-after: always;\">\r\n\t<span style=\"display: none;\">&nbsp;</span></div>";
                        }
                        ContPagesList.Add(_contentPage[i]);
                    }
                }
                else if (model.PageDesc.Contains("<div style=\"page-break-after: always\">\r\n\t<span style=\"display: none;\">&nbsp;</span></div>"))
                {
                    _contentPage = model.PageDesc.Split(new string[] { "<div style=\"page-break-after: always\">\r\n\t<span style=\"display: none;\">&nbsp;</span></div>" }, StringSplitOptions.RemoveEmptyEntries);
                    for (int i = 0; i < _contentPage.Count(); i++)
                    {
                        if (i != _contentPage.Count() - 1)
                        {
                            _contentPage[i] += "<div style=\"page-break-after: always\">\r\n\t<span style=\"display: none;\">&nbsp;</span></div>";
                        }
                        ContPagesList.Add(_contentPage[i]);
                    }
                }

            }

            return ContPagesList;
        }
        private void LoadAllContent(ref ContentListModel model, int page, string CategoryId, string Date)
        {
            model.PageRights = GetPageRights();

            model.TotalContentList = PropertiesToModel(CategoryId, Date);
            //if (page <= 1)
            //{
            //    model.PageContentList = model.TotalContentList.Take(PageSize).ToList();
            //}
            //else
            //{
            //    model.PageContentList = model.TotalContentList.Skip(PageSize * (page - 1)).Take(PageSize).ToList();
            //    if (model.PageContentList.Count == 0)
            //    {
            //        page = page - 1;
            //        model.PageContentList = model.TotalContentList.Skip(PageSize * (page - 1)).Take(PageSize).ToList();
            //    }
            //}

            //PopulatePagingData(model.TotalContentList.Count, "Publisher", "AllContent", page);


            Session["PagedModelData"] = model;
            ContentListLoadData(CategoryId, Date, ref model);
            Session["CategoryId"] = CategoryId;
            Session["Date"] = Date;
        }
        public List<ContentListModel> PropertiesToModel(string CategoryId, string Date)
        {
            var AllContents = ContentRepository.GetContentListbyUserId(accessAccount.PublisherUserId).OrderByDescending(m => m.ContentId).ToList();
            var modelList = new List<ContentListModel>();

            #region Default Load
            if (string.IsNullOrEmpty(CategoryId) && string.IsNullOrEmpty(Date))
            {
                foreach (var ContentList in AllContents)
                {
                    var model = new ContentListModel();
                    model.ContentTitle = ContentList.ContentTitle;


                    string CreatedDate = Convert.ToString(ContentList.CreatedDate);
                    if (string.IsNullOrEmpty(CreatedDate))
                    {
                        model.DateAdded = null;
                    }
                    else
                    {
                        DateTime createdate = Convert.ToDateTime(CreatedDate);
                        string date = String.Format("{0:MM/dd/yyyy}", createdate);
                        model.DateAdded = date;
                    }
                    model.Status = ContentList.ContentStatus.Value;
                    model.ContentId = ContentList.ContentId;


                    model.Category = ContentRepository.GetCategoryName(ContentList.ContentId);
                    string Username = ContentRepository.GetUsername(ContentList.PublisherUserId.Value);
                    if (Username != null)
                    {
                        model.Author = Username;
                    }
                    string PublishedDate = Convert.ToString(ContentList.PublishedDate);
                    if (string.IsNullOrEmpty(PublishedDate))
                    {
                        model.PublishedDate = null;
                    }
                    else
                    {

                        DateTime PublishDate = Convert.ToDateTime(PublishedDate);
                        if (PublishDate.Date <= DateTime.Now.Date)
                        {
                            string date = String.Format("{0:MM/dd/yyyy}", PublishDate);
                            model.PublishedDate = date;
                        }
                        else
                        {
                            model.PublishedDate = null;

                        }
                    }
                    modelList.Add(model);
                }
            }
            #endregion

            #region CategoryFilter
            else if ((!string.IsNullOrEmpty(CategoryId)) && (string.IsNullOrEmpty(Date)))
            {
                var ContentId = ContentRepository.GetCategoryContentId(Convert.ToInt32(CategoryId));
                foreach (var ContentList in AllContents)
                {
                    foreach (var ContId in ContentId)
                    {
                        if (ContentList.ContentId == ContId)
                        {
                            var model = new ContentListModel();
                            model.ContentTitle = ContentList.ContentTitle;


                            string CreatedDate = Convert.ToString(ContentList.CreatedDate);
                            if (string.IsNullOrEmpty(CreatedDate))
                            {
                                model.DateAdded = null;
                            }
                            else
                            {
                                DateTime createdate = Convert.ToDateTime(CreatedDate);
                                string date = String.Format("{0:MM/dd/yyyy}", createdate);
                                model.DateAdded = date;
                            }
                            model.Status = ContentList.ContentStatus.Value;
                            model.ContentId = ContentList.ContentId;


                            model.Category = ContentRepository.GetCategoryName(ContentList.ContentId);
                            string Username = ContentRepository.GetUsername(ContentList.PublisherUserId.Value);
                            if (Username != null)
                            {
                                model.Author = Username;
                            }
                            string PublishedDate = Convert.ToString(ContentList.PublishedDate);
                            if (string.IsNullOrEmpty(PublishedDate))
                            {
                                model.PublishedDate = null;
                            }
                            else
                            {

                                DateTime PublishDate = Convert.ToDateTime(PublishedDate);
                                if (PublishDate.Date <= DateTime.Now.Date)
                                {
                                    string date = String.Format("{0:MM/dd/yyyy}", PublishDate);
                                    model.PublishedDate = date;
                                }
                                else
                                {
                                    model.PublishedDate = null;

                                }
                            }
                            modelList.Add(model);
                        }
                    }

                }
            }
            #endregion

            #region DateFilter
            else if ((!string.IsNullOrEmpty(Date)) && (string.IsNullOrEmpty(CategoryId)))
            {
                foreach (var ContentList in AllContents)
                {
                    DateTime ContentCreated = Convert.ToDateTime(ContentList.CreatedDate);
                    if (ContentCreated >= DateTime.Now.AddDays(-(Convert.ToInt32(Date))))
                    {
                        var model = new ContentListModel();
                        model.ContentTitle = ContentList.ContentTitle;
                        string CreatedDate = Convert.ToString(ContentList.CreatedDate);
                        if (string.IsNullOrEmpty(CreatedDate))
                        {
                            model.DateAdded = null;
                        }
                        else
                        {
                            DateTime createdate = Convert.ToDateTime(CreatedDate);
                            string date = String.Format("{0:MM/dd/yyyy}", createdate);
                            model.DateAdded = date;
                        }

                        model.ContentId = ContentList.ContentId;


                        model.Category = ContentRepository.GetCategoryName(ContentList.ContentId);
                        string Username = ContentRepository.GetUsername(ContentList.PublisherUserId.Value);
                        if (Username != null)
                        {
                            model.Author = Username;
                        }
                        string PublishedDate = Convert.ToString(ContentList.PublishedDate);
                        if (string.IsNullOrEmpty(PublishedDate))
                        {
                            model.PublishedDate = null;
                        }
                        else
                        {

                            DateTime PublishDate = Convert.ToDateTime(PublishedDate);
                            if (PublishDate.Date <= DateTime.Now.Date)
                            {
                                string date = String.Format("{0:MM/dd/yyyy}", PublishDate);
                                model.PublishedDate = date;
                            }
                            else
                            {
                                model.PublishedDate = null;

                            }
                        }
                        modelList.Add(model);
                    }
                }
            }
            #endregion

            #region Date&CategoryFilter
            else if ((!string.IsNullOrEmpty(Date)) && (!string.IsNullOrEmpty(CategoryId)))
            {
                foreach (var ContentList in AllContents)
                {
                    DateTime ContentCreated = Convert.ToDateTime(ContentList.CreatedDate);
                    var ContentId = ContentRepository.GetCategoryContentId(Convert.ToInt32(CategoryId));
                    if (ContentCreated >= DateTime.Now.AddDays(-(Convert.ToInt32(Date))))
                    {
                        foreach (var ContId in ContentId)
                        {
                            if (ContentList.ContentId == ContId)
                            {
                                var model = new ContentListModel();
                                model.ContentTitle = ContentList.ContentTitle;
                                string CreatedDate = Convert.ToString(ContentList.CreatedDate);
                                if (string.IsNullOrEmpty(CreatedDate))
                                {
                                    model.DateAdded = null;
                                }
                                else
                                {
                                    DateTime createdate = Convert.ToDateTime(CreatedDate);
                                    string date = String.Format("{0:MM/dd/yyyy}", createdate);
                                    model.DateAdded = date;
                                }

                                model.ContentId = ContentList.ContentId;


                                model.Category = ContentRepository.GetCategoryName(ContentList.ContentId);
                                string Username = ContentRepository.GetUsername(ContentList.PublisherUserId.Value);
                                if (Username != null)
                                {
                                    model.Author = Username;
                                }
                                string PublishedDate = Convert.ToString(ContentList.PublishedDate);
                                if (string.IsNullOrEmpty(PublishedDate))
                                {
                                    model.PublishedDate = null;
                                }
                                else
                                {

                                    DateTime PublishDate = Convert.ToDateTime(PublishedDate);
                                    if (PublishDate.Date <= DateTime.Now.Date)
                                    {
                                        string date = String.Format("{0:MM/dd/yyyy}", PublishDate);
                                        model.PublishedDate = date;
                                    }
                                    else
                                    {
                                        model.PublishedDate = null;

                                    }
                                }
                                modelList.Add(model);
                            }
                        }
                    }
                }
            }
            #endregion

            return modelList;
        }
        public void ContentListLoadData(string selectedContentvalue, string selectedDateValue, ref ContentListModel model)
        {
            List<SelectListItem> CategoryName = new List<SelectListItem>();

            List<Categories> listCatg = ContentRepository.GetCategories(accessAccount.PublisherUserId).ToList();
            foreach (var CatgName in listCatg)
            {
                SelectListItem sl = new SelectListItem();
                sl.Text = Convert.ToString(CatgName.CategoryName);
                sl.Value = Convert.ToString(CatgName.CategoryId);


                if (selectedContentvalue == Convert.ToString(CatgName.CategoryId))
                {
                    sl.Selected = true;
                }

                CategoryName.Add(sl);
            }


            List<SelectListItem> Actions = new List<SelectListItem>();

            if (model.PageRights._delete_content == true)
            {
                Actions.Add(new SelectListItem
                {
                    Text = "Delete",
                    Value = "Delete"
                });
            }

            List<SelectListItem> Dates = new List<SelectListItem>();


            Dates.Add(new SelectListItem
            {
                Text = "Last 7 Days",
                Value = "7"

            });
            Dates.Add(new SelectListItem
            {
                Text = "Last 15 Days",
                Value = "15"
            });
            Dates.Add(new SelectListItem
            {
                Text = "Last 30 Days",
                Value = "30"
            });


            foreach (var item in Dates)
            {
                if (item.Value == selectedDateValue)
                {
                    item.Selected = true;
                    break;
                }
            }


            ViewData["ddlActions"] = Actions;
            ViewData["ddlContentCatg"] = CategoryName;
            ViewData["Dates"] = Dates;
        }

        #endregion

        #region Common

        public ActionResult LogOutRole()
        {
            if (Session["AccessModel"] != null)
                accessAccount = (AccessModel)Session["AccessModel"];

            ViewBag.IsAssigned = false; ViewBag.ClassId = 0; ViewBag.RoleId = 0;

            if (accessAccount != null)
            {
                Users userDetail = usersRepository.GetUserByID(accessAccount.CurrentUserId);

                accessAccount.PublisherUserId = accessAccount.CurrentUserId;
                accessAccount.PublisherCompanyLogo = userDetail.CompanyLogo;
                accessAccount.RoleId = 0;
                accessAccount.ClassId = 0;

                ViewBag.DisplayName = accessAccount.DisplayName;
                ViewBag.PublisherCompanyLogo = (!string.IsNullOrEmpty(accessAccount.PublisherCompanyLogo) ? "/Uploads/CompanyLogoFolder/" + accessAccount.PublisherCompanyLogo : "");
                ViewBag.CurrentUser_FirstName = accessAccount.CurrentUser_FirstName;
                ViewBag.CurrentUser_LastName = accessAccount.CurrentUser_LastName;

                if (!string.IsNullOrEmpty(accessAccount.AccessCode))
                {
                    ViewBag.IsAssigned = true;
                    ViewBag.ClassId = accessAccount.ClassId;
                }
                else
                {
                    ViewBag.RoleId = accessAccount.RoleId;
                }

            }
            else { accessAccount = new AccessModel(); }

            return RedirectToAction("Dashboard", "Publisher");

        }

        public ActionResult SignOut()
        {
            FormsAuthentication.SignOut();
            Session.Abandon();


            return RedirectToAction("Index", "Home");

        }

        private void SetMenuProperties(string mainmenu_name, string sidemenu_name, string pagenavigation_name)
        {
            ViewBag.Title = "IntelSkill - " + pagenavigation_name;

            ViewBag.MenuNavigation = mainmenu_name;
            ViewBag.SideMenuNavigation = sidemenu_name;
            ViewBag.PageNavigation = pagenavigation_name;


        }

        private void SetOverviewProperties(string OverviewClasses, string ClassContent, string ContentStudent, string StudentReport)
        {

            ViewBag.OverviewClasses = OverviewClasses;
            ViewBag.ClassContent = ClassContent;
            ViewBag.ContentStudent = ContentStudent;
            ViewBag.StudentReport = StudentReport;

        }

        private ActionResult UserAuthentication()
        {
            RedirectToRouteResult returnAction = RedirectToAction("SignIn", "Account");

            ViewBag.IsAssigned = false; ViewBag.ClassId = 0; ViewBag.RoleId = 0;

            if (Session["AccessModel"] != null)
                accessAccount = (AccessModel)Session["AccessModel"];

            if (accessAccount != null)
            {
                ViewBag.DisplayName = accessAccount.DisplayName;
                ViewBag.PublisherCompanyLogo = (!string.IsNullOrEmpty(accessAccount.PublisherCompanyLogo) ? "/Uploads/CompanyLogoFolder/" + accessAccount.PublisherCompanyLogo : "");
                ViewBag.CurrentUser_FirstName = accessAccount.CurrentUser_FirstName;
                ViewBag.CurrentUser_LastName = accessAccount.CurrentUser_LastName;
                ViewBag.PublisherUser_FirstName = accessAccount.PublisherUser_FirstName;
                ViewBag.PublisherUser_LastName = accessAccount.PublisherUser_LastName;
                if (!string.IsNullOrEmpty(accessAccount.AccessCode))
                {
                    ViewBag.IsAssigned = true;
                    ViewBag.ClassId = accessAccount.ClassId;
                }
                else
                {
                    ViewBag.RoleId = accessAccount.RoleId;
                }

            }
            else { accessAccount = new AccessModel(); return returnAction; }
            return null;
        }

        private PageRightsModel GetPageRights()
        {
            PageAction _dopage_Action = new PageAction();

            PageRightsModel rightsmodel = new PageRightsModel();

            rightsmodel = _dopage_Action._DoRightsAction(accessAccount.CurrentUserId, accessAccount.PublisherUserId, accessAccount.RoleId);

            //People : Admin, Student, User Permission
            ViewBag._edit_student = rightsmodel._edit_student;
            ViewBag._delete_student = rightsmodel._delete_student;
            ViewBag._add_new_student = rightsmodel._add_new_student;

            ViewBag._add_new_adminuser = rightsmodel._add_new_adminuser;
            ViewBag._edit_adminuser = rightsmodel._edit_adminuser;
            ViewBag._delete_adminuser = rightsmodel._delete_adminuser;

            ViewBag._view_user_permission = rightsmodel._view_user_permission;

            //Class Tools: Build Class, All Class

            ViewBag._add_new_private_class = rightsmodel._add_new_private_class;
            ViewBag._add_new_public_class = rightsmodel._add_new_public_class;

            ViewBag._view_all_private_class = rightsmodel._view_all_private_class;
            ViewBag._view_all_public_class = rightsmodel._view_all_public_class;

            //Quiz Tools : Build Question, All Questions, Build Quiz, All Quiz
            ViewBag._add_question = rightsmodel._add_question;
            ViewBag._edit_question = rightsmodel._edit_question;
            ViewBag._view_question = rightsmodel._view_question;
            ViewBag._delete_question = rightsmodel._delete_question;

            ViewBag._view_student = rightsmodel._view_student;
            ViewBag._view_user = rightsmodel._view_user;

            ViewBag._add_new_content = rightsmodel._add_new_content;
            ViewBag._build_quiz = rightsmodel._build_quiz;

            ViewBag._view_all_content_list = rightsmodel._view_all_content_list;
            ViewBag._view_all_quiz = rightsmodel._view_all_quiz;

            return rightsmodel;
        }

        // Validates Email Format.
        public static bool IsValidEmail(string email)
        {

            System.Text.RegularExpressions.Regex rx = new System.Text.RegularExpressions.Regex(
            @"^[-!#$%&'*+/0-9=?A-Z^_a-z{|}~](\.?[-!#$%&'*+/0-9=?A-Z^_a-z{|}~])*@[a-zA-Z](-?[a-zA-Z0-9])*(\.[a-zA-Z](-?[a-zA-Z0-9])*)+$");
            return rx.IsMatch(email);
        }

        // Validates Numeric Format.
        public static bool IsNumeric(string number)
        {

            System.Text.RegularExpressions.Regex rx = new System.Text.RegularExpressions.Regex(
            @"[0-9]+");
            return rx.IsMatch(number);
        }

        // Compares Two Passwords
        private bool IsPasswordMatch(string firstPassword, string secondPassword)
        {
            return (firstPassword == secondPassword);
        }

        #endregion
    }


}
