﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Data;
using System.Data.Entity.Validation;
using System.Web.Mvc;
using System.Transactions;
using IntelSkill.Repository;
using IntelSkill.DAL;
using IntelSkill.Models;

namespace IntelSkill.DAL
{
    public class ClassRepository : IClassRepository, IDisposable
    {
        public SQLDBContext sqldbcontext;

        public ClassRepository(SQLDBContext _sqldbcontext)
        {
            this.sqldbcontext = _sqldbcontext;
        }

        #region Build Class Page Functions.

        public Classes GetMyClass(int classId)
        {
            var classresult = sqldbcontext.Classes.Where(x => x.ClassId == classId).FirstOrDefault();
            if (classresult != null)
                return classresult;
            else
                return new Classes();
        }

        public List<Contents> GetMyContents(int userId)
        {
            var contentList = (from c in sqldbcontext.Contents
                               where c.PublisherUserId == userId
                               select c).ToList();


            if (contentList != null)
                return contentList;
            else
                return new List<Contents>();
        }

        public List<Contents> GetAssignedContents(int classId)
        {
            var contentList = (from cc in sqldbcontext.ClassContents.Where(cc => cc.ClassId == classId)
                               join c in sqldbcontext.Contents
                               on cc.ContentId equals c.ContentId
                               select c).ToList();


            if (contentList != null)
                return contentList;
            else
                return new List<Contents>();
        }

        public List<Users> GetMyStudents(int userId)
        {
            var userList = (from ru in sqldbcontext.RefUsers.Where(ru => ru.ParentUserId == userId && ru.RoleId == 3)
                            join u in sqldbcontext.Users
                            on ru.ChildUserId equals u.UserId
                            select u).ToList();


            if (userList != null)
                return userList;
            else
                return new List<Users>();
        }

        public List<Users> GetAssignedStudents(int classId)
        {
            var userList = (from cs in sqldbcontext.ClassSubscriptions.Where(cs => cs.ClassId == classId)
                            join u in sqldbcontext.Users
                            on cs.RequestedUserId equals u.UserId
                            where cs.RequestedDate == null
                            select u).ToList();


            if (userList != null)
                return userList;
            else
                return new List<Users>();
        }

        public int? GetUserIdByMail(string email)
        {
            return (from user in sqldbcontext.Users.Where(user => user.Email == email) select user.UserId).FirstOrDefault();
        }

        public List<Contents> GetContentList(List<int> previewContentIdList)
        {
            List<Contents> returnList = new List<Contents>();

            foreach (var item in previewContentIdList)
            {
                var results = (from content in sqldbcontext.Contents.Where(x => x.ContentId == item) select content).FirstOrDefault();
                returnList.Add(results);
            }


            return returnList;
        }

        public int SaveClass(BuildClassModel model)
        {
            using (TransactionScope tScope = new TransactionScope(TransactionScopeOption.Required))
            {

                if (model.ClassModel.ClassId == 0)
                {


                    #region INSERT CLASS, CONTENTS, STUDENTS


                    model.ClassModel = sqldbcontext.Classes.Add(model.ClassModel);
                    sqldbcontext.SaveChanges();

                    foreach (var content in model.Assigned_ContentList)
                    {
                        sqldbcontext.ClassContents.Add(new ClassContents() { ClassId = model.ClassModel.ClassId, ContentId = content.ContentId });
                       
                    } sqldbcontext.SaveChanges();

                    foreach (var student in model.Assigned_StudentList)
                    {
                        sqldbcontext.ClassSubscriptions.Add(new ClassSubscription() { ClassId = model.ClassModel.ClassId, RequestedUserId = student.UserId, SubscriptionDate = System.DateTime.Now});
                      
                    }  sqldbcontext.SaveChanges();
                    tScope.Complete();
                    return model.ClassModel.ClassId;

                    #endregion
                }
                else
                {
                    #region UPDATE CLASS, CLASS CONTENTS, CLASS SUBCRIPTION

                    var classDetail = sqldbcontext.Classes.Where(x => x.ClassId == model.ClassModel.ClassId).FirstOrDefault();

                    if (classDetail != null)
                    {
                        classDetail.ClassDescription = model.ClassModel.ClassDescription;
                        classDetail.ClassName = model.ClassModel.ClassName;
                        classDetail.ClassStatus = model.ClassModel.ClassStatus;
                        classDetail.ClassType = model.ClassModel.ClassType;
                        classDetail.IsProfitable = model.ClassModel.IsProfitable;
                        classDetail.ModifiedDate = model.ClassModel.ModifiedDate;
                        classDetail.ModifiedUserId = model.ClassModel.ModifiedUserId;
                        classDetail.ProfitAmount = model.ClassModel.ProfitAmount;
                        classDetail.PublishedDate = model.ClassModel.PublishedDate;
                        classDetail.PublishedUserId = model.ClassModel.PublishedUserId;


                        if (model.IsRequestMailEnabled)
                        {
                            classDetail.AccessCode = model.ClassModel.AccessCode;
                            classDetail.AccessCodePwd = model.ClassModel.AccessCodePwd;
                            classDetail.AssignedUserAttempts = model.ClassModel.AssignedUserAttempts;
                            classDetail.AssignedUserId = model.ClassModel.AssignedUserId;
                            classDetail.AssignedUserRequestDate = model.ClassModel.AssignedUserRequestDate;
                        }
                    }


                    sqldbcontext.Entry(classDetail).State = EntityState.Modified;

                    sqldbcontext.SaveChanges();


                    // Remove Contents

                    var classContents = sqldbcontext.ClassContents.Where(item => item.ClassId == model.ClassModel.ClassId);

                    foreach (var item in classContents)
                    {
                        sqldbcontext.ClassContents.Remove(item);
                      
                    }
  sqldbcontext.SaveChanges();
                    // Update Contents
                    foreach (var item in model.Assigned_ContentList)
                    {
                        sqldbcontext.ClassContents.Add(new ClassContents() { ClassId = model.ClassModel.ClassId, ContentId = item.ContentId });
                        
                    }
sqldbcontext.SaveChanges();


                    // Remove Subscription

                    var classSubscriptions = sqldbcontext.ClassSubscriptions.Where(item => item.ClassId == model.ClassModel.ClassId && item.RequestedDate == null);

                    foreach (var item in classSubscriptions)
                    {
                        sqldbcontext.ClassSubscriptions.Remove(item);
                      
                    }  sqldbcontext.SaveChanges();
                    // Update Subscription
                    foreach (var item in model.Assigned_StudentList)
                    {
                        sqldbcontext.ClassSubscriptions.Add(new ClassSubscription() { ClassId = model.ClassModel.ClassId, RequestedUserId = item.UserId, SubscriptionDate  = System.DateTime.Now });
                        
                    }sqldbcontext.SaveChanges();
                    tScope.Complete();
                    return model.ClassModel.ClassId;

                    #endregion
                }
            }
        }

        public bool CheckValidUser(string email, int currentUserId, int publisherUserId)
        {
            var user = sqldbcontext.Users.Where(x => x.Email == email && x.UserId != currentUserId && x.UserId != publisherUserId).FirstOrDefault();

            if (user != null)
                return true;
            else
                return false;
        }

        public AccessModel GetAccessDetails(string requestCode, string accessPwd, int _accessAttemptLimit)
        {
            var accessDetails = sqldbcontext.Classes.Where(x => x.AccessCode == requestCode && x.AccessCodePwd == accessPwd && x.AssignedUserAttempts < _accessAttemptLimit).FirstOrDefault();
            AccessModel accessModel = new AccessModel();

            if (accessDetails != null)
            {
                accessModel.AccessCode = requestCode;
                accessModel.CurrentUserId = accessDetails.AssignedUserId ?? 0;
                accessModel.PublisherUserId = accessDetails.PublisherUserId ?? 0;
                accessModel.ClassId = accessDetails.ClassId;

                var publisherDetail = sqldbcontext.Users.Where(x => x.UserId == accessModel.PublisherUserId).FirstOrDefault();

                if (publisherDetail != null)
                {
                    accessModel.PublisherCompanyLogo = publisherDetail.CompanyLogo;
                    accessModel.PublisherUser_FirstName = publisherDetail.FirstName;
                    accessModel.PublisherUser_LastName = publisherDetail.LastName;
                }

                var userDetail = sqldbcontext.Users.Where(x => x.UserId == accessModel.CurrentUserId).FirstOrDefault();

                if (userDetail != null)
                {
                    accessModel.DisplayName = userDetail.Email;
                    accessModel.CurrentUser_FirstName = userDetail.FirstName;
                    accessModel.CurrentUser_LastName = userDetail.LastName;
                }

                Classes classDetail = sqldbcontext.Classes.Where(x => x.AssignedUserId == accessModel.CurrentUserId && x.ClassId == accessDetails.ClassId).FirstOrDefault();

                if (classDetail != null)
                {
                    classDetail.AssignedUserAttempts = classDetail.AssignedUserAttempts + 1;
                    sqldbcontext.Entry(classDetail).State = EntityState.Modified;
                    sqldbcontext.SaveChanges();
                }
            }

            return accessModel;
        }

        #endregion

        #region All Class Page Functions.

        public List<ClassListModel> GetAllClassList(int userId, string searchquery, PageRightsModel pageRightsModel)
        {
            List<Classes> listClasses = new List<Classes>();

            if (pageRightsModel._view_all_private_class && pageRightsModel._view_all_public_class)
            {
                listClasses = (from list in sqldbcontext.Classes.Where(x => x.PublisherUserId == userId
                    &&
                    (
                    (searchquery.Length > 0) ? (x.ClassName.Contains(searchquery) || x.ClassDescription.Contains(searchquery)) : true)
                    )
                               select list).ToList();
            }
            else if (pageRightsModel._view_all_private_class)
            {
                listClasses = (from list in sqldbcontext.Classes.Where(x => x.PublisherUserId == userId && x.ClassType == false
                       &&
                    (
                    (searchquery.Length > 0) ? (x.ClassName.Contains(searchquery) || x.ClassDescription.Contains(searchquery)) : true)
                    )
                               select list).ToList();
            }

            else if (pageRightsModel._view_all_public_class)
            {
                listClasses = (from list in sqldbcontext.Classes.Where(x => x.PublisherUserId == userId && x.ClassType == true

                                   &&
                    (
                    (searchquery.Length > 0) ? (x.ClassName.Contains(searchquery) || x.ClassDescription.Contains(searchquery)) : true)
                    )

                               select list).ToList();
            }
            var item = (from classes in listClasses
                        join author in sqldbcontext.Users on classes.PublisherUserId equals author.UserId into group2
                        from g2 in group2.DefaultIfEmpty()

                        select new ClassListModel
                    {
                        ClassId = classes.ClassId
                            ,
                        ClassName = classes.ClassName
                            ,
                        ClassDescription = classes.ClassDescription
                            ,
                        AuthorName = g2.FirstName + " " + g2.LastName
                            ,
                        ClassType = classes.ClassType
                            ,
                        PublishedDate = classes.PublishedDate
                            ,
                        NoOfStudents = sqldbcontext.ClassSubscriptions.Where(s => s.ClassId == classes.ClassId && s.RequestedDate == null).Count()
                            ,
                        ClassStatus = classes.ClassStatus
                    }).ToList();


            return item;


        }

        public bool DeleteClass(int classId)
        {
            var subscriptions = sqldbcontext.ClassSubscriptions.Where(x => x.ClassId == classId).ToList();

            foreach (var item in subscriptions)
            {
                sqldbcontext.ClassSubscriptions.Remove(item);
               
            } sqldbcontext.SaveChanges();

            var classContents = sqldbcontext.ClassContents.Where(x => x.ClassId == classId).ToList();

            foreach (var item in classContents)
            {
                sqldbcontext.ClassContents.Remove(item);
               
            } sqldbcontext.SaveChanges();

            var classDetails = sqldbcontext.Classes.Where(x => x.ClassId == classId).FirstOrDefault();

            if (classDetails != null)
            {
                sqldbcontext.Classes.Remove(classDetails);
                sqldbcontext.SaveChanges();

                return true;
            }

            return false;
        }

        #endregion


        #region DashBoard

        public List<DashboardPostContent> DashboardGetAssignedClasses(int AssignedUserId)
        {
            var MyRoles = from Clss in sqldbcontext.Classes
                          where (Clss.AssignedUserId == AssignedUserId && Clss.AssignedUserAttempts < 3)
                          join Usrs in sqldbcontext.Users on Clss.PublisherUserId equals Usrs.UserId

                          select new DashboardPostContent
                          {
                              UserForm = Usrs
                             ,
                              ClassForm = Clss

                          };

            return MyRoles.ToList();
        }

        #endregion

        #region Common

        public void Save()
        {
            try
            {
                sqldbcontext.SaveChanges();
            }
            catch (DbEntityValidationException e)
            {
                string varException = string.Empty;
                foreach (var eve in e.EntityValidationErrors)
                {
                    varException += string.Format("Entity of type \"{0}\" in state \"{1}\" has the following validation errors:",
                           eve.Entry.Entity.GetType().Name, eve.Entry.State);
                    foreach (var ve in eve.ValidationErrors)
                    {
                        varException += string.Format("- Property: \"{0}\", Error: \"{1}\"",
                            ve.PropertyName, ve.ErrorMessage);
                    }
                }

            }
        }

        private bool disposed = false;

        public void Dispose()
        {
            Dispose(true);
            GC.SuppressFinalize(this);
        }

        protected virtual void Dispose(bool disposing)
        {
            if (!this.disposed)
            {
                if (disposing)
                {
                    sqldbcontext.Dispose();
                }
            }
            this.disposed = true;
        }

        #endregion

    }
}