﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Data.Entity.Validation;
using System.Web.Mvc;
using System.Data;

using IntelSkill.Repository;
using IntelSkill.Models;


namespace IntelSkill.DAL
{
    public class QuestionRepository : IQuestionRepository, IDisposable
    {
        public SQLDBContext sqldbcontext;

        public QuestionRepository(SQLDBContext _sqldbcontext)
        {
            this.sqldbcontext = _sqldbcontext;
        }

        #region View Questions

        public List<QuestionListModel> GetMyQuestions(int userId)
        {
            return (from q in sqldbcontext.Questions
                    join quiz in sqldbcontext.Quizzes on q.QuizId equals quiz.QuizId
                    where quiz.PublisherUserId == userId
                    select new QuestionListModel { QuestionId = q.QuestionId, QuestionDescription = q.QuestionDescription, QuestionStatus = q.QuestionStatus ?? 0 }).ToList();
        }

        public List<QuestionListModel> GetMyQuestions(int userId, int quizId)
        {
            return (from q in sqldbcontext.Questions
                    join quiz in sqldbcontext.Quizzes on q.QuizId equals quiz.QuizId
                    where quiz.PublisherUserId == userId && quiz.QuizId == quizId
                    select new QuestionListModel { QuestionId = q.QuestionId, QuestionDescription = q.QuestionDescription, QuestionStatus = q.QuestionStatus ?? 0 }).ToList();
        }

        public SelectList GetMyQuizList(int userId)
        {

            var selectelistitem = (from quiz in sqldbcontext.Quizzes where quiz.PublisherUserId == userId select new { QuizTitle = quiz.QuizTitle, QuizId = quiz.QuizId, Selected = false }).ToList();

            SelectList items = new SelectList(selectelistitem, "QuizId", "QuizTitle");

            return items;
        }

        public Quizzes GetQuizDetail(int quizId)
        {
            return (from q in sqldbcontext.Quizzes where q.QuizId == quizId select q).FirstOrDefault();
        }

        public bool DeleteQuestion(int questionId)
        {
            var answers = sqldbcontext.QuestionAnswers.Where(x => x.QuestionId == questionId).ToList();

            foreach (var item in answers)
            {
                sqldbcontext.QuestionAnswers.Remove(item);
                sqldbcontext.SaveChanges();
            }

            var images = sqldbcontext.QuestionImages.Where(x => x.QuestionId == questionId).ToList();

            foreach (var item in images)
            {
                sqldbcontext.QuestionImages.Remove(item);
                sqldbcontext.SaveChanges();
            }

            var questionDetails = sqldbcontext.Questions.Where(x => x.QuestionId == questionId).FirstOrDefault();

            if (questionDetails != null)
            {
                sqldbcontext.Questions.Remove(questionDetails);
                sqldbcontext.SaveChanges();

                return true;
            }

            return false;
        }

        public List<QuestionTypes> GetQuestionTypes()
        {
            return sqldbcontext.QuestionTypes.ToList();
        }

        public bool SaveQuestion(ViewQuestionModel model, int userId)
        {
            var currentquestion = sqldbcontext.Questions.Where(x => x.QuestionId == model.QuestionModel.Question.QuestionId).FirstOrDefault();

            if (currentquestion == null)
            {
                model.QuestionModel.Question.CreatedDate = System.DateTime.Now;
                model.QuestionModel.Question.CreatedUserId = userId;
                model.QuestionModel.Question.QuestionStatus = 1;
                sqldbcontext.Questions.Add(model.QuestionModel.Question);
                sqldbcontext.SaveChanges();

            }
            else
            {
                var question = sqldbcontext.Questions.Where(x => x.QuestionId == model.QuestionModel.Question.QuestionId).FirstOrDefault();

                question.ModifiedDate = System.DateTime.Now;
                question.ModifiedUserId = userId;
                question.QuestionDescription = model.QuestionModel.Question.QuestionDescription;
                question.QuestTypeId = model.QuestionModel.Question.QuestTypeId;
                question.QuizId = model.QuestionModel.Question.QuizId;


                sqldbcontext.Entry(question).State = EntityState.Modified;

                sqldbcontext.SaveChanges();
            }

            if (model.QuestionModel.Question.QuestionId > 0)
            {
                model.QuestionModel.AnswerList.ForEach(x => x.QuestionId = model.QuestionModel.Question.QuestionId);

                var item = (from answer in sqldbcontext.QuestionAnswers where answer.QuestionId == model.QuestionModel.Question.QuestionId select answer);

                foreach (var rowitem in item)
                {
                    sqldbcontext.QuestionAnswers.Remove(rowitem);

                } sqldbcontext.SaveChanges();

                foreach (var rowitem in model.QuestionModel.AnswerList)
                {
                    sqldbcontext.QuestionAnswers.Add(rowitem);

                } sqldbcontext.SaveChanges();



                var dbImage = sqldbcontext.QuestionImages.Where(x => x.QuestionId == model.QuestionModel.Question.QuestionId).FirstOrDefault();

                var NewImage = model.QuestionModel.ImageList.Where(x => x.ImageId == 0).FirstOrDefault();

                var OldImage = model.QuestionModel.ImageList.Where(x => x.ImageId != 0).FirstOrDefault();

                if (OldImage == null)
                {
                    if (dbImage != null)
                    {
                        sqldbcontext.QuestionImages.Remove(dbImage);
                        sqldbcontext.SaveChanges();
                    }
                }

                if (NewImage != null)
                {


                    NewImage.QuestionId = model.QuestionModel.Question.QuestionId;
                    sqldbcontext.QuestionImages.Add(NewImage);
                    sqldbcontext.SaveChanges();

                }
                return true;

            }



            return false;
        }

        public QuestionModel GetQuestionDetail(int questionId)
        {

            var model = new QuestionModel();

            if (questionId > 0)
                model.Question = sqldbcontext.Questions.Where(x => x.QuestionId == questionId).FirstOrDefault();
            else
                model.Question = new Questions();

            if (questionId > 0)
            {

                if (model.Question.QuestTypeId == 4)
                {
                    model.MatchAList = sqldbcontext.QuestionAnswers.Where(x => x.QuestionId == questionId && x.AnswerGroup == "GroupA").ToList();

                    model.MatchBList = sqldbcontext.QuestionAnswers.Where(x => x.QuestionId == questionId && x.AnswerGroup == "GroupB").ToList();

                    model.AnswerList = new List<QuestionAnswers>();
                    model.AnswerList.AddRange(model.MatchBList.OrderBy(x => x.OrderBy));
                }
                else
                {
                    model.MatchAList = new List<QuestionAnswers>();
                    model.MatchBList = new List<QuestionAnswers>();
                    model.AnswerList = sqldbcontext.QuestionAnswers.Where(x => x.QuestionId == questionId).OrderBy(x => x.OrderBy).ToList();
                }


            }
            else
                model.AnswerList = new List<QuestionAnswers>();

            if (questionId > 0)
                model.ImageList = sqldbcontext.QuestionImages.Where(x => x.QuestionId == questionId).ToList();

            return model;
        }

        public void UpdateStatus(int questionId, int status)
        {
            var question = sqldbcontext.Questions.Where(x => x.QuestionId == questionId).FirstOrDefault();


            question.QuestionStatus = status;

            sqldbcontext.Entry(question).State = EntityState.Modified;
            sqldbcontext.SaveChanges();
        
        }

        #endregion

        #region Common

        public void Save()
        {
            try
            {
                sqldbcontext.SaveChanges();
            }
            catch (DbEntityValidationException e)
            {
                string varException = string.Empty;
                foreach (var eve in e.EntityValidationErrors)
                {
                    varException += string.Format("Entity of type \"{0}\" in state \"{1}\" has the following validation errors:",
                           eve.Entry.Entity.GetType().Name, eve.Entry.State);
                    foreach (var ve in eve.ValidationErrors)
                    {
                        varException += string.Format("- Property: \"{0}\", Error: \"{1}\"",
                            ve.PropertyName, ve.ErrorMessage);
                    }
                }

            }
        }

        private bool disposed = false;

        public void Dispose()
        {
            Dispose(true);
            GC.SuppressFinalize(this);
        }

        protected virtual void Dispose(bool disposing)
        {
            if (!this.disposed)
            {
                if (disposing)
                {
                    sqldbcontext.Dispose();
                }
            }
            this.disposed = true;
        }

        #endregion
    }
}