﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using IntelSkill.Models;
using IntelSkill.Repository;
using System.Data;

namespace IntelSkill.DAL
{
    public class QuizRepository : IQuizRepository
    {
        public SQLDBContext sqldbcontext;

        public QuizRepository(SQLDBContext sqldbcontext)
        {
            this.sqldbcontext = sqldbcontext;
        }

        public Quizzes GetQuiz(string QuizName, int UserId)
        {
            Quizzes Quizdetails = (from c in sqldbcontext.Quizzes where (c.QuizTitle == QuizName && c.PublisherUserId == UserId) select c).FirstOrDefault();
            return Quizdetails;
        }

        public Quizzes GetQuizbyId(int QuizId)
        {
            Quizzes Quizdetails = (from c in sqldbcontext.Quizzes where (c.QuizId == QuizId) select c).FirstOrDefault();
            return Quizdetails;
        }

        public void InsertQuiz(Quizzes Quiz)
        {
            sqldbcontext.Quizzes.Add(Quiz);
        }

        public void UpdateQuiz(Quizzes Quiz)
        {
            sqldbcontext.Entry(Quiz).State = EntityState.Modified;
        }

        public bool DeleteQuiz(int QuizId)
        {

            List<int> QuestionId = (from cp in sqldbcontext.Questions where (cp.QuizId == QuizId) select cp.QuestionId).ToList<int>();
            if (QuestionId != null)
            {
                foreach (int Id in QuestionId)
                {
                    Questions Quest = sqldbcontext.Questions.Find(Id);
                    sqldbcontext.Questions.Remove(Quest);
                }

            }
            Quizzes quiz = sqldbcontext.Quizzes.Find(QuizId);
            if (quiz != null)
            {
                sqldbcontext.Quizzes.Remove(quiz);
                return true;
            }
            else
            {
                return false;
            }
        }

        public List<Questions> GetQuestions(int QuizId)
        {
            List<Questions> Questions = (from c in sqldbcontext.Questions where (c.QuizId == QuizId) select c).ToList();
            return Questions;
        }

        public void Save()
        {
            sqldbcontext.SaveChanges();
        }

        public IEnumerable<QuizListModel> GetAllQuiz(int publisherId)
        {

            var query = from c in sqldbcontext.Quizzes

                         where c.PublisherUserId == publisherId
                        select new QuizListModel
                        {
                            QuizId = c.QuizId,
                            QuizTitle = c.QuizTitle,
                            ModifiedDate = c.ModifiedDate,
                            Status = c.QuizStatus,
                            TotalQuestions=(from questions in sqldbcontext.Questions where (questions.QuizId == c.QuizId) select c).Count(),
                            QuestionAmount = c.QuestionAmount,
                            TimeAllowed = c.TimeAllowed,
                            AttemptLimit = c.AttemptLimit,
                            PassingScore = c.PassingScore,
                            DateAdded=c.CreatedDate,
                            PublishedDate=c.PublishedDate,
                            QuizAttempts = (from Attempts in sqldbcontext.QuizReports where (Attempts.QuizId == c.QuizId) select new { Attempts.QuizId, Attempts.CandidateUserId }).Distinct().Count()
                        };
          
            return query.ToList();
        }
        public IEnumerable<QuizListModel> GetQuizList(int publisherId)
        {

            var query = from c in sqldbcontext.Quizzes                      
                        where c.PublisherUserId == publisherId  && c.QuizStatus == 1 orderby c.QuizTitle ascending  
                        select new QuizListModel
                        {
                            QuizId = c.QuizId,
                            QuizTitle = c.QuizTitle,
                            ModifiedDate = c.ModifiedDate,
                            Status = c.QuizStatus,
                            TotalQuestions = (from questions in sqldbcontext.Questions where (questions.QuizId == c.QuizId) select c).Count(),
                            QuestionAmount = c.QuestionAmount,
                            TimeAllowed = c.TimeAllowed,
                            AttemptLimit = c.AttemptLimit,
                            PassingScore = c.PassingScore,
                            DateAdded = c.CreatedDate,
                            PublishedDate = c.PublishedDate,
                            QuizAttempts = (from Attempts in sqldbcontext.QuizReports where (Attempts.QuizId == c.QuizId) select new { Attempts.QuizId, Attempts.CandidateUserId }).Distinct().Count()
                        };

            return query.ToList();
        }

        public List<Quizzes> GetPublishedQuizzes(int publisherId)
        {
            List<Quizzes> Quizzes = (from c in sqldbcontext.Quizzes where (c.PublisherUserId == publisherId &&c.QuizStatus==1) select c).ToList();
            return Quizzes;
        }

        

        #region PublisherDashboard


        public List<DashboardListOverview> DashboardOverview(int PublisherId, string FromDate, string ToDate)
        {
            var TotalQuizzes = new List<Quizzes>();
            var TotalClasses = new List<Classes>();
            var TotalContent = new List<Contents>();
            var TotalStudents = new List<RefUsers>();
            DateTime FromDt = Convert.ToDateTime(FromDate);
            DateTime ToDt = Convert.ToDateTime(ToDate);

            if (string.IsNullOrEmpty(FromDate) && string.IsNullOrEmpty(ToDate))
            {
                TotalQuizzes = (from quizzes in sqldbcontext.Quizzes where (quizzes.PublisherUserId == PublisherId) select quizzes).ToList();
                TotalClasses = (from classes in sqldbcontext.Classes where (classes.PublisherUserId == PublisherId) select classes).ToList();
                TotalContent = (from content in sqldbcontext.Contents where (content.PublisherUserId == PublisherId) select content).ToList();
                TotalStudents = (from stds in sqldbcontext.RefUsers where (stds.ParentUserId == PublisherId && stds.RoleId == 3) select stds).ToList();
            }
            else if (!string.IsNullOrEmpty(FromDate) && !string.IsNullOrEmpty(ToDate))
            {
                TotalQuizzes = (from quizzes in sqldbcontext.Quizzes
                                where (quizzes.PublisherUserId == PublisherId && ((quizzes.CreatedDate.Value > FromDt) || ((quizzes.CreatedDate.Value.Year == FromDt.Year) && (quizzes.CreatedDate.Value.Month == FromDt.Month) && (quizzes.CreatedDate.Value.Day == FromDt.Day)))
                                && ((quizzes.CreatedDate.Value < ToDt) || ((quizzes.CreatedDate.Value.Year == ToDt.Year) && (quizzes.CreatedDate.Value.Month == ToDt.Month) && (quizzes.CreatedDate.Value.Day == ToDt.Day))))
                                select quizzes).ToList();
                TotalClasses = (from classes in sqldbcontext.Classes
                                where (classes.PublisherUserId == PublisherId && ((classes.CreatedDate.Value > FromDt) || ((classes.CreatedDate.Value.Year == FromDt.Year) && (classes.CreatedDate.Value.Month == FromDt.Month) && (classes.CreatedDate.Value.Day == FromDt.Day)))
                                && ((classes.CreatedDate.Value < ToDt) || ((classes.CreatedDate.Value.Year == ToDt.Year) && (classes.CreatedDate.Value.Month == ToDt.Month) && (classes.CreatedDate.Value.Day == ToDt.Day))))
                                select classes).ToList();
                TotalContent = (from content in sqldbcontext.Contents
                                where (content.PublisherUserId == PublisherId && ((content.CreatedDate.Value > FromDt) || ((content.CreatedDate.Value.Year == FromDt.Year) && (content.CreatedDate.Value.Month == FromDt.Month) && (content.CreatedDate.Value.Day == FromDt.Day)))
                                 && ((content.CreatedDate.Value < ToDt) || ((content.CreatedDate.Value.Year == ToDt.Year) && (content.CreatedDate.Value.Month == ToDt.Month) && (content.CreatedDate.Value.Day == ToDt.Day))))
                                select content).ToList();
                TotalStudents = (from stds in sqldbcontext.RefUsers
                                 where (stds.ParentUserId == PublisherId && stds.RoleId == 3 && ((stds.CreatedDate.Value > FromDt) || ((stds.CreatedDate.Value.Year == FromDt.Year) && (stds.CreatedDate.Value.Month == FromDt.Month) && (stds.CreatedDate.Value.Day == FromDt.Day)))
                                     && ((stds.CreatedDate.Value < ToDt) || ((stds.CreatedDate.Value.Year == ToDt.Year) && (stds.CreatedDate.Value.Month == ToDt.Month) && (stds.CreatedDate.Value.Day == ToDt.Day))))
                                 select stds).ToList();
            }
            else if (string.IsNullOrEmpty(FromDate) && !string.IsNullOrEmpty(ToDate))
            {
                TotalQuizzes = (from quizzes in sqldbcontext.Quizzes
                                where (quizzes.PublisherUserId == PublisherId &&
                                ((quizzes.CreatedDate.Value.Year == ToDt.Year) && (quizzes.CreatedDate.Value.Month == ToDt.Month) && (quizzes.CreatedDate.Value.Day == ToDt.Day)))
                                select quizzes).ToList();
                TotalClasses = (from classes in sqldbcontext.Classes
                                where (classes.PublisherUserId == PublisherId &&
                                ((classes.CreatedDate.Value.Year == ToDt.Year) && (classes.CreatedDate.Value.Month == ToDt.Month) && (classes.CreatedDate.Value.Day == ToDt.Day)))
                                select classes).ToList();
                TotalContent = (from content in sqldbcontext.Contents
                                where (content.PublisherUserId == PublisherId &&
                               ((content.CreatedDate.Value.Year == ToDt.Year) && (content.CreatedDate.Value.Month == ToDt.Month) && (content.CreatedDate.Value.Day == ToDt.Day)))
                                select content).ToList();
                TotalStudents = (from stds in sqldbcontext.RefUsers
                                 where (stds.ParentUserId == PublisherId && stds.RoleId == 3 && (((stds.CreatedDate.Value.Year == ToDt.Year) && (stds.CreatedDate.Value.Month == ToDt.Month) && (stds.CreatedDate.Value.Day == ToDt.Day))))
                                 select stds).ToList();
            }

           

            var questiontype = (from quiz in TotalQuizzes
                                join questions in sqldbcontext.Questions on quiz.QuizId equals questions.QuizId

                                where quiz.PublisherUserId == PublisherId
                                select new { questions });



            // Passing Score Average 

            var DashboardPassOverview = (from Q in TotalQuizzes
                                         join QR in sqldbcontext.QuizReports on Q.QuizId equals QR.QuizId
                                         where Q.PublisherUserId == PublisherId
                                         group QR by new { QR.CandidateUserId, QR.QuizId } into d
                                         select new { UserId = d.Key.CandidateUserId, QuizId = d.Key.QuizId, PassedScore = (from t2 in d select t2.PassedScore).Max() });
            List<QuizReports> listQR = new List<QuizReports>();

            decimal d1;
            decimal d2;
            decimal d3;

            decimal PassReportd = 0;
            decimal FailReportd = 0;
            decimal InCompleteReportd = 0;

            decimal dpassreport = 0;
            decimal dfailreport = 0;
            decimal dincompletereport = 0;

            if (DashboardPassOverview.Count() > 0)
            {
                foreach (var query in DashboardPassOverview)
                {
                    var QuizReport = (from q in sqldbcontext.QuizReports where (q.PassedScore == query.PassedScore && q.CandidateUserId == query.UserId && q.QuizId == query.QuizId) select q).FirstOrDefault();
                    listQR.Add(QuizReport);
                }
                if (listQR.Count > 0)
                {
                    d2 = listQR.Count;

                    decimal percent = 100;

                    d1 = (from PassRpt in listQR where (PassRpt.QuizResult == "Pass") select PassRpt).Count() * 100;
                    d3 = Decimal.Divide(d1, d2);
                    dpassreport = d3;
                    PassReportd = Math.Round(d3);

                    percent = percent - PassReportd;

                    d1 = (from FailRpt in listQR where (FailRpt.QuizResult == "Fail") select FailRpt).Count() * 100;
                    d3 = Decimal.Divide(d1, d2);
                    dfailreport = d3;
                    FailReportd = Math.Round(d3);
                    percent = percent - FailReportd;

                    //d1 = (from InCompleteRpt in listQR where (InCompleteRpt.QuizEndTime == null) select InCompleteRpt).Count() * 100;
                    //d3 = Decimal.Divide(d1, d2);
                    //dincompletereport=d3;
                    InCompleteReportd = percent;

                }



            }





            return new List<DashboardListOverview>()
            {
                       new DashboardListOverview 
                       {
                            TotalQuizzes = TotalQuizzes.Count(),
                            TotalClasses = TotalClasses,
                            TotalContent = TotalContent.Count(),
                            TotalStudents=TotalStudents.Count(),
                           TotalMultipleChoice=(from MultipleQuest in questiontype where MultipleQuest.questions.QuestTypeId==1 select MultipleQuest).Count(),
                           TotalMatching=(from MatchingQuest in questiontype where MatchingQuest.questions.QuestTypeId==4 select MatchingQuest).Count(),
                           TotalFillintheBlank=(from FillinBalnkQuest in questiontype where FillinBalnkQuest.questions.QuestTypeId==2 select FillinBalnkQuest).Count(),
                           TotalTrueFalse=(from TrueFalseQuest in questiontype where TrueFalseQuest.questions.QuestTypeId==3 select TrueFalseQuest).Count(),
                           PassReport=Convert.ToInt32(PassReportd),
                            FailReport=Convert.ToInt32(FailReportd),
                           InCompleteReport=Convert.ToInt32(InCompleteReportd),
            
                               dPassReport=dpassreport,
                            dFailReport=dfailreport,
                           dInCompleteReport=dincompletereport
                           
                       }
            };


        }

        public List<DashboardSubsrciptionApproval> DashboardApprval(int PublisherId)
        {
            var ClsSubList = from clsses in sqldbcontext.Classes
                             join clsSub in sqldbcontext.ClassSubscriptions on clsses.ClassId equals clsSub.ClassId
                             where clsses.PublisherUserId == PublisherId && clsSub.SubscriptionStatus == 2 && clsses.ClassStatus == 1

                             select new DashboardSubsrciptionApproval
                             {
                                 RequestedName = (from ReqName in sqldbcontext.Users where ReqName.UserId == clsSub.RequestedUserId select ReqName.FirstName).FirstOrDefault(),
                                 ClassSubId = clsSub.SubscriptionId,
                                 ClassName=clsses.ClassName
                             };

            return ClsSubList.ToList();
        }

       
        public ClassSubscription GetClsSubbyId(int ClsSubId)
        {
            var ClsSub = (from Cls in sqldbcontext.ClassSubscriptions where Cls.SubscriptionId == ClsSubId select Cls).FirstOrDefault();
            return ClsSub;
        }

        public void UpdateClsSub(ClassSubscription ClsSub)
        {
            sqldbcontext.Entry(ClsSub).State = EntityState.Modified;
        }

        public void DeleteClassSubId(int ClsSubId)
        {

            ClassSubscription ClsSub = sqldbcontext.ClassSubscriptions.Find(ClsSubId);
            sqldbcontext.ClassSubscriptions.Remove(ClsSub);

        }
        #endregion


        private bool disposed = false;

        protected virtual void Dispose(bool disposing)
        {
            if (!this.disposed)
            {
                if (disposing)
                {
                    sqldbcontext.Dispose();
                }
            }
            this.disposed = true;
        }

        public void Dispose()
        {
            Dispose(true);
            GC.SuppressFinalize(this);
        }
    }
}
