﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using IntelSkill.Models;
using IntelSkill.Repository;
using System.Data;

namespace IntelSkill.DAL
{
    public class ReaderRepository : IReaderRepository
    {
        public SQLDBContext sqldbcontext;
        public ReaderRepository(SQLDBContext sqldbcontext)
        {
            this.sqldbcontext = sqldbcontext;
        }

        public IEnumerable<ClassSubscription> GetSubscribers()
        {
            return sqldbcontext.ClassSubscriptions.ToList();

        }
        public void InsertSubscriber(ClassSubscription Subscribe)
        {
            sqldbcontext.ClassSubscriptions.Add(Subscribe);
        }
        public void UpdatSuebscriber(ClassSubscription Subscribe)
        {
            sqldbcontext.Entry(Subscribe).State = EntityState.Modified;
        }
        public void Save()
        {
            sqldbcontext.SaveChanges();
        }
        public List<Users> GetUsersOfReaders()
        {
            List<Users> usrs = (from u in sqldbcontext.Users where (u.UserTypeId == 2) select u).ToList();
            return usrs;
        }
        public List<Users> GetAllPublisher()
        {
            List<int> clas = (from c in sqldbcontext.Classes where (c.PublisherUserId != null && c.ClassStatus == 1) select c.PublisherUserId.Value).Distinct().ToList();
            List<Users> userslist = new List<Users>();
            foreach (int PublisherUserId in clas)
            {
                var Publishers = (from u in sqldbcontext.Users where (u.UserId == PublisherUserId) select u);
                userslist.AddRange(Publishers);
            }
            return userslist;

        }

        public List<Users> GetAllPublisherForReader(int currentUserid)
        {
            List<int> clas = (from c in sqldbcontext.Classes where (c.PublisherUserId != null && c.ClassStatus == 1 && c.PublisherUserId != currentUserid) select c.PublisherUserId.Value).Distinct().ToList();
            List<Users> userslist = new List<Users>();
            foreach (int PublisherUserId in clas)
            {
                var Publishers = (from u in sqldbcontext.Users where (u.UserId == PublisherUserId) select u);
                userslist.AddRange(Publishers);
            }
            return userslist;

        }

        public IEnumerable<ReaderContentModel> GetAllClass(int PublisherUserid)
        {

            var query = from c in sqldbcontext.Classes

                        //join m in context.ClassSubscription on c.ClassId equals m.ClassId
                        //into members
                        where c.PublisherUserId == PublisherUserid && c.ClassStatus == 1
                        select new ReaderContentModel
                        {
                            ClassId = c.ClassId,
                            ClassName = c.ClassName,
                            ClassDesc = c.ClassDescription,
                            ClassType = (bool)c.ClassType,
                            PublishedDate = c.PublishedDate,
                            PublisherUserId = (int)c.PublisherUserId,
                            Count = (from s in sqldbcontext.ClassSubscriptions where (s.ClassId == c.ClassId) select s).Count(),
                        };

            return query.ToList();
        }
        public int GetClassSubcriptionCount(int classid)
        {
            return (from s in sqldbcontext.ClassSubscriptions where (s.ClassId == classid) select s).Count();
        }

        public string GetPublisherName(int PublisherUserid)
        {


            string name = (from u in sqldbcontext.Users where (u.UserId == PublisherUserid) select u.FirstName + " " + u.LastName).FirstOrDefault<string>();

            return name;

        }

        public Users GetUserDetails(int id)
        {
            Users Users = (from u in sqldbcontext.Users where (u.UserId == id) select u).FirstOrDefault();
            return Users;
        }

        public IEnumerable<ClassSubscription> GetClassSubscription(int ReaderUserId)
        {


            IEnumerable<ClassSubscription> classid = (from cs in sqldbcontext.ClassSubscriptions where (cs.RequestedUserId == ReaderUserId) select cs).ToList();
            //IEnumerable<int> clsDist = classid.Distinct();
            return classid;

        }

        public int SaveClassChanges(ClassSubscription classObj)
        {
            if ((from item in sqldbcontext.ClassSubscriptions where item.RequestedUserId == classObj.RequestedUserId && item.ClassId == classObj.ClassId select item).Count() == 0)
            {
                sqldbcontext.ClassSubscriptions.Add(classObj);
                sqldbcontext.SaveChanges();
            }
            return classObj.SubscriptionId;
        }

        private bool disposed = false;

        protected virtual void Dispose(bool disposing)
        {
            if (!this.disposed)
            {
                if (disposing)
                {
                    sqldbcontext.Dispose();
                }
            }
            this.disposed = true;
        }

        public void Dispose()
        {
            Dispose(true);
            GC.SuppressFinalize(this);
        }

        public IEnumerable<Users> MyAccount(int id)
        {
            return from c in sqldbcontext.Users where (c.UserId == id) select c;
        }

        public string GetPassword(int userid)
        {
            string Password = (from c in sqldbcontext.Users where (c.UserId == userid) select c.Password).FirstOrDefault();
            return Password;

        }
        public Users ChangePassword(string Password)
        {

            return (from e in sqldbcontext.Users where (e.Password == Password) select e).FirstOrDefault();
        }
        public IEnumerable<Users> userinformation(int userid)
        {
            return from u in sqldbcontext.Users where (u.UserId == userid) select u;
        }

        public Users UserAccount(int userid)
        {
            Users Users = (from u in sqldbcontext.Users where (u.UserId == userid) select u).FirstOrDefault();
            return Users;
        }
        //public int RoleId(int userid)
        //{
        //    return (from u in sqldbcontext.Users where (u.UserId == userid) select u.RoleId).FirstOrDefault();
        //}



        public List<QuizReports> GetNumberOfQuizes(int ReaderUserId)
        {
            List<int> Qzid = new List<int>();
            QuizReports Quizname = new QuizReports();
            List<QuizReports> quiznames = new List<QuizReports>();
            Qzid = (from u in sqldbcontext.QuizReports where (u.CandidateUserId == ReaderUserId) select u.QuizId.Value).Distinct().ToList();

            foreach (var cls in Qzid)
            {
                Quizname = (from se in sqldbcontext.QuizReports where (se.QuizId == cls) select se).ToList().LastOrDefault();

                quiznames.Add(Quizname);
            }
            return quiznames;
        }

        public Quizzes GetQuizzesbyId(int quizid)
        {
            Quizzes quiz = (from q in sqldbcontext.Quizzes where (q.QuizId == quizid) select q).FirstOrDefault();
            return quiz;
        }

        public QuizReports GetQizDetails(int ReaderUserId, int quizid)
        {
            QuizReports quiz = (from q in sqldbcontext.QuizReports where (q.QuizId == quizid && q.CandidateUserId == ReaderUserId) select q).ToList().LastOrDefault();
            return quiz;

        }

        public IEnumerable<QuestionTypes> GetQuestions()
        {
            return sqldbcontext.QuestionTypes.ToList();
        }

        public List<Questions> GetAllQuestions(int quizid)
        {
            List<Questions> Questions = (from Q in sqldbcontext.Questions where (Q.QuizId == quizid) select Q).ToList();
            return Questions;
        }
        public IEnumerable<ReaderContentModel> GetAllClassSearched(string searchtext, int PublisherUserid)
        {

            var query = from c in sqldbcontext.Classes

                        //join m in context.ClassSubscription on c.ClassId equals m.ClassId
                        //into members
                        where c.ClassName.Contains(searchtext)
                        select new ReaderContentModel
                        {
                            ClassId = c.ClassId,
                            ClassName = c.ClassName,
                            ClassDesc = c.ClassDescription,
                            ClassType = (bool)c.ClassType,
                            PublishedDate = c.PublishedDate,
                            PublisherUserId = (int)c.PublishedUserId,
                            Count = (from s in sqldbcontext.ClassSubscriptions where (s.ClassId == c.ClassId) select s).Count(),
                        };

            return query.ToList();
        }

        public IEnumerable<ReaderContentModel> GetAllPublisherClassSearched(string searchtext, int PublisherUserid)
        {

            var query = from c in sqldbcontext.Classes
                        join u in sqldbcontext.Users on c.PublisherUserId equals u.UserId

                        //join m in context.ClassSubscription on c.ClassId equals m.ClassId
                        //into members
                        where u.CompanyName.Contains(searchtext) && c.ClassStatus == 1
                        select new ReaderContentModel
                        {
                            ClassId = c.ClassId,
                            ClassName = c.ClassName,
                            ClassDesc = c.ClassDescription,
                            ClassType = (bool)c.ClassType,
                            PublishedDate = c.PublishedDate,
                            PublisherUserId = (int)c.PublisherUserId,
                            Count = (from s in sqldbcontext.ClassSubscriptions where (s.ClassId == c.ClassId) select s).Count(),
                        };

            return query.ToList();
        }


        public void GetAllClass(int readerUserId, ref List<Classes> listClasses, ref List<Users> listUsers, ref List<ReaderContentModel> listContents, ref string DefaultPublisherName, ref string DefaultClassName, int ClassId)
        {
            // Class, User, Content Object Initializing...
            listClasses = new List<Classes>();
            listUsers = new List<Users>();
            listContents = new List<ReaderContentModel>();

            //Get Subscribed Classes of logged reader(User).
            var classes = (from classitem in sqldbcontext.Classes
                           join classSubscription in sqldbcontext.ClassSubscriptions
                           on classitem.ClassId equals classSubscription.ClassId
                           where classSubscription.RequestedUserId == readerUserId && (classSubscription.RequestedDate != null ? classSubscription.SubscriptionStatus == 1 : true)
                           select classitem).OrderBy(x => x.ClassName).ToList();
            if (classes.Count() > 0)
            {
                listClasses = classes;

                //Grouping Publisher from Subscribed Classes List.
                var groupedPublisherClass = listClasses
                                              .GroupBy(i => i.PublisherUserId)
                                           .Select(g => new
                                           {
                                               ID = g.Key,
                                               PublisherId = g.Max(t => t.PublisherUserId)
                                           });
                //Get Publisher Details by Grouped List.
                foreach (var listitem in groupedPublisherClass)
                {

                    var publisherItem = (from classitem in sqldbcontext.Users where classitem.UserId == listitem.PublisherId select classitem).Single();
                    listUsers.Add(publisherItem);

                }
                //Order by FirstName Ascending.

                listUsers = listUsers.OrderBy(x => x.FirstName).ToList();

                //Page Default Data for Publisher & Content Listing.
                Users DefaultUser = listUsers.Take(1).Single();
                DefaultPublisherName = DefaultUser.FirstName + "" + DefaultUser.LastName;


                var classdefault = (from class_default in listClasses where class_default.PublisherUserId == DefaultUser.UserId select class_default).ToList();

                Classes DefaultClass = new Classes();


                if (ClassId == 0)
                {
                    DefaultClass = classdefault.Take(1).Single();
                }
                else
                {
                    //Selected Class Details
                    DefaultClass = (from ItembyId in listClasses where ItembyId.ClassId == ClassId select ItembyId).FirstOrDefault();
                    if (DefaultClass != null)
                    {
                        DefaultUser = (from userItem in listUsers where userItem.UserId == DefaultClass.PublisherUserId select userItem).FirstOrDefault();
                        DefaultPublisherName = DefaultUser.FirstName + "" + DefaultUser.LastName;
                    }
                }

                if (DefaultClass != null)
                {
                    DefaultClassName = DefaultClass.ClassName;
                    var contents = (from content in sqldbcontext.Contents
                                    where content.ContentStatus == 1
                                    join classcontent in sqldbcontext.ClassContents
                                    on content.ContentId equals classcontent.ContentId
                                    join user in sqldbcontext.Users
                                    on content.PublisherUserId equals user.UserId
                                    where classcontent.ClassId == DefaultClass.ClassId
                                    select new ReaderContentModel { ContentId = content.ContentId, ContentTitle = content.ContentTitle, Author = user.FirstName + " " + user.LastName, CreatedDate = content.CreatedDate, ClassId = DefaultClass.ClassId }).ToList();

                    listContents = contents;
                }

            }
        }

        public IEnumerable<ContentPages> GetContentPagesDescr(int ContentId)
        {
            var Desc = (from c in sqldbcontext.ContentPages where (c.ContentId == ContentId) select c).ToList();

            return Desc;
        }

        public int? GetPublisherId(int ContId)
        {
            int? PubId = (from c in sqldbcontext.Contents where (c.ContentId == ContId) select c.PublisherUserId).FirstOrDefault();
            return PubId;
        }

        public List<Categories> Getallcategories()
        {
            List<Categories> cat = (from c in sqldbcontext.Categories select c).ToList();
            return cat;
        }

        //Search Functionality in ReaderDashboard

        public IEnumerable<ReaderContentModel> GetContents(string searchtext, int loggedUserId)
        {


            var SearchText = from ClsSub in sqldbcontext.ClassSubscriptions
                             join classes in sqldbcontext.Classes on ClsSub.ClassId equals classes.ClassId
                             join classcont in sqldbcontext.ClassContents on classes.ClassId equals classcont.ClassId
                             join content in sqldbcontext.Contents on classcont.ContentId equals content.ContentId
                             join user in sqldbcontext.Users on content.PublisherUserId equals user.UserId


                             where ClsSub.RequestedUserId == loggedUserId && (classes.ClassName.Contains(searchtext) || content.ContentTitle.Contains(searchtext) || user.FirstName.Contains(searchtext) || user.LastName.Contains(searchtext))
                             select new ReaderContentModel
                             {
                                 ContentId = content.ContentId,
                                 ContentTitle = content.ContentTitle,
                                 Author = user.FirstName,
                                 CreatedDate = content.PublishedDate,
                             };
            var groupedCustomerList = SearchText
              .GroupBy(u => u.ContentId)
              .Select(grp => grp.OrderBy(u => u.ContentId).FirstOrDefault()).ToList();

            return groupedCustomerList.ToList();

        }

        public IEnumerable<ReaderContentModel> GetClasses(string searchtext)
        {
            var query = from c in sqldbcontext.Classes
                        join user in sqldbcontext.Users on c.PublisherUserId equals user.UserId

                        where ((user.FirstName.Contains(searchtext) || user.LastName.Contains(searchtext) || (user.FirstName + user.LastName).Contains(searchtext) || c.ClassName.Contains(searchtext)) && c.ClassStatus == 1)
                        select new ReaderContentModel
                        {
                            ClassId = c.ClassId,
                            ClassName = c.ClassName,
                            ClassDesc = c.ClassDescription,
                            ClassType = c.ClassType,
                            PublishedDate = c.PublishedDate,
                            PublisherUserId = c.PublisherUserId,
                            Count = (from s in sqldbcontext.ClassSubscriptions where (s.ClassId == c.ClassId) select s).Count(),
                        };

            return query.ToList();
        }

        public IEnumerable<ReaderContentModel> GetClassbyPublisher(string searchtext, int UserId)
        {

            var query = from c in sqldbcontext.Classes
                        join user in sqldbcontext.Users on c.PublisherUserId equals user.UserId
                        where ((user.FirstName.Contains(searchtext) || user.LastName.Contains(searchtext) || (user.FirstName + user.LastName).Contains(searchtext) || c.ClassName.Contains(searchtext)) && c.ClassStatus == 1 && user.UserId != UserId)
                        select new ReaderContentModel
                        {
                            ClassId = c.ClassId,
                            ClassName = c.ClassName,
                            ClassDesc = c.ClassDescription,
                            ClassType = c.ClassType,
                            PublishedDate = c.PublishedDate,
                            PublisherUserId = c.PublisherUserId,
                            Count = (from s in sqldbcontext.ClassSubscriptions where (s.ClassId == c.ClassId) select s).Count(),
                        };

            return query.ToList();
        }
        //For Advanced Search.
        public IEnumerable<ReaderContentModel> AdvancedSearchClasses(string clstype, string Catid, string pubId, string IsQzt)
        {

            bool IsTypePublisher = false;
            bool IsCatgQuiz = false;
            var cls = new List<Classes>();
            if (!string.IsNullOrEmpty(clstype))
            {
                if (clstype == "Public")
                {
                   
                    cls = (from c in sqldbcontext.Classes where (c.ClassType == true) select c).ToList();
                }
                else if (clstype == "Private")
                {
                    cls = (from c in sqldbcontext.Classes
                           join user in sqldbcontext.Users on c.PublisherUserId equals user.UserId
                           where (c.ClassType == false) 
                           select c).ToList();
                    //cls = (from c in sqldbcontext.Classes where (c.ClassType == false) select c).ToList();
                }
                IsTypePublisher = true;
            }
            else
            {
                cls = (from c in sqldbcontext.Classes select c).ToList();

            }

            var Catg = new List<Categories>();
            if (!string.IsNullOrEmpty(Catid))
            {

                int CatgId = Convert.ToInt32(Catid);
                Catg = (from q in sqldbcontext.Categories where (q.CategoryId == CatgId) select q).ToList();
                IsCatgQuiz = true;
            }
            else
            {
                Catg = (from q in sqldbcontext.Categories select q).ToList();

            }

            var Usrs = new List<Users>();

            if (!string.IsNullOrEmpty(pubId))
            {
                int publisherId = Convert.ToInt32(pubId);
                Usrs = (from q in sqldbcontext.Users where (q.UserId == publisherId) select q).ToList();
                IsTypePublisher = true;
            }
            else
            {
                Usrs = (from q in sqldbcontext.Users select q).ToList();

            }

            var ContentsQuiz = new List<Contents>();
            if (!string.IsNullOrEmpty(IsQzt))
            {
                if (IsQzt == "Yes")
                {
                    ContentsQuiz = (from q in sqldbcontext.Contents where (q.QuizId != null) select q).ToList();
                }
                else if (IsQzt == "No")
                {
                    ContentsQuiz = (from q in sqldbcontext.Contents where (q.QuizId == null) select q).ToList();

                }
                IsCatgQuiz = true;
            }
            else
            {
                ContentsQuiz = (from q in sqldbcontext.Contents select q).ToList();
            }
            var searchPubType = new List<ReaderContentModel>();
            var searchQuizCatg = new List<ReaderContentModel>();
            var search = new List<ReaderContentModel>();
            if (IsCatgQuiz == true)
            {
                searchPubType = (from ClsNames in cls
                                 join PubNames in Usrs on ClsNames.PublisherUserId equals PubNames.UserId

                                 join ClsConts in sqldbcontext.ClassContents on ClsNames.ClassId equals ClsConts.ClassId

                                 join CatgrNames in sqldbcontext.ContentCategories on ClsConts.ContentId equals CatgrNames.ContentId
                                 join ctname in Catg on CatgrNames.CategoryId equals ctname.CategoryId
                                 join Conts in ContentsQuiz on CatgrNames.ContentId equals Conts.ContentId
                                 where ClsNames.ClassStatus == 1
                                 select new ReaderContentModel
                                 {
                                     ClassId = ClsNames.ClassId,
                                     ClassName = ClsNames.ClassName,
                                     ClassDesc = ClsNames.ClassDescription,
                                     ClassType = (bool)ClsNames.ClassType,
                                     PublishedDate = ClsNames.PublishedDate,
                                     PublisherUserId = (int)ClsNames.PublishedUserId,
                                     Count = (from s in sqldbcontext.ClassSubscriptions where (s.ClassId == ClsNames.ClassId) select s).Count()
                                 }).ToList();

                var groupedCustomerList = searchPubType
                .GroupBy(u => u.ClassId)
                .Select(grp => grp.OrderBy(u => u.ClassId).First()).ToList();
                searchPubType = groupedCustomerList;
                search = searchPubType;
            }
            if (IsTypePublisher == true)
            {
                searchQuizCatg = (from ClsNames in cls
                                  join PubNames in Usrs on ClsNames.PublisherUserId equals PubNames.UserId
                                  where ClsNames.ClassStatus == 1
                                  select new ReaderContentModel
                                  {
                                      ClassId = ClsNames.ClassId,
                                      ClassName = ClsNames.ClassName,
                                      ClassDesc = ClsNames.ClassDescription,
                                      ClassType = (bool)ClsNames.ClassType,
                                      PublishedDate = ClsNames.PublishedDate,
                                      PublisherUserId = (int)ClsNames.PublishedUserId,
                                      Count = (from s in sqldbcontext.ClassSubscriptions where (s.ClassId == ClsNames.ClassId) select s).Count()
                                  }).ToList();
                search = searchQuizCatg;

            }
            if (searchQuizCatg.Count() > 0)
            {
                if (!string.IsNullOrEmpty(IsQzt) || (!string.IsNullOrEmpty(Catid)))
                {
                    if (searchPubType.Count > 0)
                    {
                        List<ReaderContentModel> FilterClasses = new List<ReaderContentModel>();

                        foreach (var Item1 in searchPubType)
                        {
                            foreach (var Item2 in searchQuizCatg)
                            {
                                if (Item1.ClassId == Item2.ClassId)
                                {
                                    FilterClasses.Add(Item1);
                                }
                            }
                        }
                        search = FilterClasses;

                    }
                    else
                    { search = new List<ReaderContentModel>(); }
                }

            }
            return search.ToList();
        }
        public IEnumerable<ReaderContentModel> AdvancedSearchClassesbyPublisher(string clstype, string Catid, string pubId, string IsQzt, int userid)
        {

            bool IsTypePublisher = false;
            bool IsCatgQuiz = false;
            var cls = new List<Classes>();
            if (!string.IsNullOrEmpty(clstype))
            {
                if (clstype == "Public")
                {
                    cls= (from c in sqldbcontext.Classes
                               join user in sqldbcontext.Users on c.PublisherUserId equals user.UserId
                               where (c.ClassType==true) && (user.UserId!=userid) select c).ToList();
                   //cls = (from c in sqldbcontext.Classes where (c.ClassType == true) && !=userid select c).ToList();
                }
                else if (clstype == "Private")
                {
                    cls = (from c in sqldbcontext.Classes
                           join user in sqldbcontext.Users on c.PublisherUserId equals user.UserId
                           where (c.ClassType == false) && (user.UserId != userid)
                           select c).ToList();
                   //cls = (from c in sqldbcontext.Classes where (c.ClassType == false) select c).ToList();
                }
                IsTypePublisher = true;
            }
            else
            {
                cls = (from c in sqldbcontext.Classes select c).ToList();

            }

            var Catg = new List<Categories>();
            if (!string.IsNullOrEmpty(Catid))
            {
                int CatgId = Convert.ToInt32(Catid);
                Catg = (from q in sqldbcontext.Categories join user in sqldbcontext.Users on q.PublisherUserId equals user.UserId 
                      where (q.CategoryId == CatgId) && (user.UserId != userid) select q).ToList();
                IsCatgQuiz = true;
                //int CatgId = Convert.ToInt32(Catid);
                //Catg = (from q in sqldbcontext.Categories where (q.CategoryId == CatgId) select q).ToList();
                //IsCatgQuiz = true;
            }
            else
            {
                Catg = (from q in sqldbcontext.Categories select q).ToList();

            }

            var Usrs = new List<Users>();

            if (!string.IsNullOrEmpty(pubId))
            {
                int publisherId = Convert.ToInt32(pubId);
                Usrs = (from q in sqldbcontext.Users where (q.UserId == publisherId) select q).ToList();
                IsTypePublisher = true;
            }
            else
            {
                Usrs = (from q in sqldbcontext.Users select q).ToList();

            }

            var ContentsQuiz = new List<Contents>();
            if (!string.IsNullOrEmpty(IsQzt))
            {
                if (IsQzt == "Yes")
                {
                    ContentsQuiz = (from q in sqldbcontext.Contents
                                    join user in sqldbcontext.Users on q.PublisherUserId equals user.UserId

                                    where (q.QuizId != null) && (user.UserId != userid)
                                    select q).ToList();
                }
                else if (IsQzt == "No")
                {
                    ContentsQuiz = (from q in sqldbcontext.Contents
                                    join user in sqldbcontext.Users on q.PublisherUserId equals user.UserId

                                    where (q.QuizId == null) && (user.UserId != userid)
                                    select q).ToList();

                }
                IsCatgQuiz = true;
            }
            else
            {
                ContentsQuiz = (from q in sqldbcontext.Contents select q).ToList();
            }
            var searchPubType = new List<ReaderContentModel>();
            var searchQuizCatg = new List<ReaderContentModel>();
            var search = new List<ReaderContentModel>();
            if (IsCatgQuiz == true)
            {
                searchPubType = (from ClsNames in cls
                                 join PubNames in Usrs on ClsNames.PublisherUserId equals PubNames.UserId

                                 join ClsConts in sqldbcontext.ClassContents on ClsNames.ClassId equals ClsConts.ClassId

                                 join CatgrNames in sqldbcontext.ContentCategories on ClsConts.ContentId equals CatgrNames.ContentId
                                 join ctname in Catg on CatgrNames.CategoryId equals ctname.CategoryId
                                 join Conts in ContentsQuiz on CatgrNames.ContentId equals Conts.ContentId
                                 where ClsNames.ClassStatus == 1
                                 select new ReaderContentModel
                                 {
                                     ClassId = ClsNames.ClassId,
                                     ClassName = ClsNames.ClassName,
                                     ClassDesc = ClsNames.ClassDescription,
                                     ClassType = (bool)ClsNames.ClassType,
                                     PublishedDate = ClsNames.PublishedDate,
                                     PublisherUserId =(int)ClsNames.PublishedUserId,
                                     Count = (from s in sqldbcontext.ClassSubscriptions where (s.ClassId == ClsNames.ClassId) select s).Count()
                                 }).ToList();

                var groupedCustomerList = searchPubType
                .GroupBy(u => u.ClassId)
                .Select(grp => grp.OrderBy(u => u.ClassId).First()).ToList();
                searchPubType = groupedCustomerList;
                search = searchPubType;
            }
            if (IsTypePublisher == true)
            {
                searchQuizCatg = (from ClsNames in cls
                                  join PubNames in Usrs on ClsNames.PublisherUserId equals PubNames.UserId
                                  where ClsNames.ClassStatus == 1
                                  select new ReaderContentModel
                                  {
                                      ClassId = ClsNames.ClassId,
                                      ClassName = ClsNames.ClassName,
                                      ClassDesc = ClsNames.ClassDescription,
                                      ClassType = (bool)ClsNames.ClassType,
                                      PublishedDate = ClsNames.PublishedDate,
                                      PublisherUserId = (int)ClsNames.PublishedUserId,
                                      Count = (from s in sqldbcontext.ClassSubscriptions where (s.ClassId == ClsNames.ClassId) select s).Count()
                                  }).ToList();
                search = searchQuizCatg;

            }
            if (searchQuizCatg.Count() > 0)
            {
                if (!string.IsNullOrEmpty(IsQzt) || (!string.IsNullOrEmpty(Catid)))
                {
                    if (searchPubType.Count > 0)
                    {
                        List<ReaderContentModel> FilterClasses = new List<ReaderContentModel>();

                        foreach (var Item1 in searchPubType)
                        {
                            foreach (var Item2 in searchQuizCatg)
                            {
                                if (Item1.ClassId == Item2.ClassId)
                                {
                                    FilterClasses.Add(Item1);
                                }
                            }
                        }
                        search = FilterClasses;

                    }
                    else
                    { search = new List<ReaderContentModel>(); }
                }

            }
            return search.ToList();
        }



        #region Exam Repository


        public Quizzes GetQuizDetails(int QuizId)
        {
            var details = (from q in sqldbcontext.Quizzes where (q.QuizId == QuizId) select q).FirstOrDefault();
            return details;
        }


        public IEnumerable<QuestionAnswerModel> QuestionAnswer(int QuizId)
        {
            var query = from c in sqldbcontext.Questions
                        where c.QuestionStatus == 1
                        join m in sqldbcontext.QuestionAnswers on c.QuestionId equals m.QuestionId
                        into members
                        join image in sqldbcontext.QuestionImages on c.QuestionId equals image.QuestionId
                        into imagemembers
                        where c.QuizId == QuizId
                        select new QuestionAnswerModel
                        {
                            QuestionId = c.QuestionId,
                            QuestionDescription = c.QuestionDescription,
                            QuestionTypeId = c.QuestTypeId,
                            Status = c.QuestionStatus,
                            CreatedDate = c.CreatedDate,
                            ModifiedDate = c.ModifiedDate,
                            Answers = (from u in members where c.QuestionId == u.QuestionId select u),
                            QuestionImage = (from Imagemem in imagemembers where c.QuestionId == Imagemem.QuestionId select Imagemem),

                        };

            return query.ToList();
        }

        public QuestionAnswers CorrectAnswerId(int QuestionId)
        {
            QuestionAnswers CorrectAnswers = (from Ca in sqldbcontext.QuestionAnswers where (Ca.QuestionId == QuestionId && Ca.CorrectAnswer == true) select Ca).FirstOrDefault();
            return CorrectAnswers;
        }
        public Questions GetQuestionsbyId(int QuestionId)
        {
            Questions Quest = (from Question in sqldbcontext.Questions where (Question.QuestionId == QuestionId) select Question).FirstOrDefault();
            return Quest;
        }

        public List<QuizReports> QuizReportList(int QuizId, int UserId)
        {
            List<QuizReports> QuizRpts = (from Reports in sqldbcontext.QuizReports where (Reports.QuizId == QuizId && Reports.CandidateUserId == UserId) select Reports).ToList();
            return QuizRpts;
        }


        public void InsertQuizReport(QuizReports QuizRpt)
        {
            sqldbcontext.QuizReports.Add(QuizRpt);
        }


        public void UpdateQuizReport(QuizReports QuizRpt)
        {
            sqldbcontext.Entry(QuizRpt).State = EntityState.Modified;
        }

        public void UpdateQuestionAnswerInfo(QuestionAnswerInfo QuestInfo)
        {
            sqldbcontext.Entry(QuestInfo).State = EntityState.Modified;
        }
        public void InsertQuestionAnswerInfo(QuestionAnswerInfo QuestInfo)
        {
            sqldbcontext.QuestionAnswerInfo.Add(QuestInfo);
        }

        public List<QuestionAnswerInfo> GetQuestionAnsInfo(int QuizId, int UserId)
        {

            QuizReports LastQuizRpt = (from Reports in sqldbcontext.QuizReports where (Reports.QuizId == QuizId && Reports.CandidateUserId == UserId) select Reports).ToList().LastOrDefault();

            var query = from QuestAnsInfo in sqldbcontext.QuestionAnswerInfo
                        where QuestAnsInfo.QuizReportId == LastQuizRpt.QuizReportId
                        select QuestAnsInfo;

            return query.ToList();
        }

        public IEnumerable<QuestionAnswerModel> GetQuestionAns(int QuizId, int UserId)
        {

            QuizReports LastQuizRpt = (from Reports in sqldbcontext.QuizReports where (Reports.QuizId == QuizId && Reports.CandidateUserId == UserId) select Reports).ToList().LastOrDefault();

            var query = from QuestAnsInfo in sqldbcontext.QuestionAnswerInfo
                        where QuestAnsInfo.QuizReportId == LastQuizRpt.QuizReportId
                        join Ans in sqldbcontext.QuestionAnswers on QuestAnsInfo.QuestionId equals Ans.QuestionId
                        into members
                        join QuestList in sqldbcontext.Questions on QuestAnsInfo.QuestionId equals QuestList.QuestionId
                        select new QuestionAnswerModel
                        {
                            QuestionId = QuestAnsInfo.QuestionId.Value,
                            QuestionTypeId = QuestList.QuestTypeId,
                            CorrectAnswerId = (from u in members where QuestAnsInfo.QuestionId.Value == u.QuestionId && u.CorrectAnswer == true select u.AnswerId).FirstOrDefault(),
                            Answers = (from u in members where QuestAnsInfo.QuestionId.Value == u.QuestionId select u),
                            QuestionAnswerInfoId = QuestAnsInfo.QuestionAnswerInfoId
                        };

            return query.ToList();
        }

        public IEnumerable<QuestionAnswerModel> ResultQuestionAnswers(int QuizId, int UserId)
        {

            QuizReports LastQuizRpt = (from Reports in sqldbcontext.QuizReports where (Reports.QuizId == QuizId && Reports.CandidateUserId == UserId) select Reports).ToList().LastOrDefault();

            var query = from QuestAnsInfo in sqldbcontext.QuestionAnswerInfo
                        where QuestAnsInfo.QuizReportId == LastQuizRpt.QuizReportId
                        join Ans in sqldbcontext.QuestionAnswers on QuestAnsInfo.QuestionId equals Ans.QuestionId
                        into members
                        join image in sqldbcontext.QuestionImages on QuestAnsInfo.QuestionId equals image.QuestionId
                        into imagemembers
                        join QuestList in sqldbcontext.Questions on QuestAnsInfo.QuestionId equals QuestList.QuestionId
                        select new QuestionAnswerModel
                        {
                            QuestionId = QuestAnsInfo.QuestionId.Value,
                            QuestionDescription = QuestList.QuestionDescription,
                            QuestionTypeId = QuestList.QuestTypeId,
                            Answers = (from u in members where QuestAnsInfo.QuestionId.Value == u.QuestionId select u),
                            QuestionAnswerInfoId = QuestAnsInfo.QuestionAnswerInfoId,
                            QuestionImage = (from Imagemem in imagemembers where QuestAnsInfo.QuestionId.Value == Imagemem.QuestionId select Imagemem),
                            UserSelectAnswerValue = QuestAnsInfo.Answer,
                            CorrectAnswerId = (from u in members where QuestAnsInfo.QuestionId.Value == u.QuestionId && u.CorrectAnswer == true select u.AnswerId).FirstOrDefault(),
                            CorrectAnswerValue = (from u in members where QuestAnsInfo.QuestionId.Value == u.QuestionId && u.CorrectAnswer == true select u.AnswerValue).FirstOrDefault(),
                            IsCorrectAnswer = QuestAnsInfo.IsCorrectAnswer
                        };

            return query.ToList();
        }
        #endregion


        #region ClassPayment

        public void InsertClassPaymenet(ClassPayment clsPay)
        {
            sqldbcontext.ClassPayment.Add(clsPay);
        }

        public ClassPayment GetClassPaymentDetails(int ClassId, int ReaderUserId)
        {
            return (from ClsPay in sqldbcontext.ClassPayment where ClsPay.ClassId == ClassId && ClsPay.ReaderUserId == ReaderUserId select ClsPay).FirstOrDefault();
        }
        #endregion
    }
}
