﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using IntelSkill.Models;
using IntelSkill.Repository;
using System.Data;

namespace IntelSkill.DAL
{
    public class UsersRepository : IUsersRepository
    {
        public SQLDBContext sqldbcontext;

        public UsersRepository(SQLDBContext _sqldbcontext)
        {
            this.sqldbcontext = _sqldbcontext;
        }

        public UserListModel GetUser(string email)
        {
            return (from c in sqldbcontext.Users

                    where c.Email.Equals(email)

                    select new UserListModel { UserId = c.UserId, CompanyLogo = c.CompanyLogo, CompanyName = c.CompanyName, Email = c.Email, FirstName = c.FirstName, LastName = c.LastName, ParentUserId = 0, RoleId = (from r in sqldbcontext.RefUsers where r.RefUserId == c.UserId select r.RoleId).FirstOrDefault() }).FirstOrDefault();

        }
        public UserListModel GetUserDetail(string email, string password)
        {
            return (from c in sqldbcontext.Users

                    where c.Email.Equals(email) && c.Password.Equals(password)

                    select new UserListModel { UserId = c.UserId, CompanyLogo = c.CompanyLogo, CompanyName = c.CompanyName, Email = c.Email, FirstName = c.FirstName, LastName = c.LastName, ParentUserId =0, RoleId = (from r in sqldbcontext.RefUsers where r.RefUserId == c.UserId select r.RoleId).FirstOrDefault() }).FirstOrDefault();

        }
        public Users GetUser(string email, string Password)
        {
            return (from c in sqldbcontext.Users where c.Email.Equals(email) && c.Password.Equals(Password) select c).SingleOrDefault();
        }


        #region ForgotPasswor
        public Users GetUserbyEmail(string email)
        {
            return (from c in sqldbcontext.Users where c.Email.Equals(email) select c).SingleOrDefault();
        }
        public Users GetUserByID(int id)
        {
            return sqldbcontext.Users.Find(id);
        }

        #endregion

        public IEnumerable<Users> GetUser(int publisherUserId, int roleId)
        {
            return (from c in sqldbcontext.Users
                    join r in sqldbcontext.RefUsers
                    on c.UserId equals r.ChildUserId
                    where r.ParentUserId == publisherUserId && r.RoleId == roleId
                    select c);
        }

        public IEnumerable<UserListModel> GetUserList(int publisherUserId, string roleGroup)
        {
            return (from c in sqldbcontext.Users
                    join r in sqldbcontext.RefUsers
                    on c.UserId equals r.ChildUserId
                    join role in sqldbcontext.Roles on r.RoleId equals role.RoleId
                    where r.ParentUserId == publisherUserId && role.RoleGroup == roleGroup
                    select new UserListModel
                    {
                        UserId = c.UserId
                        ,
                        CompanyLogo = c.CompanyLogo
                        ,
                        CompanyName = c.CompanyName
                        ,
                        Email = c.Email
                        ,
                        FirstName = c.FirstName
                        ,
                        LastName = c.LastName
                        ,
                        RoleId = role.RoleId,
                        RoleName = role.RoleName
                        ,
                        ParentUserId = r.ParentUserId
                        ,
                        ChildUserId = r.ChildUserId


                    }).ToList();
        }

        public IEnumerable<UserListModel> GetUsersSearchResults(int accessAccountId, string searchquery, int roleId)
        {
            string[] splitted_string = searchquery.ToLower().Split(new char[] { ' ' });

            string search_word1 = splitted_string[0].ToString();
            string search_word2 = splitted_string.Count() > 1 ? splitted_string[1].ToString() : "";
            return (from c in sqldbcontext.Users
                    join r in sqldbcontext.RefUsers on c.UserId equals r.ChildUserId
                    join d in sqldbcontext.Roles on r.RoleId equals d.RoleId
                    where r.ParentUserId == accessAccountId && ((c.Email.ToLower().Contains(search_word1) || c.FirstName.ToLower().Contains(search_word1) || c.LastName.ToLower().Contains(search_word1) || d.RoleName.ToLower().Contains(search_word1)) || ((c.Email.ToLower().Contains(search_word2) || c.FirstName.ToLower().Contains(search_word2) || c.LastName.ToLower().Contains(search_word2) || d.RoleName.ToLower().Contains(search_word2)) && splitted_string.Count() > 1)) && ((roleId == 3) ? (r.RoleId == 3) : true) && ((roleId > 3) ? (r.RoleId > roleId) : true)
                    select new UserListModel
                    {
                        UserId = c.UserId
                        ,
                        CompanyLogo = c.CompanyLogo
                        ,
                        CompanyName = c.CompanyName
                        ,
                        Email = c.Email
                        ,
                        FirstName = c.FirstName
                        ,
                        LastName = c.LastName
                        ,
                        RoleId = d.RoleId
                        ,
                        RoleName = d.RoleName
                        ,
                        ParentUserId = r.ParentUserId
                        ,
                        ChildUserId = r.ChildUserId
                    }).ToList();
        }


        public UserListModel GetUserListonEmailAdmin(int publisherUserId, UserModel model)
        {
            return (from c in sqldbcontext.Users
                    join r in sqldbcontext.RefUsers
                    on c.UserId equals r.ChildUserId
                    join role in sqldbcontext.Roles on r.RoleId equals role.RoleId
                    where r.ParentUserId == publisherUserId && c.Email == model.User.Email && role.RoleId != 3
                    select new UserListModel
                    {
                        UserId = c.UserId
                        ,
                        CompanyLogo = c.CompanyLogo
                        ,
                        CompanyName = c.CompanyName
                        ,
                        Email = c.Email
                        ,
                        FirstName = c.FirstName
                        ,
                        LastName = c.LastName
                        ,
                        RoleName = role.RoleName
                        ,
                        ParentUserId = r.ParentUserId
                        ,
                        ChildUserId = r.ChildUserId


                    }).FirstOrDefault();


        }


        public UserListModel GetUserListonEmailStudent(int publisherUserId, UserModel model)
        {
            return (from c in sqldbcontext.Users
                    join r in sqldbcontext.RefUsers
                    on c.UserId equals r.ChildUserId
                    join role in sqldbcontext.Roles on r.RoleId equals role.RoleId
                    where r.ParentUserId == publisherUserId && c.Email == model.User.Email && role.RoleId != 4 && role.RoleId != 5 && role.RoleId != 6
                    select new UserListModel
                    {
                        UserId = c.UserId
                        ,
                        CompanyLogo = c.CompanyLogo
                        ,
                        CompanyName = c.CompanyName
                        ,
                        Email = c.Email
                        ,
                        FirstName = c.FirstName
                        ,
                        LastName = c.LastName
                        ,
                        RoleName = role.RoleName
                        ,
                        ParentUserId = r.ParentUserId
                        ,
                        ChildUserId = r.ChildUserId


                    }).FirstOrDefault();


        }


        public bool SearchEmail(UserModel model)
        {
            var intcount = (from u in sqldbcontext.Users
                            where u.Email == model.User.Email
                            select u).Count();
            if (intcount > 0)
            {
                return true;
            }
            return false;
        }
        public bool InsertRefUser(UserModel model)
        {
            var intcount = (from u in sqldbcontext.Users
                            where u.Email == model.User.Email
                            select u).Count();
            if (intcount == 1)
            {
                var query = (from a in sqldbcontext.Users where a.Email == model.User.Email select a).FirstOrDefault();
                model.RefUser.ChildUserId = query.UserId;
                model.RefUser.CreatedDate = System.DateTime.Now;
                sqldbcontext.RefUsers.Add(model.RefUser);
                sqldbcontext.SaveChanges();
                return true;
            }
            return false;
        }



        public IEnumerable<Roles> GetUserRoles(string roleGroup)
        {
            return from r in sqldbcontext.Roles where r.RoleGroup == roleGroup select r;
        }

        public IEnumerable<Users> GetUsers()
        {
            return sqldbcontext.Users.ToList();
        }

        public UserListModel GetUserbyId(int id)
        {
            return (from c in sqldbcontext.Users
                    join r in sqldbcontext.RefUsers on c.UserId equals r.ChildUserId into group1
                    from g1 in group1.DefaultIfEmpty()
                    join role in sqldbcontext.Roles on g1.RoleId equals role.RoleId into group2
                    from g2 in group2.DefaultIfEmpty()

                    where c.UserId == id

                    select new UserListModel
                    {
                        UserId = c.UserId
                        ,
                        CompanyLogo = c.CompanyLogo
                        ,
                        CompanyName = c.CompanyName
                        ,
                        Email = c.Email
                        ,
                        Password = c.Password
                        ,
                        FirstName = c.FirstName
                        ,
                        LastName = c.LastName
                        ,
                        RoleName = ((g2 == null) ? "" : g2.RoleName)
                        ,
                        RoleId = ((g2 == null) ? 0 : g2.RoleId)
                        ,
                        ParentUserId = ((g1 == null) ? 0 : g1.ParentUserId)
                        ,
                        ChildUserId = ((g1 == null) ? 0 : g1.ChildUserId)

                        ,
                        RefUserId = ((g1 == null) ? 0 : g1.RefUserId)


                    }).FirstOrDefault();
        }



        public UserListModel GetUsersbyIdandcurrentuserid(int id, int publisherUserId, int roleId)
        {
            return (from c in sqldbcontext.Users
                    join r in sqldbcontext.RefUsers on c.UserId equals r.ChildUserId into group1
                    from g1 in group1.DefaultIfEmpty()
                    join role in sqldbcontext.Roles on g1.RoleId equals role.RoleId into group2
                    from g2 in group2.DefaultIfEmpty()

                    where c.UserId == id && g1.ParentUserId == publisherUserId && g1.RoleId == roleId

                    select new UserListModel
                    {
                        UserId = c.UserId
                        ,
                        CompanyLogo = c.CompanyLogo
                        ,
                        CompanyName = c.CompanyName
                        ,
                        Email = c.Email
                        ,
                        Password = c.Password
                        ,
                        FirstName = c.FirstName
                        ,
                        LastName = c.LastName
                        ,
                        RoleName = ((g2 == null) ? "" : g2.RoleName)
                        ,
                        RoleId = ((g2 == null) ? 0 : g2.RoleId)
                        ,
                        ParentUserId = ((g1 == null) ? 0 : g1.ParentUserId)
                        ,
                        ChildUserId = ((g1 == null) ? 0 : g1.ChildUserId)

                        ,
                        RefUserId = ((g1 == null) ? 0 : g1.RefUserId)


                    }).FirstOrDefault();
        }


        public bool InsertUser(Users model)
        {
            var intcount = (from u in sqldbcontext.Users
                            join r in sqldbcontext.RefUsers on u.UserId equals r.ChildUserId
                            where u.Email == model.Email
                            select u).Count();
            if (intcount == 0)
            {
                sqldbcontext.Users.Add(model);

                return true;
            }

            return false;
        }

        public bool InsertUser(UserModel model)
        {
            var intcount = (from u in sqldbcontext.Users
                            where u.Email == model.User.Email
                            select u).Count();
            if (intcount == 0)
            {
                sqldbcontext.Users.Add(model.User);
                sqldbcontext.SaveChanges();
                var query = (from a in sqldbcontext.Users where a.Email == model.User.Email select a).FirstOrDefault();
                model.RefUser.ChildUserId = query.UserId;
                model.RefUser.CreatedDate = System.DateTime.Now;
                sqldbcontext.RefUsers.Add(model.RefUser);
                sqldbcontext.SaveChanges();
                return true;
            }

            return false;
        }


        public bool RegistrationUserInsert(UserModel model)
        {
            var intcount = (from u in sqldbcontext.Users
                            where u.Email == model.User.Email
                            select u).Count();
            if (intcount == 0)
            {
                sqldbcontext.Users.Add(model.User);
                sqldbcontext.SaveChanges();

                return true;
            }

            return false;
        }

        public bool DeleteUser(int selectedUserId, int accessAccountId, int roleId)
        {

            UserModel model = new UserModel();


            var refusers = new RefUsers();

            var RefUserPermissions = new List<RefUsersPermission>();

            if (roleId == 3)
            {
                refusers = (from r in sqldbcontext.RefUsers where r.ChildUserId == selectedUserId && r.ParentUserId == accessAccountId && r.RoleId == roleId select r).SingleOrDefault();
            }
            else
            {
                refusers = (from r in sqldbcontext.RefUsers where r.ChildUserId == selectedUserId && r.ParentUserId == accessAccountId && r.RoleId != 3 select r).SingleOrDefault();
                RefUserPermissions = (from refPermission in sqldbcontext.RefUsersPermission where refPermission.PublisherUserId == accessAccountId && refPermission.UserId == selectedUserId select refPermission).ToList();
            }
            if (RefUserPermissions != null && RefUserPermissions.Count > 0)
            {
                foreach(var RefPermissionId in RefUserPermissions)
                {
                    sqldbcontext.RefUsersPermission.Remove(RefPermissionId);
                }
            }
            if (refusers != null)
            {

                sqldbcontext.RefUsers.Remove(refusers);
                sqldbcontext.SaveChanges();
                return true;
            }

            return false;

        }

        public bool UpdateUser(Users userModel)
        {
            var userDetail = sqldbcontext.Users.Where(x => x.UserId == userModel.UserId).FirstOrDefault();

            if (userDetail != null)
            {

                userDetail.FirstName = userModel.FirstName;
                userDetail.LastName = userModel.LastName;
                userDetail.Password = userModel.Password;

                userDetail.ModifiedDate = System.DateTime.Now;
                userDetail.ModifiedUserId = userModel.ModifiedUserId;

                sqldbcontext.Entry(userDetail).State = EntityState.Modified;
            }
            return true;
        }

        public bool UpdateUser(RefUsers refuser)
        {

            sqldbcontext.Entry(refuser).State = EntityState.Modified;
            sqldbcontext.SaveChanges();
            return true;
        }
        public bool UpdateUser(UserModel model)
        {
            var userDetail = sqldbcontext.Users.Where(x => x.UserId == model.User.UserId).FirstOrDefault();

            if (userDetail != null)
            {

                userDetail.FirstName = model.User.FirstName;
                userDetail.LastName = model.User.LastName;
                userDetail.Password = model.User.Password;

                userDetail.ModifiedDate = System.DateTime.Now;
                userDetail.ModifiedUserId = model.User.ModifiedUserId;
            }
            sqldbcontext.Entry(userDetail).State = EntityState.Modified;

            sqldbcontext.SaveChanges();
            model.RefUser.CreatedDate = System.DateTime.Now;
            sqldbcontext.Entry(model.RefUser).State = EntityState.Modified;
            sqldbcontext.SaveChanges();
            return true;
        }

        public RefUsers GetUserRow(int pid, int cid)
        {
            return (from refusrs in sqldbcontext.RefUsers
                    where refusrs.ParentUserId == pid && refusrs.ChildUserId == cid
                    select refusrs).FirstOrDefault();
        }

        public IEnumerable<Rights> GetRights()
        {
            return sqldbcontext.Rights.ToList();
        }

        public List<PageRightDataModel> GetPageRights(int currentUserId, int publisherUserId)
        {
            var pageRights = (from userrights in sqldbcontext.RefUsersPermission
                              join rights in sqldbcontext.Rights
                              on userrights.RightId equals rights.RightId
                              where userrights.UserId == currentUserId && userrights.PublisherUserId == publisherUserId
                              select new PageRightDataModel { ModuleName = rights.ModuleName, PageName = rights.PageName }).ToList();

            return pageRights;
        }

        public IEnumerable<PageRightDataModel> GetModules()
        {
            var result = (from right in sqldbcontext.Rights group right by right.ModuleName into g select new PageRightDataModel { ModuleName = g.OrderByDescending(x => x.ModuleName).FirstOrDefault().ModuleName }).ToList();

            return result;
        }

        public IEnumerable<UserListModel> GetUsersbyRole(int publisherUserId, int roleId)
        {
            return (from c in sqldbcontext.Users
                    join r in sqldbcontext.RefUsers
                    on c.UserId equals r.ChildUserId
                    join role in sqldbcontext.Roles on r.RoleId equals role.RoleId
                    where r.ParentUserId == publisherUserId && role.RoleId == roleId
                    select new UserListModel
                    {
                        UserId = c.UserId
                        ,
                        CompanyLogo = c.CompanyLogo
                        ,
                        CompanyName = c.CompanyName
                        ,
                        Email = c.Email
                        ,
                        FirstName = c.FirstName
                        ,
                        LastName = c.LastName
                        ,
                        RoleName = role.RoleName
                        ,
                        ParentUserId = r.ParentUserId
                        ,
                        ChildUserId = r.ChildUserId


                    }).ToList();
        }

        public IEnumerable<Rights> GetUserRights(int userId)
        {
            // return from rr in context.RoleRights where rr.RoleId == RoleId select new Rights {rr.RoleId,rr.};

            return (from right in sqldbcontext.Rights
                    join userright in sqldbcontext.RefUsersPermission
                    on right.RightId equals userright.RightId

                    where userright.UserId == userId

                    select right).ToList();

        }

        public IEnumerable<Roles> GetSignUpRoles()
        {
            return from r in sqldbcontext.Roles where r.RoleGroup == "Users" select r;
        }

        public bool IsValidUser(string emailId, int publisherId)
        {

            var intcount = (from r in sqldbcontext.Users where r.Email == emailId select r).Count();
            return intcount == 0;
        }

        public bool SaveUserRights(int selectedUserId, int publisherUserId, List<int> rightid)
        {

            var userPermission = sqldbcontext.RefUsersPermission.Where(x => x.UserId == selectedUserId && x.PublisherUserId == publisherUserId);


            if (userPermission != null)
            {

                foreach (var item in userPermission)
                {
                    sqldbcontext.RefUsersPermission.Remove(item);
                }

                RefUsersPermission rolerights = null;

                
                foreach (int i in rightid)
                {
                    rolerights = new RefUsersPermission();

                    rolerights.UserId = selectedUserId;
                    rolerights.RightId = i;
                    rolerights.PublisherUserId = publisherUserId;

                    sqldbcontext.RefUsersPermission.Add(rolerights);
                }

                return true;
            }

            return false;
        }

        public IEnumerable<Users> GetAllStudents(int publisherUserId)
        {
            return (from c in sqldbcontext.Users
                    join r in sqldbcontext.RefUsers
                    on c.UserId equals r.ChildUserId
                    join role in sqldbcontext.Roles on r.RoleId equals role.RoleId
                    where r.ParentUserId == publisherUserId && role.RoleName.ToLower() == "Student"

                    select c);
        }

        public void Save()
        {
            sqldbcontext.SaveChanges();
        }


        //

        // Current Logged as User (Publisher/ Reader/ etc.,)

        public Users GetMyProfileInfo(int userId)
        {

            return sqldbcontext.Users.Where(model => model.UserId == userId).SingleOrDefault();
        }

        public bool MyProfileUpdate(Users userInfo)
        {
            sqldbcontext.Entry(userInfo).State = EntityState.Modified;

            if (sqldbcontext.SaveChanges() > 0)
                return true;
            else
                return false;

        }

        public Users GetAccountByPassword(int userId, string currentPassword)
        {
            return sqldbcontext.Users.Where(model => model.UserId == userId && model.Password == currentPassword).SingleOrDefault();
        }

        public bool ChangePassword(ChangePasswordModel model)
        {
            var currentUserDetail = sqldbcontext.Users.Where(x => x.UserId == model.CurrentUserId && x.Password == model.OldPassword).SingleOrDefault();

            if (currentUserDetail != null)
            {
                currentUserDetail.Password = model.NewPassword;

                sqldbcontext.Entry(currentUserDetail).State = EntityState.Modified;
                sqldbcontext.SaveChanges();

                return true;
            }

            return false;
        }

        #region Dashboard
        public IEnumerable<StudentExamOverviewModel> GetStudentExamOverView(int loggedId, string FromDate, string ToDate)
        {
            DateTime FromDt = new DateTime();
            DateTime ToDt = new DateTime();

            if (!string.IsNullOrEmpty(FromDate) && !string.IsNullOrEmpty(ToDate))
            {
                FromDt = Convert.ToDateTime(FromDate);
                ToDt = Convert.ToDateTime(ToDate);
            }

            ToDt = ToDt.AddDays(+1);


            var Group1 = (from RefUsr in sqldbcontext.RefUsers
                          where RefUsr.ParentUserId == loggedId && RefUsr.RoleId == 3
                          join User in sqldbcontext.Users on RefUsr.ChildUserId equals User.UserId
                          select User);
            var Group2 = (from RefUsr in sqldbcontext.RefUsers
                          join User in sqldbcontext.Users on RefUsr.ChildUserId equals User.UserId
                          join QR in sqldbcontext.QuizReports on User.UserId equals QR.CandidateUserId
                          join Quiz in sqldbcontext.Quizzes on QR.QuizId equals Quiz.QuizId
                          where RefUsr.ParentUserId == loggedId && RefUsr.RoleId == 3 && Quiz.PublisherUserId==loggedId &&

                        (!string.IsNullOrEmpty(FromDate) ?
                         (QR.QuizStartTime >= FromDt.Date && QR.QuizStartTime <= ToDt.Date) : true)
                          group QR by new { User.UserId } into g

                          select new { Key = g.Key, quizreportgroup = g });



            List<StudentExamOverviewModel> newList = new List<StudentExamOverviewModel>();
            var item_cond = 1;
           
            foreach (var item1 in Group2)
            {
                var MaxAttempt = 0;
                var query = from t in item1.quizreportgroup
                            group t by new { a = t.CandidateUserId, b = t.QuizId } into g
                            select g;
                foreach (var x in query)
                {
                  MaxAttempt +=  x.Max(y => y.AttendedOrder.Value);
                }
                newList.Add(new StudentExamOverviewModel()
                {
                    StudentId = item1.Key.UserId
                    ,
                    StudentName = (from u in Group1 where u.UserId == item1.Key.UserId  select u.FirstName).FirstOrDefault()
                    ,
                    ExamTaken = (from r in item1.quizreportgroup where r.CandidateUserId == item1.Key.UserId group r by new { r.QuizId, r.CandidateUserId } into rd select rd).Count()
                    ,
                    ExamReTaken = (from r in item1.quizreportgroup where r.CandidateUserId == item1.Key.UserId && r.AttendedOrder > item_cond select r.AttendedOrder).Count()
                    ,
                    ExamOutstanding = (from r in item1.quizreportgroup where r.CandidateUserId == item1.Key.UserId && r.PassedScore == 100 select r.PassedScore).Count()
                    ,
                    ExamMaxAttempt = MaxAttempt
                    ,
                    ExamAverageScore = Convert.ToInt32((from r in item1.quizreportgroup where r.CandidateUserId == item1.Key.UserId select r.PassedScore).Average())
                });

            }

            return newList;
        }
        public IEnumerable<StudentExamOverviewModel> GetStudentExamOverviewByQuiz(int logged_userId, int quizid, string FromDate, string ToDate)
        {
            DateTime FromDt = new DateTime();
            DateTime ToDt = new DateTime();

            if (!string.IsNullOrEmpty(FromDate) && !string.IsNullOrEmpty(ToDate))
            {
                FromDt = Convert.ToDateTime(FromDate);
                ToDt = Convert.ToDateTime(ToDate);
            }
            ToDt = ToDt.AddDays(+1);
            var Group1 = (from RefUsr in sqldbcontext.RefUsers
                          where RefUsr.ParentUserId == logged_userId && RefUsr.RoleId == 3
                          join User in sqldbcontext.Users on RefUsr.ChildUserId equals User.UserId
                          select User);


            var Group2 = (from RefUsr in sqldbcontext.RefUsers
                          join User in sqldbcontext.Users on RefUsr.ChildUserId equals User.UserId
                          join QR in sqldbcontext.QuizReports on User.UserId equals QR.CandidateUserId
                          where RefUsr.ParentUserId == logged_userId && RefUsr.RoleId == 3 && QR.QuizId == quizid &&

                        (!string.IsNullOrEmpty(FromDate) ?
                         (QR.QuizStartTime >= FromDt.Date && QR.QuizStartTime <= ToDt.Date) : true)
                          group QR by new { User.UserId } into g

                          select new { Key = g.Key, quizreportgroup = g });


            List<StudentExamOverviewModel> newList = new List<StudentExamOverviewModel>();
            var item_cond = 1;
            foreach (var item1 in Group2)
            {
                newList.Add(new StudentExamOverviewModel()
                {
                    StudentId = item1.Key.UserId
                    ,
                    StudentName = (from u in Group1 where u.UserId == item1.Key.UserId select u.FirstName).FirstOrDefault()
                    ,
                    ExamTaken = (from r in sqldbcontext.QuizReports where r.CandidateUserId == item1.Key.UserId && r.QuizId == quizid group r by new { r.QuizId, r.CandidateUserId } into rd select rd).Count()
                    ,
                    ExamReTaken = (from r in sqldbcontext.QuizReports where r.CandidateUserId == item1.Key.UserId && r.QuizId == quizid && r.AttendedOrder > item_cond select r.AttendedOrder).Count()
                    ,
                    ExamOutstanding = (from r in sqldbcontext.QuizReports where r.CandidateUserId == item1.Key.UserId && r.QuizId == quizid && r.PassedScore == 100 select r.PassedScore).Count()
                    ,
                    ExamMaxAttempt = (from r in sqldbcontext.QuizReports where r.CandidateUserId == item1.Key.UserId && r.QuizId == quizid select r.AttendedOrder.Value).DefaultIfEmpty(0).Max()
                    ,
                    ExamAverageScore = Convert.ToInt32((from r in sqldbcontext.QuizReports where r.CandidateUserId == item1.Key.UserId && r.QuizId == quizid select r.PassedScore).Average())
                });

            }

            return newList;
        }
        public IEnumerable<NewStudent> GetNewStudent(int logged_userId, string FromDate, string ToDate)
        {
            DateTime FromDt = new DateTime();
            DateTime ToDt = new DateTime();

            if (!string.IsNullOrEmpty(FromDate) && !string.IsNullOrEmpty(ToDate))
            {
                FromDt = Convert.ToDateTime(FromDate);
                ToDt = Convert.ToDateTime(ToDate);
            }

            ToDt = ToDt.AddDays(+1);
            return (from RefUsr in sqldbcontext.RefUsers
                    where RefUsr.ParentUserId == logged_userId && RefUsr.RoleId == 3
                    join User in sqldbcontext.Users on RefUsr.ChildUserId equals User.UserId
                    orderby RefUsr.CreatedDate descending
                    where (!string.IsNullOrEmpty(FromDate) ?
                        (RefUsr.CreatedDate >= FromDt.Date && RefUsr.CreatedDate <= ToDt.Date) : true)
                    select new NewStudent { StudentName = (User.FirstName + " " + User.LastName), Email = User.Email }).ToList().Take(10);
        }
        public List<DashboardRoles> DashboardRoles(int PublisherId)
        {
            var MyRoles = from RefUser in sqldbcontext.RefUsers
                          join Usrs in sqldbcontext.Users on RefUser.ParentUserId equals Usrs.UserId
                          join Role in sqldbcontext.Roles on RefUser.RoleId equals Role.RoleId
                          where RefUser.ChildUserId == PublisherId && RefUser.RoleId > 3

                          select new DashboardRoles
                          {
                              UserForm = Usrs
                             ,
                              RoleForm = Role

                          };

            return MyRoles.ToList();
        }
        #endregion

        #region Subscribers

        public List<SubscribersList> SubscribersList(int PublisherId)
        {
            var SubscriptionList = from Cls in sqldbcontext.Classes
                          join ClsSub in sqldbcontext.ClassSubscriptions on Cls.ClassId equals ClsSub.ClassId
                          join Usrs in sqldbcontext.Users on ClsSub.RequestedUserId equals Usrs.UserId
                          where Cls.PublisherUserId == PublisherId && ClsSub.SubscriptionStatus==1   
                          group Usrs by new { Usrs.UserId} into g
                          select new { Key = g.Key, UserForm = g };

            List<SubscribersList> newList = new List<SubscribersList>();

                      foreach(var item in SubscriptionList)
                          {
                              newList.Add(new SubscribersList()
                              {
                                  UserForm = (from Usr in sqldbcontext.Users where Usr.UserId==item.Key.UserId select Usr).FirstOrDefault()
                                ,
                                  SubscribedClassCount = item.UserForm.Count()

                              });
                          }

                      return newList;
        }

        public List<SubcribedClasses> SubcribedClasses(int UserId, int PublisherId)
        {
            var SubscibedClass = from Usr in sqldbcontext.Users
                          join ClsSub in sqldbcontext.ClassSubscriptions on Usr.UserId equals ClsSub.RequestedUserId
                          join Cls in sqldbcontext.Classes on ClsSub.ClassId equals Cls.ClassId                         
                          where Usr.UserId == UserId && ClsSub.SubscriptionStatus==1 && Cls.PublisherUserId==PublisherId

                          select new SubcribedClasses
                          {
                              ClassForm = Cls
                             ,
                              ClassPaymentForm = (from ClsPay in sqldbcontext.ClassPayment where ClsPay.ClassId==Cls.ClassId && ClsPay.ReaderUserId==ClsSub.RequestedUserId select ClsPay).FirstOrDefault()
                              ,
                              ClassSubscribtion = ClsSub

                          };

            return SubscibedClass.ToList();
        }

        public List<SubcribedContentResult> SubcribedContentResult(int ClassId)
        {
            var QuizId = from Cls in sqldbcontext.Classes
                          join ClsCont in sqldbcontext.ClassContents on Cls.ClassId equals ClsCont.ClassId
                          join Cont in sqldbcontext.Contents on ClsCont.ContentId equals Cont.ContentId                         
                           where Cls.ClassId == ClassId && Cont.ContentStatus==1
                            group Cont by new { QuizId = Cont.QuizId} into g                       
                            select new { Key = g.Key.QuizId};

            var SubcribedContentResult = (from Quiz in QuizId
                                          join QuizRpt in sqldbcontext.QuizReports on Quiz.Key equals QuizRpt.QuizId                                        
                                          select QuizRpt).ToList();

            List<SubcribedContentResult> newList = new List<SubcribedContentResult>();

                      foreach(var item in SubcribedContentResult)
                          {
                              newList.Add(new SubcribedContentResult()
                              {
                                  QuizRptForm=item
                                 ,
                                  Content=(from Content in sqldbcontext.Contents where Content.QuizId==item.QuizId select Content).FirstOrDefault()

                              });
                          }
            return newList;
        }
        #endregion

        #region Access Login

        public AccessModel GetRoleAccessDetails(int publisherId, int currentUserId, int roleId)
        {
            AccessModel accessModel = new AccessModel();

            accessModel.CurrentUserId = currentUserId;
            accessModel.PublisherUserId = publisherId;
            accessModel.RoleId = roleId;

            var publisherDetail = sqldbcontext.Users.Where(x => x.UserId == accessModel.PublisherUserId).FirstOrDefault();

            if (publisherDetail != null)
            {
                accessModel.PublisherCompanyLogo = publisherDetail.CompanyLogo;
                accessModel.PublisherUser_FirstName = publisherDetail.FirstName;
                accessModel.PublisherUser_LastName = publisherDetail.LastName;
            }

            var userDetail = sqldbcontext.Users.Where(x => x.UserId == accessModel.CurrentUserId).FirstOrDefault();

            if (userDetail != null)
            {
                accessModel.DisplayName = userDetail.Email;
                accessModel.CurrentUser_FirstName = userDetail.FirstName;
                accessModel.CurrentUser_LastName = userDetail.LastName;
            }

            return accessModel;
        }
        #endregion
    }
}